@@ -5,7 +5,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -244,7 +244,7 @@ static void bot_priv(int idx, char *par)
   }
   if (!to[0])
     return; /* Silently ignore notes to '@bot' */
-  if (!egg_strcasecmp(tobot, botnetnick)) { /* For me! */
+  if (!strcasecmp(tobot, botnetnick)) { /* For me! */
     if (p == from)
       add_note(to, from, par, -2, 0);
     else {
@@ -342,7 +342,11 @@ static void remote_tell_who(int idx, char *nick, int chan)
       i += l;
       strncat(s, c->dname, ssize);
 
-      /* check if we need to trunc, normally only for first chans on the line */
+      /* check if we need to trunc, normally only for first chans on the line.
+       * CHANNELLEN is 80, so we likely won't ever hit this *now*, but if we
+       * ever change that for some reason (twitch? ircv3 stuff?) this is still
+       * a good check to have, 'just in case'
+       */
       if (i > ssize) {
         unsigned int trunc = 4;
         if (c->next) {
@@ -452,7 +456,7 @@ static void bot_who(int idx, char *par)
     from = TBUF;
   }
   to = newsplit(&par);
-  if (!egg_strcasecmp(to, botnetnick))
+  if (!strcasecmp(to, botnetnick))
     to[0] = 0;
 #ifndef NO_OLD_BOTNET
   if (b_numver(idx) < NEAT_BOTNET)
@@ -552,7 +556,7 @@ static void bot_link(int idx, char *par)
   from = newsplit(&par);
   bot = newsplit(&par);
 
-  if (!egg_strcasecmp(bot, botnetnick)) {
+  if (!strcasecmp(bot, botnetnick)) {
     if ((rfrom = strchr(from, ':')))
       rfrom++;
     else
@@ -580,7 +584,7 @@ static void bot_unlink(int idx, char *par)
   from = newsplit(&par);
   bot = newsplit(&par);
   undes = newsplit(&par);
-  if (!egg_strcasecmp(bot, botnetnick)) {
+  if (!strcasecmp(bot, botnetnick)) {
     if ((rfrom = strchr(from, ':')))
       rfrom++;
     else
@@ -665,7 +669,7 @@ static void bot_nlinked(int idx, char *par)
     simple_sprintf(s, "%s %s (%s)", MISC_DISCONNECTED, dcc[idx].nick,
                    MISC_INVALIDBOT);
     dprintf(idx, "error invalid eggnet protocol for 'nlinked'\n");
-  } else if ((in_chain(newbot)) || (!egg_strcasecmp(newbot, botnetnick))) {
+  } else if ((in_chain(newbot)) || (!strcasecmp(newbot, botnetnick))) {
     /* Loop! */
     putlog(LOG_BOTS, "*", "%s %s (mutual: %s)",
            BOT_LOOPDETECT, dcc[idx].nick, newbot);
@@ -719,7 +723,7 @@ static void bot_nlinked(int idx, char *par)
     putlog(LOG_BOTMSG, "*", "(%s) %s %s.", next, NET_LINKEDTO, newbot);
     x = '-';
   }
-  addbot(newbot, dcc[idx].nick, next, x, i);
+  addbot(newbot, dcc[idx].nick, next, x, i, dcc[idx].ssl);
   check_tcl_link(newbot, next);
   u = get_user_by_handle(userlist, newbot);
   if (bot_flags(u) & BOT_REJECT) {
@@ -779,7 +783,7 @@ static void bot_trace(int idx, char *par)
   dest = newsplit(&par);
   simple_sprintf(TBUF, "%s:%s", par, botnetnick);
   botnet_send_traced(idx, from, TBUF);
-  if (egg_strcasecmp(dest, botnetnick) && ((i = nextbot(dest)) > 0))
+  if (strcasecmp(dest, botnetnick) && ((i = nextbot(dest)) > 0))
     botnet_send_trace(i, from, dest, par);
 }
 
@@ -798,7 +802,7 @@ static void bot_traced(int idx, char *par)
     *p = 0;
     p++;
   }
-  if (!egg_strcasecmp(p, botnetnick)) {
+  if (!strcasecmp(p, botnetnick)) {
     time_t t = 0;
     char *p = par, *ss = TBUF;
 
@@ -817,7 +821,7 @@ static void bot_traced(int idx, char *par)
     }
     for (i = 0; i < dcc_total; i++)
       if ((dcc[i].type->flags & DCT_CHAT) &&
-          (!egg_strcasecmp(dcc[i].nick, to)) &&
+          (!strcasecmp(dcc[i].nick, to)) &&
           ((sock == -1) || (sock == dcc[i].sock))) {
         if (t) {
           int j = 0;
@@ -871,7 +875,7 @@ static void bot_reject(int idx, char *par)
     if (i < 0) {
       botnet_send_priv(idx, botnetnick, from, NULL, "%s %s (%s)",
                        BOT_CANTUNLINK, who, BOT_DOESNTEXIST);
-    } else if (!egg_strcasecmp(dcc[i].nick, who)) {
+    } else if (!strcasecmp(dcc[i].nick, who)) {
       char s[1024];
 
       /* I'm the connection to the rejected bot */
@@ -889,7 +893,7 @@ static void bot_reject(int idx, char *par)
     }
   } else {                        /* Rejecting user */
     *destbot++ = 0;
-    if (!egg_strcasecmp(destbot, botnetnick)) {
+    if (!strcasecmp(destbot, botnetnick)) {
       /* Kick someone here! */
       int ok = 0;
 
@@ -909,7 +913,7 @@ static void bot_reject(int idx, char *par)
         ok = 1;
       }
       for (i = 0; (i < dcc_total) && (!ok); i++)
-        if ((!egg_strcasecmp(who, dcc[i].nick)) &&
+        if ((!strcasecmp(who, dcc[i].nick)) &&
             (dcc[i].type->flags & DCT_CHAT)) {
           u = get_user_by_handle(userlist, dcc[i].nick);
           if (u && (u->flags & USER_OWNER)) {
@@ -932,7 +936,7 @@ static void bot_reject(int idx, char *par)
 
 static void bot_thisbot(int idx, char *par)
 {
-  if (egg_strcasecmp(par, dcc[idx].nick)) {
+  if (strcasecmp(par, dcc[idx].nick)) {
     char s[1024];
 
     putlog(LOG_BOTS, "*", NET_WRONGBOT, dcc[idx].nick, par);
@@ -980,7 +984,7 @@ static void bot_zapf(int idx, char *par)
     fake_alert(idx, "direction", from);
     return;
   }
-  if (!egg_strcasecmp(to, botnetnick)) {
+  if (!strcasecmp(to, botnetnick)) {
     /* For me! */
     char *opcode;
 
@@ -1023,7 +1027,7 @@ static void bot_motd(int idx, char *par)
   struct flag_record fr = { FR_BOT, USER_BOT, 0, 0, 0, 0 };
 
   who = newsplit(&par);
-  if (!par[0] || !egg_strcasecmp(par, botnetnick)) {
+  if (!par[0] || !strcasecmp(par, botnetnick)) {
     int irc = 0;
 
     p = strchr(who, ':');
@@ -1087,7 +1091,7 @@ static void bot_filereject(int idx, char *par)
     tobot++;
   else
     tobot = to;                 /* Bot wants a file?! :) */
-  if (egg_strcasecmp(tobot, botnetnick)) {      /* for me! */
+  if (strcasecmp(tobot, botnetnick)) {      /* for me! */
     p = strchr(to, ':');
     if (p != NULL) {
       *p = 0;
@@ -1117,7 +1121,7 @@ static void bot_filereq(int idx, char *tobot)
   if ((path = strchr(tobot, ':'))) {
     *path++ = 0;
 
-    if (!egg_strcasecmp(tobot, botnetnick)) {   /* For me! */
+    if (!strcasecmp(tobot, botnetnick)) {   /* For me! */
       /* Process this */
       module_entry *fs = module_find("filesys", 0, 0);
 
@@ -1151,7 +1155,7 @@ static void bot_filesend(int idx, char *par)
     tobot++;
   } else
     tobot = to;
-  if (!egg_strcasecmp(tobot, botnetnick)) {     /* For me! */
+  if (!strcasecmp(tobot, botnetnick)) {     /* For me! */
     nfn = strrchr(botpath, '/');
     if (nfn == NULL) {
       nfn = strrchr(botpath, ':');
@@ -1461,7 +1465,7 @@ static void bot_versions(int sock, char *par)
 
   if (nextbot(frombot) != sock)
     fake_alert(sock, "versions-direction", frombot);
-  else if (egg_strcasecmp(tobot = newsplit(&par), botnetnick)) {
+  else if (strcasecmp(tobot = newsplit(&par), botnetnick)) {
     if ((sock = nextbot(tobot)) >= 0)
       dprintf(sock, "v %s %s %s\n", frombot, tobot, par);
   } else {
@@ -1488,7 +1492,7 @@ static void bot_starttls(int idx, char *par)
 
   con = newsplit(&par);
   /* check who's going to play the server */
-  if (!egg_strcasecmp(con, "s")) { /* we're server */
+  if (!strcasecmp(con, "s")) { /* we're server */
     putlog(LOG_BOTS, "*", "Got STARTTLS from %s. Replying...", dcc[idx].nick);
     dprintf(idx, "starttls c\n");
     ssl_handshake(dcc[idx].sock, TLS_LISTEN, tls_vfybots, LOG_BOTS,
