@@ -9,7 +9,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -37,7 +37,7 @@ extern Tcl_Interp *interp;
 
 tand_t *tandbot;                   /* Keep track of tandem bots on the botnet */
 party_t *party;                    /* Keep track of people on the botnet */
-static int maxparty = 50;          /* Maximum space for party line members */
+static int party_size = 0;
 int tands = 0;                     /* Number of bots on the botnet */
 int parties = 0;                   /* Number of people on the botnet */
 char botnetnick[HANDLEN + 1] = ""; /* Botnet nickname */
@@ -49,11 +49,9 @@ int tls_vfybots = 0;               /* Verify SSL certificates from bots? */
 int expmem_botnet()
 {
   int size = 0, i;
-  tand_t *bot;
 
-  for (bot = tandbot; bot; bot = bot->next)
-    size += sizeof(tand_t);
-  size += (maxparty * sizeof(party_t));
+  size = tands * sizeof(tand_t);
+  size += party_size * sizeof(party_t);
   for (i = 0; i < parties; i++) {
     if (party[i].away)
       size += strlen(party[i].away) + 1;
@@ -66,9 +64,6 @@ int expmem_botnet()
 void init_bots()
 {
   tandbot = NULL;
-  /* Grab space for 50 bots for now -- expand later as needed */
-  maxparty = 50;
-  party = nmalloc(maxparty * sizeof(*party));
 }
 
 tand_t *findbot(char *who)
@@ -76,19 +71,19 @@ tand_t *findbot(char *who)
   tand_t *ptr;
 
   for (ptr = tandbot; ptr; ptr = ptr->next)
-    if (!egg_strcasecmp(ptr->bot, who))
+    if (!strcasecmp(ptr->bot, who))
       return ptr;
   return NULL;
 }
 
 /* Add a tandem bot to our chain list
  */
-void addbot(char *who, char *from, char *next, char flag, int vernum)
+void addbot(char *who, char *from, char *next, char flag, int vernum, int ssl)
 {
   tand_t **ptr = &tandbot, *ptr2;
 
   while (*ptr) {
-    if (!egg_strcasecmp((*ptr)->bot, who))
+    if (!strcasecmp((*ptr)->bot, who))
       putlog(LOG_BOTS, "*", "!!! Duplicate botnet bot entry!!");
     ptr = &((*ptr)->next);
   }
@@ -98,10 +93,11 @@ void addbot(char *who, char *from, char *next, char flag, int vernum)
   ptr2->share = flag;
   ptr2->ver = vernum;
   ptr2->next = *ptr;
+  ptr2->ssl = ssl;
   *ptr = ptr2;
   /* May be via itself */
   ptr2->via = findbot(from);
-  if (!egg_strcasecmp(next, botnetnick))
+  if (!strcasecmp(next, botnetnick))
     ptr2->uplink = (tand_t *) 1;
   else
     ptr2->uplink = findbot(next);
@@ -129,8 +125,8 @@ int partysock(char *bot, char *nick)
   int i;
 
   for (i = 0; i < parties; i++) {
-    if ((!egg_strcasecmp(party[i].bot, bot)) &&
-        (!egg_strcasecmp(party[i].nick, nick)))
+    if ((!strcasecmp(party[i].bot, bot)) &&
+        (!strcasecmp(party[i].nick, nick)))
       return party[i].sock;
   }
   return 0;
@@ -138,7 +134,7 @@ int partysock(char *bot, char *nick)
 
 /* Set the botnetnick and truncate as necessary */
 void set_botnetnick(const char *newnick) {
-  strncpyz(botnetnick, newnick, sizeof botnetnick);
+  strlcpy(botnetnick, newnick, sizeof botnetnick);
 }
 
 /* New botnet member
@@ -150,7 +146,7 @@ int addparty(char *bot, char *nick, int chan, char flag, int sock,
 
   for (i = 0; i < parties; i++) {
     /* Just changing the channel of someone already on? */
-    if (!egg_strcasecmp(party[i].bot, bot) && (party[i].sock == sock)) {
+    if (!strcasecmp(party[i].bot, bot) && (party[i].sock == sock)) {
       int oldchan = party[i].chan;
 
       party[i].chan = chan;
@@ -169,9 +165,14 @@ int addparty(char *bot, char *nick, int chan, char flag, int sock,
     }
   }
   /* New member */
-  if (parties == maxparty) {
-    maxparty += 50;
-    party = (party_t *) nrealloc((void *) party, maxparty * sizeof(party_t));
+  if (!party_size) {
+      party_size = 1;
+      party = nmalloc(party_size * sizeof(party_t));
+  }
+  else if (parties == party_size) {
+    party_size <<= 1;
+    party = nrealloc(party, party_size * sizeof(party_t));
+    debug1("botnet: party size doubled to %i.", party_size);
   }
   strncpy(party[parties].nick, nick, HANDLEN);
   party[parties].nick[HANDLEN] = 0;
@@ -205,7 +206,7 @@ void partystat(char *bot, int sock, int add, int rem)
   int i;
 
   for (i = 0; i < parties; i++) {
-    if ((!egg_strcasecmp(party[i].bot, bot)) && (party[i].sock == sock)) {
+    if ((!strcasecmp(party[i].bot, bot)) && (party[i].sock == sock)) {
       party[i].status |= add;
       party[i].status &= ~rem;
     }
@@ -219,7 +220,7 @@ void partysetidle(char *bot, int sock, int secs)
   int i;
 
   for (i = 0; i < parties; i++) {
-    if ((!egg_strcasecmp(party[i].bot, bot)) && (party[i].sock == sock)) {
+    if ((!strcasecmp(party[i].bot, bot)) && (party[i].sock == sock)) {
       party[i].timer = (now - (time_t) secs);
     }
   }
@@ -232,7 +233,7 @@ int getparty(char *bot, int sock)
   int i;
 
   for (i = 0; i < parties; i++) {
-    if (!egg_strcasecmp(party[i].bot, bot) && (party[i].sock == sock)) {
+    if (!strcasecmp(party[i].bot, bot) && (party[i].sock == sock)) {
       return i;
     }
   }
@@ -246,8 +247,8 @@ int partyidle(char *bot, char *nick)
   int i, ok = 0;
 
   for (i = 0; i < parties; i++) {
-    if ((!egg_strcasecmp(party[i].bot, bot)) &&
-        (!egg_strcasecmp(party[i].nick, nick))) {
+    if ((!strcasecmp(party[i].bot, bot)) &&
+        (!strcasecmp(party[i].nick, nick))) {
       party[i].timer = now;
       ok = 1;
     }
@@ -263,7 +264,7 @@ int partynick(char *bot, int sock, char *nick)
   int i;
 
   for (i = 0; i < parties; i++) {
-    if (!egg_strcasecmp(party[i].bot, bot) && (party[i].sock == sock)) {
+    if (!strcasecmp(party[i].bot, bot) && (party[i].sock == sock)) {
       strcpy(work, party[i].nick);
       strncpy(party[i].nick, nick, HANDLEN);
       party[i].nick[HANDLEN] = 0;
@@ -281,7 +282,7 @@ void partyaway(char *bot, int sock, char *msg)
   int i;
 
   for (i = 0; i < parties; i++) {
-    if ((!egg_strcasecmp(party[i].bot, bot)) && (party[i].sock == sock)) {
+    if ((!strcasecmp(party[i].bot, bot)) && (party[i].sock == sock)) {
       if (party[i].away)
         nfree(party[i].away);
       if (msg[0]) {
@@ -304,10 +305,10 @@ void rembot(char *whoin)
   /* Need to save the nick for later as it MAY be a pointer to ptr->bot, and we free(ptr) in here. */
   len = strlen(whoin);
   who = nmalloc(len + 1);
-  strncpyz(who, whoin, len + 1);
+  strlcpy(who, whoin, len + 1);
 
   while (*ptr) {
-    if (!egg_strcasecmp((*ptr)->bot, who))
+    if (!strcasecmp((*ptr)->bot, who))
       break;
     ptr = &((*ptr)->next);
   }
@@ -336,7 +337,7 @@ void remparty(char *bot, int sock)
   int i;
 
   for (i = 0; i < parties; i++)
-    if ((!egg_strcasecmp(party[i].bot, bot)) && (party[i].sock == sock)) {
+    if ((!strcasecmp(party[i].bot, bot)) && (party[i].sock == sock)) {
       parties--;
       if (party[i].from)
         nfree(party[i].from);
@@ -363,7 +364,7 @@ void rempartybot(char *bot)
   int i;
 
   for (i = 0; i < parties; i++)
-    if (!egg_strcasecmp(party[i].bot, bot)) {
+    if (!strcasecmp(party[i].bot, bot)) {
       if (party[i].chan >= 0)
         check_tcl_chpt(bot, party[i].nick, party[i].sock, party[i].chan);
       remparty(bot, party[i].sock);
@@ -409,7 +410,7 @@ int nextbot(char *who)
     return -1;
 
   for (j = 0; j < dcc_total; j++)
-    if (bot->via && !egg_strcasecmp(bot->via->bot, dcc[j].nick) &&
+    if (bot->via && !strcasecmp(bot->via->bot, dcc[j].nick) &&
         (dcc[j].type == &DCC_BOT))
       return j;
   return -1;                    /* We're not connected to 'via' */
@@ -434,7 +435,7 @@ char *lastbot(char *who)
 void answer_local_whom(int idx, int chan)
 {
   char format[81];
-  char c, idle[40];
+  char c, idle[64];
   int i, t, nicklen, botnicklen, total = 0;
 
   if (chan == -1)
@@ -584,7 +585,8 @@ void tell_bots(int idx)
 void tell_bottree(int idx, int showver)
 {
   char s[161];
-  tand_t *last[20], *this, *bot, *bot2 = NULL;
+  char c = '-';
+  tand_t *last[20], *this, *bot, *bot2 = NULL, *lastbot = NULL;
   int lev = 0, more = 1, mark[20], ok, cnt, i, imark;
   char work[1024];
   int tothops = 0;
@@ -596,7 +598,7 @@ void tell_bottree(int idx, int showver)
   s[0] = 0;
   i = 0;
 
-  for (bot = tandbot; bot; bot = bot->next)
+  for (bot = tandbot; bot; bot = bot->next) {
     if (!bot->uplink) {
       if (i) {
         s[i++] = ',';
@@ -605,6 +607,9 @@ void tell_bottree(int idx, int showver)
       strcpy(s + i, bot->bot);
       i += strlen(bot->bot);
     }
+  }
+  dprintf(idx, "- Link    = Encrypted link    + Userfile Sharing\n");
+  dprintf(idx, "------------------------------------------------\n");
   if (s[0])
     dprintf(idx, "(%s %s)\n", BOT_NOTRACEINFO, s);
   if (showver)
@@ -626,6 +631,7 @@ void tell_bottree(int idx, int showver)
     for (bot = tandbot; bot; bot = bot->next)
       if (bot->uplink == this)
         cnt++;
+    bot = tandbot;
     if (cnt) {
       imark = 0;
       for (i = 0; i < lev; i++) {
@@ -635,16 +641,30 @@ void tell_bottree(int idx, int showver)
           strcpy(work + imark, "     ");
         imark += 5;
       }
-      if (cnt > 1)
-        strcpy(work + imark, "  |-");
-      else
-        strcpy(work + imark, "  `-");
+      if (cnt > 1) {
+        if (bot->ssl) {
+          strcpy(work + imark, "  |=");
+        } else {
+          strcpy(work + imark, "  |-");
+        }
+      } else {
+        if (bot->ssl) {
+          strcpy(work + imark, "  `=");
+        } else {
+          strcpy(work + imark, "  `-");
+        }
+      }
       s[0] = 0;
       bot = tandbot;
       while (!s[0]) {
         if (bot->uplink == this) {
           if (bot->ver) {
-            i = sprintf(s, "%c%s", bot->share, bot->bot);
+            if ((bot->share=='-') && (bot->ssl)) {
+              c = '=';
+            } else {
+              c = bot->share;
+            }
+            i = sprintf(s, "%c%s", c, bot->bot);
             if (showver)
               sprintf(s + i, " (%d.%d.%d.%d)",
                       bot->ver / 1000000,
@@ -682,7 +702,12 @@ void tell_bottree(int idx, int showver)
               if (cnt == 1) {
                 bot2 = bot;
                 if (bot->ver) {
-                  i = sprintf(s, "%c%s", bot->share, bot->bot);
+                  if ((bot->share=='-') && (bot->ssl)) {
+                    c = '=';
+                  } else {
+                    c = bot->share;
+                  }
+                  i = sprintf(s, "%c%s", c, bot->bot);
                   if (showver)
                     sprintf(s + i, " (%d.%d.%d.%d)",
                             bot->ver / 1000000,
@@ -693,6 +718,7 @@ void tell_bottree(int idx, int showver)
               }
             }
           }
+          lastbot = bot;
         }
         if (cnt) {
           imark = 0;
@@ -705,9 +731,9 @@ void tell_bottree(int idx, int showver)
           }
           more = 1;
           if (cnt > 1)
-            dprintf(idx, "%s  |-%s\n", work, s);
+            dprintf(idx, "%s  |%s%s\n", work, lastbot->ssl ? "=" : "-", s);
           else
-            dprintf(idx, "%s  `-%s\n", work, s);
+            dprintf(idx, "%s  `%s%s\n", work, lastbot->ssl ? "=" : "-", s);
           this = bot2;
           work[0] = 0;
           if (cnt > 1)
@@ -724,6 +750,7 @@ void tell_bottree(int idx, int showver)
             more = 1;
             this = last[lev];
           }
+          dprintf(idx, "------------------------------------------------\n");
         }
       }
     }
@@ -737,7 +764,7 @@ void tell_bottree(int idx, int showver)
  */
 void dump_links(int z)
 {
-  register int i, l;
+  int i, l;
   char x[1024];
   tand_t *bot;
 
@@ -766,7 +793,7 @@ void dump_links(int z)
             (dcc[i].u.chat->channel < GLOBAL_CHANS)) {
 #ifndef NO_OLD_BOTNET
           if (b_numver(z) < NEAT_BOTNET)
-             l =simple_sprintf(x, "join %s %s %d %c%d %s\n",
+            l = simple_sprintf(x, "join %s %s %d %c%d %s\n",
                                botnetnick, dcc[i].nick,
                                dcc[i].u.chat->channel, geticon(i),
                                dcc[i].sock, dcc[i].host);
@@ -834,7 +861,7 @@ int in_chain(char *who)
 {
   if (findbot(who))
     return 1;
-  if (!egg_strcasecmp(who, botnetnick))
+  if (!strcasecmp(who, botnetnick))
     return 1;
   return 0;
 }
@@ -863,7 +890,7 @@ int users_in_subtree(tand_t *bot)
   if (!bot)
     return 0;
   for (i = 0; i < parties; i++)
-    if (!egg_strcasecmp(party[i].bot, bot->bot))
+    if (!strcasecmp(party[i].bot, bot->bot))
       nr++;
   for (b = tandbot; b; b = b->next)
     if (b->bot[0] && (b->uplink == bot))
@@ -876,14 +903,14 @@ int users_in_subtree(tand_t *bot)
 int botunlink(int idx, char *nick, char *reason, char *from)
 {
   char s[20];
-  register int i;
+  int i;
   int bots, users;
   tand_t *bot;
 
   if (nick[0] == '*')
     dprintf(idx, "%s\n", BOT_UNLINKALL);
   for (i = 0; i < dcc_total; i++) {
-    if ((nick[0] == '*') || !egg_strcasecmp(dcc[i].nick, nick)) {
+    if ((nick[0] == '*') || !strcasecmp(dcc[i].nick, nick)) {
       if (dcc[i].type == &DCC_FORK_BOT) {
         if (idx >= 0)
           dprintf(idx, "%s: %s -> %s.\n", BOT_KILLLINKATTEMPT,
@@ -985,13 +1012,13 @@ int botlink(char *linker, int idx, char *nick)
 {
   struct bot_addr *bi;
   struct userrec *u;
-  register int i;
+  int i;
 
   u = get_user_by_handle(userlist, nick);
   if (!u || !(u->flags & USER_BOT)) {
     if (idx >= 0)
       dprintf(idx, "%s %s\n", nick, BOT_BOTUNKNOWN);
-  } else if (!egg_strcasecmp(nick, botnetnick)) {
+  } else if (!strcasecmp(nick, botnetnick)) {
     if (idx >= 0)
       dprintf(idx, "%s\n", BOT_CANTLINKMYSELF);
   } else if (in_chain(nick) && (idx != -3)) {
@@ -1076,28 +1103,29 @@ static void botlink_resolve_failure(int i)
 
 static void botlink_resolve_success(int i)
 {
-  int ret;
   int idx = dcc[i].u.dns->ibuf;
   char *linker = dcc[i].u.dns->cptr;
 
   changeover_dcc(i, &DCC_FORK_BOT, sizeof(struct bot_info));
   dcc[i].timeval = now;
-  strncpyz(dcc[i].u.bot->linker, linker, sizeof dcc[i].u.bot->linker);
+  strlcpy(dcc[i].u.bot->linker, linker, sizeof dcc[i].u.bot->linker);
   strcpy(dcc[i].u.bot->version, "(primitive bot)");
   dcc[i].u.bot->numver = idx;
   dcc[i].u.bot->port = dcc[i].port;     /* Remember where i started */
+#ifdef TLS
+  dcc[i].u.bot->ssl = dcc[i].ssl;       /* Remember where I started */
+#endif
   nfree(linker);
   setsnport(dcc[i].sockname, dcc[i].port);
   dcc[i].sock = getsock(dcc[i].sockname.family, SOCK_STRONGCONN);
-  if (dcc[i].sock < 0)
-    failed_link(i);
-  ret = open_telnet_raw(dcc[i].sock, &dcc[i].sockname);
-  if (ret < 0)
+  if (dcc[i].sock < 0 || open_telnet_raw(dcc[i].sock, &dcc[i].sockname) < 0) {
     failed_link(i);
+  }
 #ifdef TLS
   else if (dcc[i].ssl && ssl_handshake(dcc[i].sock, TLS_CONNECT,
-           tls_vfybots, LOG_BOTS, dcc[i].host, NULL))
+           tls_vfybots, LOG_BOTS, dcc[i].host, NULL)) {
     failed_link(i);
+  }
 #endif
 }
 
@@ -1151,7 +1179,7 @@ static void tandem_relay_resolve_success(int);
 
 /* Relay to another tandembot
  */
-void tandem_relay(int idx, char *nick, register int i)
+void tandem_relay(int idx, char *nick, int i)
 {
   struct userrec *u;
   struct bot_addr *bi;
@@ -1162,7 +1190,7 @@ void tandem_relay(int idx, char *nick, register int i)
     dprintf(idx, "%s %s\n", nick, BOT_BOTUNKNOWN);
     return;
   }
-  if (!egg_strcasecmp(nick, botnetnick)) {
+  if (!strcasecmp(nick, botnetnick)) {
     dprintf(idx, "%s\n", BOT_CANTRELAYMYSELF);
     return;
   }
@@ -1224,7 +1252,7 @@ void tandem_relay(int idx, char *nick, register int i)
 static void tandem_relay_resolve_failure(int idx)
 {
   struct chat_info *ci;
-  register int uidx = -1, i;
+  int uidx = -1, i;
 
   for (i = 0; i < dcc_total; i++)
     if ((dcc[i].type == &DCC_PRE_RELAY) &&
@@ -1286,9 +1314,9 @@ static void tandem_relay_resolve_success(int i)
 
 /* Input from user before connect is ready
  */
-static void pre_relay(int idx, char *buf, register int i)
+static void pre_relay(int idx, char *buf, int i)
 {
-  register int tidx = -1;
+  int tidx = -1;
 
   for (i = 0; i < dcc_total; i++)
     if ((dcc[i].type == &DCC_FORK_RELAY) &&
@@ -1312,7 +1340,7 @@ static void pre_relay(int idx, char *buf, register int i)
     lostdcc(idx);
     return;
   }
-  if (!egg_strcasecmp(buf, "*bye*")) {
+  if (!strcasecmp(buf, "*bye*")) {
     /* Disconnect */
     struct chat_info *ci = dcc[idx].u.relay->chat;
 
@@ -1334,7 +1362,7 @@ static void pre_relay(int idx, char *buf, register int i)
  */
 static void failed_pre_relay(int idx)
 {
-  register int tidx = -1, i;
+  int tidx = -1, i;
 
   for (i = 0; i < dcc_total; i++)
     if ((dcc[i].type == &DCC_FORK_RELAY) &&
@@ -1376,9 +1404,9 @@ static void failed_pre_relay(int idx)
   lostdcc(idx);
 }
 
-static void cont_tandem_relay(int idx, char *buf, register int i)
+static void cont_tandem_relay(int idx, char *buf, int i)
 {
-  register int uidx = -1;
+  int uidx = -1;
   struct relay_info *ri;
 
   for (i = 0; i < dcc_total; i++)
@@ -1417,7 +1445,7 @@ static void cont_tandem_relay(int idx, char *buf, register int i)
 
 static void eof_dcc_relay(int idx)
 {
-  register int j;
+  int j;
   struct chat_info *ci;
 
   for (j = 0; j < dcc_total; j++)
@@ -1454,7 +1482,7 @@ static void eof_dcc_relay(int idx)
 
 static void eof_dcc_relaying(int idx)
 {
-  register int j, x = dcc[idx].u.relay->sock;
+  int j, x = dcc[idx].u.relay->sock;
 
   putlog(LOG_MISC, "*", "%s [%s]%s/%d", BOT_LOSTDCCUSER, dcc[idx].nick,
          dcc[idx].host, dcc[idx].port);
@@ -1468,36 +1496,44 @@ static void eof_dcc_relaying(int idx)
 
 static void dcc_relay(int idx, char *buf, int j)
 {
-  unsigned char *p = (unsigned char *) buf;
-  int mark;
+  unsigned char *src, *dst;
 
   for (j = 0; dcc[j].sock != dcc[idx].u.relay->sock ||
        dcc[j].type != &DCC_RELAYING; j++);
-  /* If redirecting to a non-telnet user, swallow telnet codes and
-   * escape sequences. */
+  /* If redirecting to a non-telnet user, swallow telnet IAC, escape sequences
+   * and CR. */
   if (!(dcc[j].status & STAT_TELNET)) {
-    while (*p != 0) {
-      while (*p != 255 && (*p != '\033' || *(p + 1) != '[') && *p != '\r' && *p)
-        p++;                    /* Search for IAC, escape sequences and CR. */
-      if (*p == 255) {
-        mark = 2;
-        if (!*(p + 1))
-          mark = 1;             /* Bogus */
-        if ((*(p + 1) >= 251) || (*(p + 1) <= 254)) {
-          mark = 3;
-          if (!*(p + 2))
-            mark = 2;           /* Bogus */
+    src = (unsigned char *) buf;
+    dst = (unsigned char *) buf;
+    while (*src) {
+      /* Search for IAC, escape sequences and CR. */
+      if (*src == TLN_IAC) {
+        src++;
+        if ((*src >= TLN_WILL) && (*src <= TLN_DONT)) {
+          src++;
+          if (*src)
+            src++;
         }
-        strcpy((char *) p, (char *) (p + mark));
-      } else if (*p == '\033') {
-        unsigned char *e;
-
-        /* Search for the end of the escape sequence. */
-        for (e = p + 2; *e != 'm' && *e; e++);
-        strcpy((char *) p, (char *) (e + 1));
-      } else if (*p == '\r')
-        memmove(p, p + 1, strlen((char *)p + 1) + 1);
+        else if (*src)
+          src++;
+      } else if (*src == ESC) {
+        src++;
+        if (*src == '[') { /* CSI */
+          src++;
+          /* Search for the end of the escape sequence. */
+          while (*src && *src++ != 'm');
+        }
+      } else if (*src == '\r') /* CR */
+        src++;
+      else {
+        if (src > dst)
+          *dst = *src;
+        src++;
+        dst++;
+      }
     }
+    if (src > dst)
+      *dst = 0;
     if (!buf[0])
       dprintf(-dcc[idx].u.relay->sock, " \n");
     else
@@ -1515,7 +1551,7 @@ static void dcc_relaying(int idx, char *buf, int j)
 {
   struct chat_info *ci;
 
-  if (egg_strcasecmp(buf, "*bye*")) {
+  if (strcasecmp(buf, "*bye*")) {
     dprintf(-dcc[idx].u.relay->sock, "%s\n", buf);
     return;
   }
@@ -1569,7 +1605,7 @@ static void display_pre_relay(int i, char *other)
 
 static int expmem_relay(void *x)
 {
-  register struct relay_info *p = (struct relay_info *) x;
+  struct relay_info *p = (struct relay_info *) x;
   int tot = sizeof(struct relay_info);
 
   if (p->chat)
@@ -1579,7 +1615,7 @@ static int expmem_relay(void *x)
 
 static void kill_relay(int idx, void *x)
 {
-  register struct relay_info *p = (struct relay_info *) x;
+  struct relay_info *p = (struct relay_info *) x;
 
   if (p->chat)
     DCC_CHAT.kill(idx, p->chat);
@@ -1601,7 +1637,7 @@ struct dcc_table DCC_RELAY = {
 
 static void out_relay(int idx, char *buf, void *x)
 {
-  register struct relay_info *p = (struct relay_info *) x;
+  struct relay_info *p = (struct relay_info *) x;
 
   if (p && p->chat)
     DCC_CHAT.output(idx, buf, p->chat);
