@@ -5,7 +5,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -44,6 +44,7 @@
 typedef struct memstruct {
   char nick[NICKLEN];
   char userhost[UHOSTLEN];
+  char account[NICKLEN];
   time_t joined;
   unsigned long flags;
   time_t split; /* in case they were just netsplit */
@@ -72,6 +73,8 @@ typedef struct memstruct {
 #define SENTDEHALFOP 0x08000 /* a mode -h was already sent out for this user */
 #define WASHALFOP    0x10000 /* was a halfop before a split                  */
 #define WHO_SYNCED   0x20000 /* who reply received for this member           */
+#define IRCAWAY      0x40000 /* is marked as away on IRC server              */
+#define IRCBOT       0x80000 /* is marked as a bot, per 005/IRCv3 standard   */
 
 #define chan_hasvoice(x)     (x->flags & CHANVOICE)
 #define chan_hasop(x)        (x->flags & CHANOP)
@@ -90,6 +93,8 @@ typedef struct memstruct {
 #define chan_washalfop(x)    (x->flags & WASHALFOP)
 #define chan_stopcheck(x)    (x->flags & STOPCHECK)
 #define chan_whosynced(x)    (x->flags & WHO_SYNCED)
+#define chan_ircaway(x)      (x->flags & IRCAWAY)
+#define chan_ircbot(x)       (x->flags & IRCBOT)
 
 /* Why duplicate this struct for exempts and invites only under another
  * name? <cybah>
