@@ -9,7 +9,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -35,9 +35,7 @@
 #  endif
 #endif
 
-#ifdef HAVE_UNAME
-#  include <sys/utsname.h>
-#endif
+#include <sys/utsname.h>
 
 #include "modules.h"
 
@@ -61,8 +59,8 @@ unsigned long timer_id = 1;        /* Next timer of any sort will
                                     * have this number             */
 struct chanset_t *chanset = NULL;  /* Channel list                 */
 char admin[121] = "";              /* Admin info                   */
-char origbotname[NICKLEN + 1];
-char botname[NICKLEN + 1];         /* Primary botname              */
+char origbotname[NICKLEN];
+char botname[NICKLEN];             /* Primary botname              */
 char owner[121] = "";              /* Permanent botowner(s)        */
 
 
@@ -142,7 +140,7 @@ struct userrec *check_chanlist(const char *host)
   nick = splitnick(&uhost);
   for (chan = chanset; chan; chan = chan->next)
     for (m = chan->channel.member; m && m->nick[0]; m = m->next)
-      if (!rfc_casecmp(nick, m->nick) && !egg_strcasecmp(uhost, m->userhost))
+      if (!rfc_casecmp(nick, m->nick) && !strcasecmp(uhost, m->userhost))
         return m->user;
   return NULL;
 }
@@ -156,7 +154,7 @@ struct userrec *check_chanlist_hand(const char *hand)
 
   for (chan = chanset; chan; chan = chan->next)
     for (m = chan->channel.member; m && m->nick[0]; m = m->next)
-      if (m->user && !egg_strcasecmp(m->user->handle, hand))
+      if (m->user && !strcasecmp(m->user->handle, hand))
         return m->user;
   return NULL;
 }
@@ -210,7 +208,7 @@ void set_chanlist(const char *host, struct userrec *rec)
   nick = splitnick(&uhost);
   for (chan = chanset; chan; chan = chan->next)
     for (m = chan->channel.member; m && m->nick[0]; m = m->next)
-      if (!rfc_casecmp(nick, m->nick) && !egg_strcasecmp(uhost, m->userhost))
+      if (!rfc_casecmp(nick, m->nick) && !strcasecmp(uhost, m->userhost))
         m->user = rec;
 }
 
@@ -239,11 +237,7 @@ float getcputime()
   stime = ru.ru_stime.tv_sec + (ru.ru_stime.tv_usec / 1000000.00);
   return (utime + stime);
 #else
-#  ifdef HAVE_CLOCK
   return (clock() / (CLOCKS_PER_SEC * 1.00));
-#  else
-  return -1.00;
-#  endif
 #endif
 }
 
@@ -286,30 +280,15 @@ void tell_verbose_uptime(int idx)
  */
 void tell_verbose_status(int idx)
 {
-  char s[256], s1[121], s2[81];
-  char *vers_t, *uni_t;
+  char s[256], s1[121], s2[81], *sysrel;
   int i;
   time_t now2 = now - online_since, hr, min;
-  float cputime;
-#ifdef HAVE_UNAME
-  struct utsname un;
-
-  if (uname(&un) < 0) {
-#endif
-    vers_t = " ";
-    uni_t  = "*unknown*";
-#ifdef HAVE_UNAME
-  } else {
-    vers_t = un.release;
-    uni_t  = un.sysname;
-  }
-#endif
+  double cputime, cache_total;
 
   i = count_users(userlist);
   dprintf(idx, "I am %s, running %s: %d user%s (mem: %uk).\n",
           botnetnick, ver, i, i == 1 ? "" : "s",
           (int) (expected_memory() / 1024));
-
   s[0] = 0;
   if (now2 > 86400) {
     /* days */
@@ -342,16 +321,19 @@ void tell_verbose_status(int idx)
     cputime -= hr * 60;
     sprintf(s2, "CPU: %02d:%05.2f", (int) hr, cputime); /* Actually min/sec */
   }
-  dprintf(idx, "%s %s (%s) - %s - %s: %4.1f%%\n", MISC_ONLINEFOR,
-          s, s1, s2, MISC_CACHEHIT,
-          100.0 * ((float) cache_hit) / ((float) (cache_hit + cache_miss)));
-
+  if (cache_hit + cache_miss) {      /* 2019, still can't divide by zero */
+    cache_total = 100.0 * (cache_hit) / (cache_hit + cache_miss);
+  } else
+    cache_total = 0;
+  dprintf(idx, "%s %s (%s) - %s - %s: %4.1f%%\n", MISC_ONLINEFOR, s, s1, s2,
+          MISC_CACHEHIT, cache_total);
   dprintf(idx, "Configured with: " EGG_AC_ARGS "\n");
   if (admin[0])
     dprintf(idx, "Admin: %s\n", admin);
-
   dprintf(idx, "Config file: %s\n", configfile);
-  dprintf(idx, "OS: %s %s\n", uni_t, vers_t);
+  sysrel = egg_uname();
+  if (*sysrel)
+    dprintf(idx, "OS: %s\n", sysrel);
   dprintf(idx, "Process ID: %d (parent %d)\n", getpid(), getppid());
 
   /* info library */
@@ -368,17 +350,31 @@ void tell_verbose_status(int idx)
   if (tcl_threaded())
     dprintf(idx, "Tcl is threaded.\n");
 #ifdef TLS
-  dprintf(idx, "TLS support is enabled.\n");
-  dprintf(idx, "TLS library: %s\n", SSLeay_version(SSLEAY_VERSION));
+  dprintf(idx, "TLS support is enabled.\n"
+  #if defined HAVE_EVP_PKEY_GET1_EC_KEY && defined HAVE_OPENSSL_MD5
+               "TLS library: %s\n",
+  #elif !defined HAVE_EVP_PKEY_GET1_EC_KEY && defined HAVE_OPENSSL_MD5
+               "TLS library: %s\n             (no elliptic curve support)\n",
+  #elif defined HAVE_EVP_PKEY_GET1_EC_KEY && !defined HAVE_OPENSSL_MD5
+               "TLS library: %s\n             (no MD5 support)\n",
+  #elif !defined HAVE_EVP_PKEY_GET1_EC_KEY && !defined HAVE_OPENSSL_MD5
+               "TLS library: %s\n             (no elliptic curve or MD5 support)\n",
+  #endif
+          SSLeay_version(SSLEAY_VERSION));
 #else
   dprintf(idx, "TLS support is not available.\n");
 #endif
 #ifdef IPV6
-  dprintf(idx, "IPv6 support is enabled.\n");
+  dprintf(idx, "IPv6 support is enabled.\n"
 #else
-  dprintf(idx, "IPv6 support is not available.\n");
+  dprintf(idx, "IPv6 support is not available.\n"
 #endif
-  dprintf(idx, "Socket table: %d/%d\n", threaddata()->MAXSOCKS, max_socks);
+#ifdef EGG_TDNS
+               "Threaded DNS core is enabled.\n"
+#else
+               "Threaded DNS core is disabled.\n"
+#endif
+               "Socket table: %d/%d\n", threaddata()->MAXSOCKS, max_socks);
 }
 
 /* Show all internal state variables
@@ -603,7 +599,7 @@ int remove_timer(tcl_timer_t ** stack, unsigned long id)
 void do_check_timers(tcl_timer_t ** stack)
 {
   tcl_timer_t *mark = *stack, *old = NULL;
-  char x[16];
+  char x[26];
 
   /* New timers could be added by a Tcl script inside a current timer
    * so i'll just clear out the timer list completely, and add any
@@ -616,7 +612,7 @@ void do_check_timers(tcl_timer_t ** stack)
     old = mark;
     mark = mark->next;
     if (!old->mins) {
-      egg_snprintf(x, sizeof x, "timer%lu", old->id);
+      snprintf(x, sizeof x, "timer%lu", old->id);
       do_tcl(x, old->cmd);
       if (old->count == 1) {
         nfree(old->cmd);
@@ -652,14 +648,14 @@ void wipe_timers(Tcl_Interp *irp, tcl_timer_t **stack)
  */
 void list_timers(Tcl_Interp *irp, tcl_timer_t *stack)
 {
-  char mins[10], count[10], id[16], *x;
+  char mins[11], count[11], id[26], *x;
   EGG_CONST char *argv[4];
   tcl_timer_t *mark;
 
   for (mark = stack; mark; mark = mark->next) {
-    egg_snprintf(mins, sizeof mins, "%u", mark->mins);
-    egg_snprintf(id, sizeof id, "timer%lu", mark->id);
-    egg_snprintf(count, sizeof count, "%u", mark->count);
+    snprintf(mins, sizeof mins, "%u", mark->mins);
+    snprintf(id, sizeof id, "timer%lu", mark->id);
+    snprintf(count, sizeof count, "%u", mark->count);
     argv[0] = mins;
     argv[1] = mark->cmd;
     argv[2] = id;
