@@ -8,7 +8,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -57,7 +57,7 @@ int max_dcc = 0;       /* indicates the current dcc limit in the main thread */
 int increase_socks_max()
 {
   struct threaddata *td = threaddata();
-  int osock = td->MAXSOCKS;
+  int osock = td->MAXSOCKS, max_dcc_new;
 
   if (max_socks < 1)
     max_socks = 1;
@@ -79,9 +79,15 @@ int increase_socks_max()
     td->socklist[osock].flags = SOCK_UNUSED;
 
   if (td->mainthread) {
-    max_dcc = td->MAXSOCKS - 10;
-    if (max_dcc < 1)
-      max_dcc = 1;
+    max_dcc_new = td->MAXSOCKS - 10;
+    if (max_dcc_new > max_dcc)
+      max_dcc = max_dcc_new;
+    else if (max_dcc == 0)
+        max_dcc = 1;
+    else {
+        putlog(LOG_MISC, "*", "Maximum dcc limit reached. Consider raising max-socks.");
+        return -1;
+    }
     if (dcc)
       dcc = nrealloc(dcc, sizeof(struct dcc_t) * max_dcc);
     else
@@ -116,9 +122,9 @@ int findidx(int z)
   return -1;
 }
 
-int findanyidx(register int z)
+int findanyidx(int z)
 {
-  register int j;
+  int j;
 
   for (j = 0; j < dcc_total; j++)
     if (dcc[j].sock == z)
@@ -145,7 +151,7 @@ extern void (*qserver) (int, char *, int);
 
 void dprintf EGG_VARARGS_DEF(int, arg1)
 {
-  char buf[1024];
+  char buf[LOGLINEMAX];
   char *format;
   int idx, len;
   va_list va;
@@ -153,7 +159,7 @@ void dprintf EGG_VARARGS_DEF(int, arg1)
   idx = EGG_VARARGS_START(int, arg1, va);
   format = va_arg(va, char *);
 
-  egg_vsnprintf(buf, 1023, format, va);
+  egg_vsnprintf(buf, LOGLINEMAX-1, format, va);
   va_end(va);
   /* We can not use the return value vsnprintf() to determine where
    * to null terminate. The C99 standard specifies that vsnprintf()
@@ -196,10 +202,10 @@ void dprint(int idx, char *buf, int len)
     }
     return;
   } else {
-    if (len > 500) {            /* Truncate to fit */
-      buf[500] = 0;
-      strcat(buf, "\n");
-      len = 501;
+    if (len > LOGLINEMAX-11) {            /* Truncate to fit */
+      buf[LOGLINEMAX-11] = '\n';
+      buf[LOGLINEMAX-10] = 0;
+      len = LOGLINEMAX-10;
     }
     if (dcc[idx].type && ((long) (dcc[idx].type->output) == 1)) {
       char *p = add_cr(buf);
@@ -284,7 +290,7 @@ void dcc_chatter(int idx)
     if (dcc[idx].u.chat->channel == 234567) {
       /* If the chat channel has already been altered it's *highly*
        * probably join/part messages have been broadcast everywhere,
-       * so dont bother sending them
+       * so don't bother sending them
        */
       if (i == -2)
         i = 0;
@@ -364,7 +370,7 @@ void removedcc(int n)
     nfree(dcc[n].u.other);
   dcc_total--;
   if (n < dcc_total)
-    egg_memcpy(&dcc[n], &dcc[dcc_total], sizeof(struct dcc_t));
+    memcpy(&dcc[n], &dcc[dcc_total], sizeof(struct dcc_t));
   else
     egg_bzero(&dcc[n], sizeof(struct dcc_t));   /* drummer */
 }
@@ -498,14 +504,11 @@ void *_get_data_ptr(int size, char *file, int line)
   return p;
 }
 
-/* Make a password, 10-15 random letters and digits
+/* Make a password with max length and random lower case letters and digits
  */
-void makepass(char *s)
+void makepass(char *pass)
 {
-  int i;
-
-  i = 10 + randint(6);
-  make_rand_str(s, i);
+  make_rand_str_from_chars(pass, PASSWORDMAX, CHARSET_PASSWORD);
 }
 
 void flush_lines(int idx, struct chat_info *ci)
