@@ -6,7 +6,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,6 +26,8 @@
 #ifndef _EGG_EGGDROP_H
 #define _EGG_EGGDROP_H
 
+#include <ctype.h>
+
 /*
  * If you're *only* going to link to new version bots (1.3.0 or higher)
  * then you can safely define this.
@@ -50,17 +52,23 @@
 #define CHANNELLEN 80 /* FIXME see issue #3 and issue #38 and rfc1459 <= 200 */
 #define HANDLEN    32 /* valid values 9->NICKMAX                             */
 #define NICKMAX    32 /* valid values HANDLEN->32                            */
+#define USERLEN    10
 
 
 /* Handy string lengths */
+#define HOSTMAX     63            /* DNS RFC 1035, IRC RFC 2812          */
 #define UHOSTMAX    291 + NICKMAX /* 32 (ident) + 3 (\0, !, @) + NICKMAX */
 #define DIRMAX      512           /* paranoia                            */
-#define LOGLINEMAX  767           /* for misc.c/putlog() <cybah>         */
+#define LOGLINEMAX  9000          /* for misc.c/putlog() <cybah>         */
 
 /* Invalid characters */
-#define BADNICKCHARS "-,+*=:!.@#;$%&"
 #define BADHANDCHARS "-,+*=:!.@#;$%&"
 
+/* And now valid characters! */
+#define CHARSET_LOWER_ALPHA     "abcdefghijklmnopqrstuvwxyz"
+#define CHARSET_LOWER_ALPHA_NUM "0123456789abcdefghijklmnopqrstuvwxyz"
+#define CHARSET_PASSWORD        "0123456789?ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz"
+
 
 /* Language stuff */
 #define LANGDIR  "./language" /* language file directory                   */
@@ -72,11 +80,13 @@
 /* The 'configure' script should make this next part automatic, so you
  * shouldn't need to adjust anything below.
  */
-#define NICKLEN      NICKMAX + 1
-#define UHOSTLEN     UHOSTMAX + 1
-#define DIRLEN       DIRMAX + 1
-#define LOGLINELEN   LOGLINEMAX + 1
-#define NOTENAMELEN  ((HANDLEN * 2) + 1)
+#define NICKLEN     NICKMAX + 1
+#define UHOSTLEN    UHOSTMAX + 1
+#define DIRLEN      DIRMAX + 1
+#define LOGLINELEN  LOGLINEMAX + 1
+#define NOTENAMELEN ((HANDLEN * 2) + 1)
+#define PASSWORDMAX 30 /* highest value compatible to older eggdrop */
+#define PASSWORDLEN PASSWORDMAX + 1
 
 
 /* We have to generate compiler errors in a weird way since not all compilers
@@ -85,10 +95,6 @@
 #  include "Error: Your system must have standard ANSI C headers."
 #endif
 
-#ifndef HAVE_VPRINTF
-#  include "Error: You need vsprintf to compile eggdrop."
-#endif
-
 #ifdef HAVE_UNISTD_H
 #  include <unistd.h>
 #endif
@@ -256,19 +262,6 @@
 #define randint(n) (unsigned long) (random() / (RANDOM_MAX + 1.0) * n)
 
 
-#ifndef HAVE_SIGACTION /* old "weird signals" */
-#  define sigaction sigvec
-#  ifndef sa_handler
-#    define sa_handler sv_handler
-#    define sa_mask sv_mask
-#    define sa_flags sv_flags
-#  endif
-#endif
-
-#ifndef HAVE_SIGEMPTYSET
-#  define sigemptyset(x) ((*(int *)(x))=0)
-#endif
-
 #ifdef TLS
 #  include <openssl/ssl.h>
 #endif
@@ -309,11 +302,12 @@
 typedef uint32_t IP;
 
 /* Debug logging macros */
-#define debug0(x)             putlog(LOG_DEBUG,"*",x)
-#define debug1(x,a1)          putlog(LOG_DEBUG,"*",x,a1)
-#define debug2(x,a1,a2)       putlog(LOG_DEBUG,"*",x,a1,a2)
-#define debug3(x,a1,a2,a3)    putlog(LOG_DEBUG,"*",x,a1,a2,a3)
-#define debug4(x,a1,a2,a3,a4) putlog(LOG_DEBUG,"*",x,a1,a2,a3,a4)
+#define debug0(x)                putlog(LOG_DEBUG,"*",x)
+#define debug1(x,a1)             putlog(LOG_DEBUG,"*",x,a1)
+#define debug2(x,a1,a2)          putlog(LOG_DEBUG,"*",x,a1,a2)
+#define debug3(x,a1,a2,a3)       putlog(LOG_DEBUG,"*",x,a1,a2,a3)
+#define debug4(x,a1,a2,a3,a4)    putlog(LOG_DEBUG,"*",x,a1,a2,a3,a4)
+#define debug5(x,a1,a2,a3,a4,a5) putlog(LOG_DEBUG,"*",x,a1,a2,a3,a4,a5)
 
 /* These apparently are unsafe without recasting. */
 #define egg_isdigit(x)  isdigit((int)  (unsigned char) (x))
@@ -322,11 +316,14 @@ typedef uint32_t IP;
 #define egg_isspace(x)  isspace((int)  (unsigned char) (x))
 #define egg_islower(x)  islower((int)  (unsigned char) (x))
 
-/* The following 3 functions are for backward compatibility only */
+/* The following functions are for backward compatibility only */
 #define egg_bzero(dest, len) memset(dest, 0, len)
-#define egg_memcpy(dst, src, len) memcpy(dst, src, len)
-#define egg_memset(dest, c, len) memset(dest, c, len)
-#define my_memcpy(dst, src, len) memcpy(dst, src, len)
+#define egg_memcpy memcpy
+#define egg_memset memset
+#define egg_strcasecmp strcasecmp
+#define egg_strftime strftime
+#define egg_strncasecmp strncasecmp
+#define my_memcpy memcpy
 
 /***********************************************************************/
 
@@ -541,6 +538,7 @@ struct dupwait_info {
 #define STAT_BOTONLY 0x00020    /* telnet on bots-only connect          */
 #define STAT_USRONLY 0x00040    /* telnet on users-only connect         */
 #define STAT_PAGE    0x00080    /* page output to the user              */
+#define STAT_SERV    0x00100    /* this is a server connection          */
 
 /* For stripping out mIRC codes. */
 #define STRIP_COLOR     0x00001    /* remove mIRC color codes            */
@@ -565,9 +563,6 @@ struct dupwait_info {
 #define STAT_LINKING 0x00100    /* the bot is currently going through
                                  * the linking stage                     */
 #define STAT_AGGRESSIVE 0x00200 /* aggressively sharing with this bot    */
-#ifdef TLS
-#define STAT_STARTTLS   0x00400 /* have we sent a starttls request?      */
-#endif
 
 /* Flags for listening sockets */
 #define LSTN_PUBLIC  0x000001   /* No access restrictions               */
@@ -655,10 +650,6 @@ typedef struct {
 #define SOCK_VIRTUAL    0x0200  /* not-connected socket (dont read it!) */
 #define SOCK_BUFFER     0x0400  /* buffer data; don't notify dcc funcs  */
 #define SOCK_TCL        0x0800  /* tcl socket, don't do anything on it  */
-#ifdef TLS
-#  define SOCK_SENTTLS  0x1000  /* Socket that awaits a starttls in the
-                                 * next read                            */
-#endif
 
 /* Flags to sock_has_data
  */
@@ -785,4 +776,28 @@ enum {
 #define TLN_STATUS      5       /* STATUS (RFC 859)      */
 #define TLN_STATUS_C    "\005"
 
+/* From tcl.h */
+#ifndef STRINGIFY
+#  define STRINGIFY(x) STRINGIFY1(x)
+#  define STRINGIFY1(x) #x
+#endif
+
+#ifdef EGG_TDNS
+#define DTN_TYPE_HOSTBYIP 0
+#define DTN_TYPE_IPBYHOST 1
+
+/* linked list instead of array because of multi threading */
+struct dns_thread_node {
+  pthread_mutex_t mutex;
+  int fildes[2];
+  int type;
+  sockname_t addr;
+  char host[256];
+  int ok;
+  struct dns_thread_node *next;
+};
+
+extern struct dns_thread_node *dns_thread_head;
+#endif
+
 #endif /* _EGG_EGGDROP_H */
