@@ -7,7 +7,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -47,11 +47,11 @@
 
 #include "main.h"
 
-#include <fcntl.h>
 #include <errno.h>
-#include <signal.h>
-#include <netdb.h>
+#include <fcntl.h>
+#include <resolv.h>
 #include <setjmp.h>
+#include <signal.h>
 
 #ifdef TIME_WITH_SYS_TIME
 #  include <sys/time.h>
@@ -79,11 +79,15 @@
 #  include <sys/resource.h>             /* setrlimit() */
 #endif
 
+#ifdef HAVE_GETRANDOM
+#  include <sys/random.h>
+#endif
+
 #ifndef _POSIX_SOURCE
 #  define _POSIX_SOURCE 1               /* Solaris needs this */
 #endif
 
-extern char origbotname[], userfile[], botnetnick[];
+extern char origbotname[], botnetnick[]; 
 extern int dcc_total, conmask, cache_hit, cache_miss, max_logs, quick_logs,
            quiet_save;
 extern struct dcc_t *dcc;
@@ -104,20 +108,17 @@ static char **argv;
  * modified versions of this bot.
  */
 
-char egg_version[1024] = EGG_STRINGVER;
+char egg_version[1024];
 int egg_numver = EGG_NUMVER;
-#ifdef EGG_PATCH
-char egg_patch[] = EGG_PATCH;
-#endif
 
 char notify_new[121] = "";      /* Person to send a note to for new users */
 int default_flags = 0;          /* Default user flags                     */
 int default_uflags = 0;         /* Default user-definied flags            */
 
-int backgrd = 1;        /* Run in the background?                        */
-int con_chan = 0;       /* Foreground: constantly display channel stats? */
-int term_z = 0;         /* Foreground: use the terminal as a partyline?  */
-int use_stderr = 1;     /* Send stuff to stderr instead of logfiles?     */
+int backgrd = 1;    /* Run in the background?                        */
+int con_chan = 0;   /* Foreground: constantly display channel stats? */
+int term_z = -1;    /* Foreground: use the terminal as a partyline?  */
+int use_stderr = 1; /* Send stuff to stderr instead of logfiles?     */
 
 char configfile[121] = "eggdrop.conf";  /* Default config file name */
 char pid_file[121];                     /* Name of the pid file     */
@@ -137,9 +138,13 @@ int notify_users_at = 0; /* Minutes past the hour to notify users of notes? */
 char version[81];    /* Version info (long form)  */
 char ver[41];        /* Version info (short form) */
 
-int do_restart = 0;       /* .restart has been called, restart ASAP */
-int resolve_timeout = 15; /* Hostname/address lookup timeout        */
-char quit_msg[1024];      /* Quit message                           */
+volatile sig_atomic_t do_restart = 0; /* .restart has been called, restart ASAP */
+int resolve_timeout = RES_TIMEOUT;    /* Hostname/address lookup timeout        */
+char quit_msg[1024];                  /* Quit message                           */
+
+/* Moved here for n flag warning, put back in do_arg if removed */
+unsigned char cliflags = 0;
+
 
 /* Traffic stats */
 unsigned long otraffic_irc = 0;
@@ -167,7 +172,7 @@ unsigned long itraffic_unknown_today = 0;
 
 #ifdef DEBUG_CONTEXT
 /* Context storage for fatal crashes */
-char cx_file[16][30];
+char cx_file[16][32];
 char cx_note[16][256];
 int cx_line[16];
 int cx_ptr = 0;
@@ -262,18 +267,18 @@ static void write_debug()
     /* Yoicks, if we have this there's serious trouble!
      * All of these are pretty reliable, so we'll try these.
      *
-     * NOTE: dont try and display context-notes in here, it's
+     * NOTE: don't try and display context-notes in here, it's
      *       _not_ safe <cybah>
      */
     x = creat("DEBUG.DEBUG", 0644);
     if (x >= 0) {
       setsock(x, SOCK_NONSOCK);
-      strncpyz(s, ctime(&now), sizeof s);
+      strlcpy(s, ctime(&now), sizeof s);
       dprintf(-x, "Debug (%s) written %s\n", ver, s);
       dprintf(-x, "Please report problem to bugs@eggheads.org\n");
       dprintf(-x, "after a visit to http://www.eggheads.org/bugzilla/\n");
 #ifdef EGG_PATCH
-      dprintf(-x, "Patch level: %s\n", egg_patch);
+      dprintf(-x, "Patch level: %s\n", EGG_PATCH);
 #else
       dprintf(-x, "Patch level: %s\n", "stable");
 #endif
@@ -299,10 +304,10 @@ static void write_debug()
   if (x < 0) {
     putlog(LOG_MISC, "*", "* Failed to write DEBUG");
   } else {
-    strncpyz(s, ctime(&now), sizeof s);
+    strlcpy(s, ctime(&now), sizeof s);
     dprintf(-x, "Debug (%s) written %s\n", ver, s);
 #ifdef EGG_PATCH
-    dprintf(-x, "Patch level: %s\n", egg_patch);
+    dprintf(-x, "Patch level: %s\n", EGG_PATCH);
 #else
     dprintf(-x, "Patch level: %s\n", "stable");
 #endif
@@ -328,13 +333,11 @@ static void write_debug()
 #else
     dprintf(-x, "Compiled without IPv6 support\n");
 #endif
-
 #ifdef TLS
     dprintf(-x, "Compiled with TLS support\n");
 #else
     dprintf(-x, "Compiled without TLS support\n");
 #endif
-
     if (!strcmp(EGG_AC_ARGS, "")) {
       dprintf(-x, "Configure flags: none\n");
     } else {
@@ -453,17 +456,7 @@ static void got_ill(int z)
  */
 void eggContext(const char *file, int line, const char *module)
 {
-  char x[31], *p;
-
-  p = strrchr(file, '/');
-  if (!module) {
-    strncpyz(x, p ? p + 1 : file, sizeof x);
-  } else
-    egg_snprintf(x, 31, "%s:%s", module, p ? p + 1 : file);
-  cx_ptr = ((cx_ptr + 1) & 15);
-  strcpy(cx_file[cx_ptr], x);
-  cx_line[cx_ptr] = line;
-  cx_note[cx_ptr][0] = 0;
+  eggContextNote(file, line, module, NULL);
 }
 
 /* Called from the ContextNote macro.
@@ -471,17 +464,19 @@ void eggContext(const char *file, int line, const char *module)
 void eggContextNote(const char *file, int line, const char *module,
                     const char *note)
 {
-  char x[31], *p;
+  char *p;
 
   p = strrchr(file, '/');
-  if (!module)
-    strncpyz(x, p ? p + 1 : file, sizeof x);
-  else
-    egg_snprintf(x, 31, "%s:%s", module, p ? p + 1 : file);
   cx_ptr = ((cx_ptr + 1) & 15);
-  strcpy(cx_file[cx_ptr], x);
+  if (!module)
+    strlcpy(cx_file[cx_ptr], p ? p + 1 : file, sizeof cx_file[cx_ptr]);
+  else
+    snprintf(cx_file[cx_ptr], sizeof cx_file[cx_ptr], "%s:%s", module, p ? p + 1 : file);
   cx_line[cx_ptr] = line;
-  strncpyz(cx_note[cx_ptr], note, sizeof cx_note[cx_ptr]);
+  if (!note)
+    cx_note[cx_ptr][0] = 0;
+  else
+    strlcpy(cx_note[cx_ptr], note, sizeof cx_note[cx_ptr]);
 }
 #endif /* DEBUG_CONTEXT */
 
@@ -499,10 +494,10 @@ void eggAssert(const char *file, int line, const char *module)
 }
 #endif
 
-void show_ver() {
+static void show_ver() {
   char x[512], *z = x;
 
-  strncpyz(x, egg_version, sizeof x);
+  strlcpy(x, egg_version, sizeof x);
   newsplit(&z);
   newsplit(&z);
   printf("%s\n", version);
@@ -520,6 +515,9 @@ void show_ver() {
 #endif
 #ifdef TLS
   printf("TLS, ");
+#endif
+#ifdef EGG_TDNS
+  printf("Threaded DNS core, ");
 #endif
   printf("handlen=%d\n", HANDLEN);
   bg_send_quit(BG_ABORT);
@@ -529,89 +527,84 @@ void show_ver() {
    meaning other languages can't be loaded yet.
    English (or an error) is the only possible option.
 */
-void show_help() {
+static void show_help() {
   printf("\n%s\n\n", version);
-  printf("Usage: eggdrop [options] [config-file]\n\n"
+  printf("Usage: %s [options] [config-file]\n\n"
          "Options:\n"
-         "-n Don't background; send all log entries to console.\n"
-         "-nc  Don't background; display channel stats every 10 seconds.\n"
-         "-nt  Don't background; use terminal to simulate DCC chat.\n"
-         "-m   Create userfile.\n"
-         "-h   Show this help.\n"
-         "-v   Show version info, then quit.\n\n");
+         "-c  Don't background; display channel stats every 10 seconds.\n"
+         "-t  Don't background; use terminal to simulate DCC chat.\n"
+         "-m  Create userfile.\n"
+         "-h  Show this help and exit.\n"
+         "-v  Show version info and exit.\n\n", argv[0]);
   bg_send_quit(BG_ABORT);
 }
 
 static void do_arg()
 {
   int option = 0;
-/* Bitmask structure to hold cli flags
-   | QUIT| BAD FLAG| h| n| c| t| m| v|
-   |  128|       64|32|16| 8| 4| 2| 1|
-*/
+/* Put this back if removing n flag warning
   unsigned char cliflags = 0;
+*/
+  #define CLI_V        1 << 0
+  #define CLI_M        1 << 1
+  #define CLI_T        1 << 2
+  #define CLI_C        1 << 3
+  #define CLI_N        1 << 4
+  #define CLI_H        1 << 5
+  #define CLI_BAD_FLAG 1 << 6
 
   while ((option = getopt(argc, argv, "hnctmv")) != -1) {
     switch (option) {
       case 'n':
-        cliflags |= 16;
+        cliflags |= CLI_N;
         backgrd = 0;
         break;
       case 'c':
-        cliflags |= 8;
+        cliflags |= CLI_C;
         con_chan = 1;
-        term_z = 0;
+        term_z = -1;
+        backgrd = 0;
         break;
       case 't':
-        cliflags |= 4;
+        cliflags |= CLI_T;
         con_chan = 0;
-        term_z = 1;
+        term_z = 0;
+        backgrd = 0;
         break;
       case 'm':
-        cliflags |= 2;
+        cliflags |= CLI_M;
         make_userfile = 1;
         break;
       case 'v':
-        cliflags |= 129;		//128 + 1
+        cliflags |= CLI_V;
         break;
       case 'h':
-        cliflags |= 160;		//128 + 32
+        cliflags |= CLI_H;
         break;
       default:
-        cliflags |= 192;		//128 + 64
+        cliflags |= CLI_BAD_FLAG;
         break;
     }
   }
-  if ((cliflags & 64) || (cliflags & 32)) {
+  if (cliflags & CLI_H) {
     show_help();
     exit(0);
-  } else if (cliflags & 1) {
+  } else if (cliflags & CLI_BAD_FLAG) {
+    show_help();
+    exit(1);
+  } else if (cliflags & CLI_V) {
     show_ver();
     exit(0);
-  } else if (!(cliflags & 16) && ((cliflags & 8) || (cliflags & 4))) {
-    printf("\n%s\n", version);
-    printf("ERROR: The -n flag is required when using the -c or -t flags. Exiting...\n\n");
-    exit(1);
   } else if (argc > (optind + 1)) {
     printf("\n");
     printf("WARNING: More than one config file value detected\n");
     printf("         Using %s as config file\n", argv[optind]);
   }
   if (argc > optind) {
-    strncpyz(configfile, argv[optind], sizeof configfile);
+    strlcpy(configfile, argv[optind], sizeof configfile);
   }
 }
 
-void backup_userfile(void)
-{
-  char s[125];
-
-  if (quiet_save < 2)
-    putlog(LOG_MISC, "*", USERF_BACKUP);
-  egg_snprintf(s, sizeof s, "%s~bak", userfile);
-  copyfile(userfile, s);
-}
-
 /* Timer info */
 static time_t lastmin;
 static time_t then;
@@ -623,9 +616,10 @@ static struct tm nowtm;
  */
 static void core_secondly()
 {
-  static int cnt = 0;
+  static int cnt = 10; /* Don't wait the first 10 seconds to display */
   int miltime;
   time_t nowmins;
+  int i;
 
   do_check_timers(&utimer);     /* Secondly timers */
   cnt++;
@@ -634,16 +628,20 @@ static void core_secondly()
     check_expired_dcc();
     if (con_chan && !backgrd) {
       dprintf(DP_STDOUT, "\033[2J\033[1;1H");
+      if ((cliflags & CLI_N) && (cliflags & CLI_C)) {
+        printf("NOTE: You don't need to use the -n flag with the\n");
+        printf("       -t or -c flag anymore.\n");
+      }
       tell_verbose_status(DP_STDOUT);
       do_module_report(DP_STDOUT, 0, "server");
       do_module_report(DP_STDOUT, 0, "channels");
       tell_mem_status_dcc(DP_STDOUT);
     }
   }
-  egg_memcpy(&nowtm, localtime(&now), sizeof(struct tm));
   nowmins = time(NULL) / 60;
   if (nowmins > lastmin) {
-    int i = 0;
+    memcpy(&nowtm, localtime(&now), sizeof(struct tm));
+    i = 0;
 
     /* Once a minute */
     ++lastmin;
@@ -672,7 +670,7 @@ static void core_secondly()
         char s[25];
         int j;
 
-        strncpyz(s, ctime(&now), sizeof s);
+        strlcpy(s, ctime(&now), sizeof s);
         if (quiet_save < 3)
           putlog(LOG_ALL, "*", "--- %.11s%s", s, s + 20);
         call_hook(HOOK_BACKUP);
@@ -713,8 +711,7 @@ static void core_secondly()
 
 static void core_minutely()
 {
-  check_tcl_time(&nowtm);
-  check_tcl_cron(&nowtm);
+  check_tcl_time_and_cron(&nowtm);
   do_check_timers(&timer);
   if (quick_logs != 0) {
     flushlogs();
@@ -792,9 +789,9 @@ int init_language(int);
 int ssl_init();
 #endif
 
-static inline void garbage_collect(void)
+static void garbage_collect(void)
 {
-  static u_8bit_t run_cnt = 0;
+  static uint8_t run_cnt = 0;
 
   if (run_cnt == 3)
     garbage_collect_tclhash();
@@ -806,21 +803,12 @@ int mainloop(int toplevel)
 {
   static int socket_cleanup = 0;
   int xx, i, eggbusy = 1, tclbusy = 0;
-  char buf[520];
+  char buf[8702];
 
-  /* Lets move some of this here, reducing the numer of actual
+  /* Lets move some of this here, reducing the number of actual
    * calls to periodic_timers
    */
   now = time(NULL);
-  /*
-   * FIXME: Get rid of this, it's ugly and wastes lots of cpu.
-   *
-   * pre-1.3.0 Eggdrop had random() in the once a second block below.
-   *
-   * This attempts to keep random() more random by constantly
-   * calling random() and updating the state information.
-   */
-  random();                /* Woop, lets really jumble things */
 
   /* If we want to restart, we have to unwind to the toplevel.
    * Tcl will Panic if we kill the interp with Tcl_Eval in progress.
@@ -878,9 +866,8 @@ int mainloop(int toplevel)
           }
           dcc[idx].type->activity(idx, buf, i);
         } else
-          putlog(LOG_MISC, "*",
-                 "!!! untrapped dcc activity: type %s, sock %d",
-                 dcc[idx].type->name, dcc[idx].sock);
+          putlog(LOG_MISC, "*", "ERROR: untrapped dcc activity: type %s, sock %d",
+                 dcc[idx].type ? dcc[idx].type->name : "UNKNOWN", dcc[idx].sock);
         break;
       }
   } else if (xx == -1) {        /* EOF from someone */
@@ -955,7 +942,7 @@ int mainloop(int toplevel)
             d = d->next;
           }
           if (ok) {
-            strncpyz(name, p->name, sizeof name);
+            strlcpy(name, p->name, sizeof name);
             if (module_unload(name, botnetnick) == NULL) {
               f = 1;
               break;
@@ -965,12 +952,13 @@ int mainloop(int toplevel)
       }
 
       /* Make sure we don't have any modules left hanging around other than
-       * "eggdrop" and the two that are supposed to be.
+       * "eggdrop" and the 3 that are supposed to be.
        */
       for (f = 0, p = module_list; p; p = p->next) {
         if (strcmp(p->name, "eggdrop") && strcmp(p->name, "encryption") &&
-            strcmp(p->name, "uptime")) {
+            strcmp(p->name, "encryption2") && strcmp(p->name, "uptime")) {
           f++;
+          debug1("stagnant module %s", p->name);
         }
       }
       if (f != 0) {
@@ -1016,6 +1004,27 @@ int mainloop(int toplevel)
   return (eggbusy || tclbusy);
 }
 
+static void init_random(void) {
+  unsigned int seed;
+#ifdef HAVE_GETRANDOM
+  if (getrandom(&seed, sizeof(seed), 0) != sizeof(seed)) {
+    if (errno != ENOSYS) {
+      fatal("ERROR: getrandom()\n", 0);
+    } else {
+      /* getrandom() is available in header but syscall is not!
+       * This can happen with glibc>=2.25 and linux<3.17
+       */
+#endif
+      struct timeval tp;
+      gettimeofday(&tp, NULL);
+      seed = (((int64_t) tp.tv_sec * tp.tv_usec)) ^ getpid();
+#ifdef HAVE_GETRANDOM
+    }
+  }
+#endif
+  srandom(seed);
+}
+
 int main(int arg_c, char **arg_v)
 {
   int i, xx;
@@ -1054,15 +1063,18 @@ int main(int arg_c, char **arg_v)
 
   /* Version info! */
 #ifdef EGG_PATCH
-  egg_snprintf(&egg_version[strlen(egg_version)], sizeof egg_version, 
-               "+%s", egg_patch);
-#endif
-  egg_snprintf(ver, sizeof ver, "eggdrop v%s", egg_version);
+  egg_snprintf(egg_version, sizeof egg_version, "%s+%s %u", EGG_STRINGVER, EGG_PATCH, egg_numver);
+  egg_snprintf(ver, sizeof ver, "eggdrop v%s+%s", EGG_STRINGVER, EGG_PATCH);
   egg_snprintf(version, sizeof version,
-               "Eggdrop v%s (C) 1997 Robey Pointer (C) 2010-2018 Eggheads",
-               egg_version);
-  /* Now add on the patchlevel (for Tcl) */
-  sprintf(&egg_version[strlen(egg_version)], " %u", egg_numver);
+               "Eggdrop v%s+%s (C) 1997 Robey Pointer (C) 2010-2022 Eggheads",
+                EGG_STRINGVER, EGG_PATCH);
+#else
+  egg_snprintf(egg_version, sizeof egg_version, "%s %u", EGG_STRINGVER, egg_numver);
+  egg_snprintf(ver, sizeof ver, "eggdrop v%s", EGG_STRINGVER);
+  egg_snprintf(version, sizeof version,
+               "Eggdrop v%s (C) 1997 Robey Pointer (C) 2010-2022 Eggheads",
+                EGG_STRINGVER);
+#endif
 
 /* For OSF/1 */
 #ifdef STOP_UAC
@@ -1104,9 +1116,8 @@ int main(int arg_c, char **arg_v)
   /* Initialize variables and stuff */
   now = time(NULL);
   chanset = NULL;
-  egg_memcpy(&nowtm, localtime(&now), sizeof(struct tm));
   lastmin = now / 60;
-  srandom((unsigned int) (now % (getpid() + getppid())));
+  init_random();
   init_mem();
   if (argc > 1)
     do_arg();
@@ -1137,11 +1148,16 @@ int main(int arg_c, char **arg_v)
 #ifdef STATIC
   link_statics();
 #endif
-  strncpyz(s, ctime(&now), sizeof s);
-  memmove(&s[11], &s[20], strlen(&s[20])+1);
+#ifdef EGG_TDNS
+  /* initialize dns_thread_head before chanprog() */
+  dns_thread_head = nmalloc(sizeof(struct dns_thread_node));
+  dns_thread_head->next = NULL;
+#endif
+  strlcpy(s, ctime(&now), sizeof s);
+  memmove(&s[11], &s[20], strlen(&s[20]) + 1);
   putlog(LOG_ALL, "*", "--- Loading %s (%s)", ver, s);
   chanprog();
-  if (!encrypt_pass) {
+  if (!encrypt_pass2 && !encrypt_pass) {
     printf("%s", MOD_NOCRYPT);
     bg_send_quit(BG_ABORT);
     exit(1);
@@ -1151,6 +1167,11 @@ int main(int arg_c, char **arg_v)
     i++;
   putlog(LOG_MISC, "*", "=== %s: %d channels, %d users.",
          botnetnick, i, count_users(userlist));
+  if ((cliflags & CLI_N) && (cliflags & CLI_T)) {
+    printf("\n");
+    printf("NOTE: The -n flag is no longer used, it is as effective as Han\n");
+    printf("      without Chewie\n");
+  }
 #ifdef TLS
   ssl_init();
 #endif
@@ -1206,9 +1227,7 @@ int main(int arg_c, char **arg_v)
   use_stderr = 0;               /* Stop writing to stderr now */
   if (backgrd) {
     /* Ok, try to disassociate from controlling terminal (finger cross) */
-#ifdef HAVE_SETPGID
     setpgid(0, 0);
-#endif
     /* Tcl wants the stdin, stdout and stderr file handles kept open. */
     if (freopen("/dev/null", "r", stdin) == NULL) {
       putlog(LOG_MISC, "*", "Error renaming stdin file handle: %s", strerror(errno));
@@ -1225,12 +1244,12 @@ int main(int arg_c, char **arg_v)
   }
 
   /* Terminal emulating dcc chat */
-  if (!backgrd && term_z) {
+  if (!backgrd && term_z >= 0) {
     /* reuse term_z as glob var to pass it's index in the dcc table around */
     term_z = new_dcc(&DCC_CHAT, sizeof(struct chat_info));
 
-    /* new_dcc returns -1 on error, and 0 should always be taken by the listening socket */
-    if (term_z < 1)
+    /* new_dcc returns -1 on error */
+    if (term_z < 0)
       fatal("ERROR: Failed to initialize foreground chat.", 0);
 
     getvhost(&dcc[term_z].sockname, AF_INET);
@@ -1248,7 +1267,13 @@ int main(int arg_c, char **arg_v)
     dcc_chatter(term_z);
   }
 
-  then = now;
+  /* -1 to make mainloop() call
+   * call_hook(HOOK_SECONDLY)->server_secondly()->connect_server() before first
+   * sockgets()->sockread()->select() to avoid an unnecessary select timeout of
+   * up to 1 sec while starting up
+   */
+  then = now - 1;
+
   online_since = now;
   autolink_cycle(NULL);         /* Hurry and connect to tandem bots */
   add_help_reference("cmds1.help");
