@@ -4,7 +4,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -109,8 +109,6 @@ extern struct dcc_table DCC_CHAT, DCC_BOT, DCC_LOST, DCC_SCRIPT, DCC_BOT_NEW,
                         DCC_IDENTWAIT, DCC_DNSWAIT;
 #endif
 
-#define iptolong(a) (0xffffffff & (long) (htonl((unsigned long) a)))
-
 #ifdef IPV6
 # define setsnport(s, p) do {                                           \
   if ((s).family == AF_INET6)                                           \
@@ -144,4 +142,6 @@ extern struct dcc_table DCC_CHAT, DCC_BOT, DCC_LOST, DCC_SCRIPT, DCC_BOT_NEW,
 
 #define EGG_AC_ARGS EGG_MACRO_STR(EGG_AC_ARGS_RAW)
 
+#define ARRAY_SIZE(x) (sizeof (x) / sizeof *(x))
+
 #endif /* _EGG_MAIN_H */
