@@ -2,12 +2,10 @@
  * mem.c -- handles:
  *   memory allocation and deallocation
  *   keeping track of what memory is being used by whom
- *
- * $Id: mem.c,v 1.32 2011/08/15 18:16:29 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,7 +22,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#define MEMTBLSIZE 25000        /* yikes! */
+#define MEMTBLSIZE_START (1 << 12) /* 4096    */
+#define MEMTBLSIZE_MAX   (1 << 20) /* 1048576 */
 #define COMPILING_MEM
 
 #include "main.h"
@@ -47,7 +46,9 @@ struct {
   int size;
   short line;
   char file[20];
-} memtbl[MEMTBLSIZE];
+} *memtbl;
+
+static int memtbl_size = MEMTBLSIZE_START;
 #endif
 
 /* Prototypes */
@@ -66,16 +67,25 @@ int expmem_modules();
 int expmem_language();
 int expmem_tcldcc();
 int expmem_dns();
-
+#ifdef TLS
+int expmem_tls();
+#endif
 
 /* Initialize the memory structure
  */
 void init_mem()
 {
 #ifdef DEBUG_MEM
+  size_t size;
   int i;
 
-  for (i = 0; i < MEMTBLSIZE; i++)
+  size =  memtbl_size * sizeof *memtbl;
+  if (!(memtbl = malloc(size))) {
+    putlog(LOG_MISC, "*", "*** FAILED MALLOC mem.c (memtbl) (%i): %s", size,
+           strerror(errno));
+    fatal("Memory allocation failed", 0);
+  } 
+  for (i = 0; i < memtbl_size; i++)
     memtbl[i].ptr = NULL;
 #endif
 }
@@ -87,9 +97,9 @@ void tell_mem_status(char *nick)
 #ifdef DEBUG_MEM
   float per;
 
-  per = ((lastused * 1.0) / (MEMTBLSIZE * 1.0)) * 100.0;
+  per = ((lastused * 1.0) / (memtbl_size * 1.0)) * 100.0;
   dprintf(DP_HELP, "NOTICE %s :Memory table usage: %d/%d (%.1f%% full)\n",
-          nick, lastused, MEMTBLSIZE, per);
+          nick, lastused, memtbl_size, per);
 #endif
   dprintf(DP_HELP, "NOTICE %s :Think I'm using about %dk.\n", nick,
           (int) (expected_memory() / 1024));
@@ -102,22 +112,26 @@ void tell_mem_status_dcc(int idx)
   float per;
 
   exp = expected_memory();      /* in main.c ? */
-  per = ((lastused * 1.0) / (MEMTBLSIZE * 1.0)) * 100.0;
-  dprintf(idx, "Memory table: %d/%d (%.1f%% full)\n", lastused, MEMTBLSIZE,
+  per = ((lastused * 1.0) / (memtbl_size * 1.0)) * 100.0;
+  dprintf(idx, "Memory table: %d/%d (%.1f%% full)\n", lastused, memtbl_size,
           per);
   per = ((exp * 1.0) / (memused * 1.0)) * 100.0;
   if (per != 100.0)
     dprintf(idx, "Memory fault: only accounting for %d/%ld (%.1f%%)\n",
             exp, memused, per);
   dprintf(idx, "Memory table itself occupies an additional %dk static\n",
-          (int) (sizeof(memtbl) / 1024));
+          (int) memtbl_size * sizeof *memtbl / 1024);
 #endif
 }
 
 void debug_mem_to_dcc(int idx)
 {
 #ifdef DEBUG_MEM
-#  define MAX_MEM 13
+#  ifdef TLS
+#    define MAX_MEM 14
+#  else
+#    define MAX_MEM 13
+#  endif
   unsigned long exp[MAX_MEM], use[MAX_MEM], l;
   int i, j;
   char fn[20], sofar[81];
@@ -137,6 +151,9 @@ void debug_mem_to_dcc(int idx)
   exp[10] = expmem_modules(1);
   exp[11] = expmem_tcldcc();
   exp[12] = expmem_dns();
+#ifdef TLS
+  exp[13] = expmem_tls();
+#endif
 
   for (me = module_list; me; me = me->next)
     me->mem_work = 0;
@@ -176,6 +193,10 @@ void debug_mem_to_dcc(int idx)
       use[11] += l;
     else if (!strcmp(fn, "dns.c"))
       use[12] += l;
+#ifdef TLS
+    else if (!strcmp(fn, "tls.c"))
+      use[13] += l;
+#endif
     else if (p) {
       for (me = module_list; me; me = me->next)
         if (!strcmp(fn, me->name))
@@ -225,6 +246,11 @@ void debug_mem_to_dcc(int idx)
     case 12:
       strcpy(fn, "dns.c");
       break;
+#ifdef TLS
+    case 13:
+      strcpy(fn, "tls.c");
+      break;
+#endif
     }
 
     if (use[i] == exp[i])
@@ -237,7 +263,7 @@ void debug_mem_to_dcc(int idx)
       for (j = 0; j < lastused; j++) {
         if ((p = strchr(memtbl[j].file, ':')))
           *p = 0;
-        if (!egg_strcasecmp(memtbl[j].file, fn)) {
+        if (!strcasecmp(memtbl[j].file, fn)) {
           if (p)
             sprintf(&sofar[strlen(sofar)], "%-10s/%-4d:(%04d) ",
                     p + 1, memtbl[j].line, memtbl[j].size);
@@ -278,7 +304,7 @@ void debug_mem_to_dcc(int idx)
         strcpy(fn, memtbl[j].file);
         if ((p = strchr(fn, ':')) != NULL) {
           *p = 0;
-          if (!egg_strcasecmp(fn, me->name)) {
+          if (!strcasecmp(fn, me->name)) {
             sprintf(&sofar[strlen(sofar)], "%-10s/%-4d:(%04X) ", p + 1,
                     memtbl[j].line, memtbl[j].size);
             if (strlen(sofar) > 60) {
@@ -304,44 +330,16 @@ void debug_mem_to_dcc(int idx)
   tell_netdebug(idx);
 }
 
-#ifdef DEBUG_MEM
-static inline void addtomemtbl(void *x, int size, const char *file, int line)
-{
-  int i = 0;
-  char *p;
-
-  if (lastused == MEMTBLSIZE) {
-    putlog(LOG_MISC, "*", "*** MEMORY TABLE FULL: %s (%d)", file, line);
-    fatal("Memory table full", 0);
-  }
-  i = lastused;
-  memtbl[i].ptr = x;
-  memtbl[i].line = line;
-  memtbl[i].size = size;
-  p = strrchr(file, '/');
-  strncpy(memtbl[i].file, p ? p + 1 : file, 19);
-  memtbl[i].file[19] = 0;
-  memused += size;
-  lastused++;
-}
-#endif
-
-char *n_strdup(const char *s, const char *file, int line)
-{
-  char *x;
-
-  x = egg_strdup(s);
-/* compat strdup uses nmalloc itself */
-#if defined(DEBUG_MEM) && defined(HAVE_STRDUP)
-  addtomemtbl(x, strlen(s)+1, file, line);
-#endif
-  return x;
-}
-
 void *n_malloc(int size, const char *file, int line)
 {
   void *x;
 
+#ifdef DEBUG_MEM
+  size_t size2;
+  int i = 0;
+  char *p;
+#endif
+
   x = (void *) malloc(size);
   if (x == NULL) {
     putlog(LOG_MISC, "*", "*** FAILED MALLOC %s (%d) (%d): %s", file, line,
@@ -349,7 +347,30 @@ void *n_malloc(int size, const char *file, int line)
     fatal("Memory allocation failed", 0);
   }
 #ifdef DEBUG_MEM
-  addtomemtbl(x, size, file, line);
+  if (lastused == memtbl_size) {
+    memtbl_size <<= 1;
+    if (memtbl_size > MEMTBLSIZE_MAX) {
+      putlog(LOG_MISC, "*", "*** MEMORY TABLE FULL: %s (%d)", file, line);
+      fatal("Memory table full", 0);
+    }
+    size2 = memtbl_size * sizeof *memtbl;
+    if (!(memtbl = realloc(memtbl, size2))) {
+      putlog(LOG_MISC, "*", "*** FAILED REALLOC mem.c (memtbl) (%i): %s", size2,
+             strerror(errno));
+      fatal("Memory allocation failed", 0);
+    } 
+    for (i = lastused; i < memtbl_size; i++)
+      memtbl[i].ptr = NULL;
+    debug1("mem: memtbl size doubled to %i.", memtbl_size);
+  }
+  i = lastused;
+  memtbl[i].ptr = x;
+  memtbl[i].line = line;
+  memtbl[i].size = size;
+  p = strrchr(file, '/');
+  strlcpy(memtbl[i].file, p ? p + 1 : file, sizeof memtbl[i].file);
+  memused += size;
+  lastused++;
 #endif
   return x;
 }
@@ -357,9 +378,8 @@ void *n_malloc(int size, const char *file, int line)
 void *n_realloc(void *ptr, int size, const char *file, int line)
 {
   void *x;
-  int i = 0;
-
 #ifdef DEBUG_MEM
+  int i = 0;
   char *p;
 #endif
 
@@ -369,7 +389,6 @@ void *n_realloc(void *ptr, int size, const char *file, int line)
 
   x = (void *) realloc(ptr, size);
   if (x == NULL && size > 0) {
-    i = i;
     putlog(LOG_MISC, "*", "*** FAILED REALLOC %s (%d)", file, line);
     return NULL;
   }
@@ -378,15 +397,14 @@ void *n_realloc(void *ptr, int size, const char *file, int line)
   if (i == lastused) {
     putlog(LOG_MISC, "*", "*** ATTEMPTING TO REALLOC NON-MALLOC'D PTR: %s (%d)",
            file, line);
-    return NULL;
+    /* If we reach this, realloc returned a pointer anyway. So we use it. */
   }
   memused -= memtbl[i].size;
   memtbl[i].ptr = x;
   memtbl[i].line = line;
   memtbl[i].size = size;
   p = strrchr(file, '/');
-  strncpy(memtbl[i].file, p ? p + 1 : file, 19);
-  memtbl[i].file[19] = 0;
+  strlcpy(memtbl[i].file, p ? p + 1 : file, sizeof memtbl[i].file);
   memused += size;
 #endif
   return x;
@@ -394,12 +412,13 @@ void *n_realloc(void *ptr, int size, const char *file, int line)
 
 void n_free(void *ptr, const char *file, int line)
 {
+#ifdef DEBUG_MEM
   int i = 0;
+#endif
 
   if (ptr == NULL) {
     putlog(LOG_MISC, "*", "*** ATTEMPTING TO FREE NULL PTR: %s (%d)",
            file, line);
-    i = i;
     return;
   }
 #ifdef DEBUG_MEM
