@@ -9,7 +9,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,14 +30,11 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <ctype.h>
 #include "chan.h"
 #include "tandem.h"
 #include "modules.h"
 
-#ifdef HAVE_UNAME
-#  include <sys/utsname.h>
-#endif
+#include <sys/utsname.h>
 
 #include "stat.h"
 
@@ -47,7 +44,7 @@ extern struct chanset_t *chanset;
 extern char helpdir[], version[], origbotname[], botname[], admin[], network[],
             motdfile[], ver[], botnetnick[], bannerfile[], textdir[];
 extern int  backgrd, con_chan, term_z, use_stderr, dcc_total, keep_all_logs,
-            quick_logs, strict_host;
+            quick_logs;
 
 extern time_t now;
 extern Tcl_Interp *interp;
@@ -60,8 +57,8 @@ log_t *logs = 0;                /* Logfiles */
 int max_logs = 5;               /* Max log files, mismatch config on purpose */
 int max_logsize = 0;            /* Maximum logfile size, 0 for no limit */
 int raw_log = 0;                /* Display output to server to LOG_SERVEROUT */
-
 int conmask = LOG_MODES | LOG_CMDS | LOG_MISC; /* Console mask */
+int show_uname = 1;
 
 struct help_list_t {
   struct help_list_t *next;
@@ -469,7 +466,7 @@ void daysago(time_t now, time_t then, char *out)
     sprintf(out, "%d day%s ago", days, (days == 1) ? "" : "s");
     return;
   }
-  egg_strftime(out, 6, "%H:%M", localtime(&then));
+  strftime(out, 6, "%H:%M", localtime(&then));
 }
 
 /* Convert an interval (in seconds) to one of:
@@ -483,7 +480,7 @@ void days(time_t now, time_t then, char *out)
     sprintf(out, "in %d day%s", days, (days == 1) ? "" : "s");
     return;
   }
-  egg_strftime(out, 9, "at %H:%M", localtime(&now));
+  strftime(out, 9, "at %H:%M", localtime(&now));
 }
 
 /* Convert an interval (in seconds) to one of:
@@ -520,10 +517,16 @@ void putlog EGG_VARARGS_DEF(int, arg1)
 {
   static int inhere = 0;
   int i, type, tsl = 0;
-  char *format, *chname, s[LOGLINELEN], s1[256], *out, ct[81], *s2, stamp[34];
+  char *format, *chname, s[LOGLINELEN], s1[LOGLINELEN], *out, ct[81], *s2, stamp[34];
   va_list va;
   time_t now2 = time(NULL);
-  struct tm *t = localtime(&now2);
+  static time_t now2_last = 0; /* cache expensive localtime() */
+  static struct tm *t;
+
+  if (now2 != now2_last) {
+    now2_last = now2;
+    t = localtime(&now2);
+  }
 
   type = EGG_VARARGS_START(int, arg1, va);
   chname = va_arg(va, char *);
@@ -531,7 +534,7 @@ void putlog EGG_VARARGS_DEF(int, arg1)
 
   /* Create the timestamp */
   if (shtime) {
-    egg_strftime(stamp, sizeof(stamp) - 2, log_ts, t);
+    strftime(stamp, sizeof(stamp) - 2, log_ts, t);
     strcat(stamp, " ");
     tsl = strlen(stamp);
   }
@@ -547,9 +550,9 @@ void putlog EGG_VARARGS_DEF(int, arg1)
   out[LOGLINEMAX - tsl] = 0;
   if (keep_all_logs) {
     if (!logfile_suffix[0])
-      egg_strftime(ct, 12, ".%d%b%Y", t);
+      strftime(ct, 12, ".%d%b%Y", t);
     else {
-      egg_strftime(ct, 80, logfile_suffix, t);
+      strftime(ct, 80, logfile_suffix, t);
       ct[80] = 0;
       s2 = ct;
       /* replace spaces by underscores */
@@ -589,7 +592,7 @@ void putlog EGG_VARARGS_DEF(int, arg1)
           /* Check if this is the same as the last line added to
            * the log. <cybah>
            */
-          if (!egg_strcasecmp(out + tsl, logs[i].szlast))
+          if (!strcasecmp(out + tsl, logs[i].szlast))
             /* It is a repeat, so increment repeats */
             logs[i].repeats++;
           else {
@@ -715,7 +718,7 @@ void flushlogs()
          */
         char stamp[33];
 
-        egg_strftime(stamp, sizeof(stamp) - 1, log_ts, localtime(&now));
+        strftime(stamp, sizeof(stamp) - 1, log_ts, localtime(&now));
         fprintf(logs[i].f, "%s ", stamp);
         fprintf(logs[i].f, MISC_LOGREPEAT, logs[i].repeats);
         /* Reset repeats */
@@ -775,6 +778,23 @@ static void subst_addcol(char *s, char *newcol)
   }
 }
 
+char *egg_uname()
+{
+  struct utsname u;
+  static char sysrel[(sizeof u.sysname) + (sizeof u.release)];
+
+  if (show_uname) {
+    if (uname(&u) < 0)
+      return "*unknown*";
+    else {
+      snprintf(sysrel, sizeof sysrel, "%s %s", u.sysname, u.release);
+      return sysrel;
+    }
+  }
+  else
+    return "";
+}
+
 /* Substitute %x codes in help files
  *
  * %B = bot nickname
@@ -802,15 +822,11 @@ static void subst_addcol(char *s, char *newcol)
 void help_subst(char *s, char *nick, struct flag_record *flags,
                 int isdcc, char *topic)
 {
-  char xx[HELP_BUF_LEN + 1], sub[161], *current, *q, chr, *writeidx,
-       *readidx, *towrite;
   struct chanset_t *chan;
   int i, j, center = 0;
   static int help_flags;
-
-#ifdef HAVE_UNAME
-  struct utsname uname_info;
-#endif
+  char xx[HELP_BUF_LEN + 1], *current, *q, chr, *writeidx, *readidx, *towrite,
+       sub[512];
 
   if (s == NULL) {
     /* Used to reset substitutions */
@@ -900,14 +916,7 @@ void help_subst(char *s, char *nick, struct flag_record *flags,
       }
       break;
     case 'U':
-#ifdef HAVE_UNAME
-      if (uname(&uname_info) >= 0) {
-        egg_snprintf(sub, sizeof sub, "%s %s", uname_info.sysname,
-                     uname_info.release);
-        towrite = sub;
-      } else
-#endif
-        towrite = "*UNKNOWN*";
+      towrite = egg_uname();
       break;
     case 'B':
       towrite = (isdcc ? botnetnick : botname);
@@ -925,7 +934,7 @@ void help_subst(char *s, char *nick, struct flag_record *flags,
       towrite = network;
       break;
     case 'T':
-      egg_strftime(sub, 6, "%H:%M", localtime(&now));
+      strftime(sub, sizeof sub, "%H:%M", localtime(&now));
       towrite = sub;
       break;
     case 'N':
@@ -961,7 +970,7 @@ void help_subst(char *s, char *nick, struct flag_record *flags,
         q += 2;
         /* Now q is the string and p is where the rest of the fcn expects */
         if (!strncmp(q, "help=", 5)) {
-          if (topic && egg_strcasecmp(q + 5, topic))
+          if (topic && strcasecmp(q + 5, topic))
             blind |= 2;
           else
             blind &= ~2;
@@ -980,7 +989,7 @@ void help_subst(char *s, char *nick, struct flag_record *flags,
               blind &= ~1;
           } else if (q[0] == '-')
             blind &= ~1;
-          else if (!egg_strcasecmp(q, "end")) {
+          else if (!strcasecmp(q, "end")) {
             blind &= ~1;
             subwidth = 70;
             if (cols) {
@@ -991,7 +1000,7 @@ void help_subst(char *s, char *nick, struct flag_record *flags,
               cols = 0;
               towrite = sub;
             }
-          } else if (!egg_strcasecmp(q, "center"))
+          } else if (!strcasecmp(q, "center"))
             center = 1;
           else if (!strncmp(q, "cols=", 5)) {
             char *r;
@@ -1181,7 +1190,7 @@ FILE *resolve_help(int dcc, char *file)
   struct help_ref *current;
   struct help_list_t *item;
 
-  /* Somewhere here goes the eventual substituation */
+  /* Somewhere here goes the eventual substitution */
   if (!(dcc & HELP_TEXT)) {
     for (current = help_list; current; current = current->next)
       for (item = current->first; item; item = item->next)
@@ -1434,21 +1443,22 @@ void show_banner(int idx)
   fclose(vv);
 }
 
-/* Create a string with random letters and digits
- */
-void make_rand_str(char *s, int len)
+void make_rand_str_from_chars(char *s, const int len, char *chars)
 {
-  int j;
+  int i;
 
-  for (j = 0; j < len; j++) {
-    if (!randint(3))
-      s[j] = '0' + randint(10);
-    else
-      s[j] = 'a' + randint(26);
-  }
+  for (i = 0; i < len; i++)
+    s[i] = chars[randint(strlen(chars))];
   s[len] = 0;
 }
 
+/* Create a string with random lower case letters and digits
+ */
+void make_rand_str(char *s, const int len)
+{
+  make_rand_str_from_chars(s, len, CHARSET_LOWER_ALPHA_NUM);
+}
+
 /* Convert an octal string into a decimal integer value.  If the string
  * is empty or contains non-octal characters, -1 is returned.
  */
@@ -1576,3 +1586,45 @@ void kill_bot(char *s1, char *s2)
   write_userfile(-1);
   fatal(s2, 2);
 }
+
+/* Compares two strings with constant-time algorithm to avoid timing attack and
+ * returns 0, if strings match, similar to strcmp().
+ */
+/* https://github.com/jedisct1/libsodium/blob/451bafc0d3d95d18f916dd7051687d343597228c/src/libsodium/crypto_verify/sodium/verify.c */
+/*
+ * ISC License
+ *
+ * Copyright (c) 2013-2020
+ * Frank Denis <j at pureftpd dot org>
+ *
+ * Permission to use, copy, modify, and/or distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ */
+int crypto_verify(const char *x_, const char *y_)
+{
+  const volatile unsigned char *volatile x =
+    (const volatile unsigned char *volatile) x_;
+  const volatile unsigned char *volatile y =
+    (const volatile unsigned char *volatile) y_;
+  volatile uint_fast16_t d = 0U;
+  int n, i;
+
+  /* Could leak string length */
+  n = strlen(x_);
+  if (n != strlen(y_))
+    return 1;
+
+  for (i = 0; i < n; i++) {
+    d |= x[i] ^ y[i];
+  }
+  return (1 & ((d - 1) >> 8)) - 1;
+}
