@@ -1,10 +1,9 @@
 /*
- * cmdschan.c -- part of channels.mod
  *   commands from a user via dcc that cause server interaction
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,23 +26,38 @@ static struct flag_record user = { FR_GLOBAL | FR_CHAN, 0, 0, 0, 0, 0 };
 static struct flag_record victim = { FR_GLOBAL | FR_CHAN, 0, 0, 0, 0, 0 };
 
 
+/* RFC 1035/2812- hostmasks can't be longer than 63 characters */
+void truncate_mask_hostname(char *s) {
+char *r = NULL;
+
+  if ( (r = strchr(s, '@')) ) {
+    r++;
+    if (strlen(r)  > HOSTMAX) {
+      r[62] = '*';
+      r[63] = 0;
+      putlog(LOG_MISC, "*", "Maximum hostlength exceeded, truncating");
+    }
+  }
+}
+
 static void cmd_pls_ban(struct userrec *u, int idx, char *par)
 {
   char *chname, *who, s[UHOSTLEN], s1[UHOSTLEN], *p, *p_expire;
-  unsigned long int expire_time = 0, expire_foo;
+  long expire_foo;
+  unsigned long expire_time = 0;
   int sticky = 0;
   struct chanset_t *chan = NULL;
   module_entry *me;
 
   if (!par[0]) {
-    dprintf(idx, "Usage: +ban <hostmask> [channel] [%%<XdXhXm>] [reason]\n");
+    dprintf(idx, "Usage: +ban <hostmask> [channel] [%%<XyXdXhXm>] [reason]\n");
   } else {
     who = newsplit(&par);
     /* Sanity check for <channel> <ban> vs. <ban> <channel> */
     if (par[0] && strchr(CHANMETA, who[0])) {
       chname = who;
       who = newsplit(&par);
-      dprintf(idx, "Usage: +ban <hostmask> [channel] [%%<XdXhXm>] [reason]\n");
+      dprintf(idx, "Usage: +ban <hostmask> [channel] [%%<XyXdXhXm>] [reason]\n");
       dprintf(idx, "Did you mean: .+ban %s %s %s\n", who, chname, par);
       return;
     } else if (par[0] && strchr(CHANMETA, par[0]))
@@ -72,31 +86,36 @@ static void cmd_pls_ban(struct userrec *u, int idx, char *par)
       p_expire = p + 1;
       while (*(++p) != 0) {
         switch (tolower((unsigned) *p)) {
+        case 'y':
+          *p = 0;
+          expire_foo = strtol(p_expire, NULL, 10);
+          expire_time += 60 * 60 * 24 * 365 * expire_foo;
+          p_expire = p + 1;
+          break;
         case 'd':
           *p = 0;
           expire_foo = strtol(p_expire, NULL, 10);
-          if (expire_foo > 365)
-            expire_foo = 365;
-          expire_time += 86400 * expire_foo;
+          expire_time += 60 * 60 * 24 * expire_foo;
           p_expire = p + 1;
           break;
         case 'h':
           *p = 0;
           expire_foo = strtol(p_expire, NULL, 10);
-          if (expire_foo > 8760)
-            expire_foo = 8760;
-          expire_time += 3600 * expire_foo;
+          expire_time += 60 * 60 * expire_foo;
           p_expire = p + 1;
           break;
         case 'm':
           *p = 0;
           expire_foo = strtol(p_expire, NULL, 10);
-          if (expire_foo > 525600)
-            expire_foo = 525600;
           expire_time += 60 * expire_foo;
           p_expire = p + 1;
         }
       }
+      if (expire_time > (60 * 60 * 24 * 365 * 5)) {
+        dprintf(idx, "Ban expiration time cannot exceed 5 years "
+            "(1825 days)\n");
+        return;
+      }
     }
     if (!par[0])
       par = "requested";
@@ -113,7 +132,7 @@ static void cmd_pls_ban(struct userrec *u, int idx, char *par)
     } else if (!strchr(who, '@'))
       egg_snprintf(s, sizeof s, "%s@*", who);   /* brain-dead? */
     else
-      strncpyz(s, who, sizeof s);
+      strlcpy(s, who, sizeof s);
     if ((me = module_find("server", 0, 0)) && me->funcs) {
       egg_snprintf(s1, sizeof s1, "%s!%s", me->funcs[SERVER_BOTNAME],
                    me->funcs[SERVER_BOTUSERHOST]);
@@ -123,55 +142,52 @@ static void cmd_pls_ban(struct userrec *u, int idx, char *par)
         return;
       }
     }
-      /* IRC can't understand bans longer than 70 characters */
-      if (strlen(s) > 70) {
-        s[69] = '*';
-        s[70] = 0;
-      }
-      if (chan) {
-        u_addban(chan, s, dcc[idx].nick, par,
-                 expire_time ? now + expire_time : 0, 0);
-        if (par[0] == '*') {
-          sticky = 1;
-          par++;
-          putlog(LOG_CMDS, "*", "#%s# (%s) +ban %s %s (%s) (sticky)",
-                 dcc[idx].nick, dcc[idx].u.chat->con_chan, s, chan->dname, par);
-          dprintf(idx, "New %s sticky ban: %s (%s)\n", chan->dname, s, par);
-        } else {
-          putlog(LOG_CMDS, "*", "#%s# (%s) +ban %s %s (%s)", dcc[idx].nick,
-                 dcc[idx].u.chat->con_chan, s, chan->dname, par);
-          dprintf(idx, "New %s ban: %s (%s)\n", chan->dname, s, par);
-        }
-        /* Avoid unnesessary modes if you got +dynamicbans, and there is
-         * no reason to set mode if irc.mod aint loaded. (dw 001120)
-         */
-        if ((me = module_find("irc", 0, 0)))
-          (me->funcs[IRC_CHECK_THIS_BAN]) (chan, s, sticky);
+    truncate_mask_hostname(s);
+    if (chan) {
+      u_addban(chan, s, dcc[idx].nick, par,
+               expire_time ? now + expire_time : 0, 0);
+      if (par[0] == '*') {
+        sticky = 1;
+        par++;
+        putlog(LOG_CMDS, "*", "#%s# (%s) +ban %s %s (%s) (sticky)",
+               dcc[idx].nick, dcc[idx].u.chat->con_chan, s, chan->dname, par);
+        dprintf(idx, "New %s sticky ban: %s (%s)\n", chan->dname, s, par);
       } else {
-        u_addban(NULL, s, dcc[idx].nick, par,
-                 expire_time ? now + expire_time : 0, 0);
-        if (par[0] == '*') {
-          sticky = 1;
-          par++;
-          putlog(LOG_CMDS, "*", "#%s# (GLOBAL) +ban %s (%s) (sticky)",
-                 dcc[idx].nick, s, par);
-          dprintf(idx, "New sticky ban: %s (%s)\n", s, par);
-        } else {
-          putlog(LOG_CMDS, "*", "#%s# (GLOBAL) +ban %s (%s)", dcc[idx].nick,
-                 s, par);
-          dprintf(idx, "New ban: %s (%s)\n", s, par);
-        }
-        if ((me = module_find("irc", 0, 0)))
-          for (chan = chanset; chan != NULL; chan = chan->next)
-            (me->funcs[IRC_CHECK_THIS_BAN]) (chan, s, sticky);
+        putlog(LOG_CMDS, "*", "#%s# (%s) +ban %s %s (%s)", dcc[idx].nick,
+               dcc[idx].u.chat->con_chan, s, chan->dname, par);
+        dprintf(idx, "New %s ban: %s (%s)\n", chan->dname, s, par);
       }
+      /* Avoid unnesessary modes if you got +dynamicbans, and there is
+       * no reason to set mode if irc.mod aint loaded. (dw 001120)
+       */
+      if ((me = module_find("irc", 0, 0)))
+        (me->funcs[IRC_CHECK_THIS_BAN]) (chan, s, sticky);
+    } else {
+      u_addban(NULL, s, dcc[idx].nick, par,
+               expire_time ? now + expire_time : 0, 0);
+      if (par[0] == '*') {
+        sticky = 1;
+        par++;
+        putlog(LOG_CMDS, "*", "#%s# (GLOBAL) +ban %s (%s) (sticky)",
+               dcc[idx].nick, s, par);
+        dprintf(idx, "New sticky ban: %s (%s)\n", s, par);
+      } else {
+        putlog(LOG_CMDS, "*", "#%s# (GLOBAL) +ban %s (%s)", dcc[idx].nick,
+               s, par);
+        dprintf(idx, "New ban: %s (%s)\n", s, par);
+      }
+      if ((me = module_find("irc", 0, 0)))
+        for (chan = chanset; chan != NULL; chan = chan->next)
+          (me->funcs[IRC_CHECK_THIS_BAN]) (chan, s, sticky);
+    }
   }
 }
 
 static void cmd_pls_exempt(struct userrec *u, int idx, char *par)
 {
   char *chname, *who, s[UHOSTLEN], *p, *p_expire;
-  unsigned long int expire_time = 0, expire_foo;
+  long expire_foo;
+  unsigned long expire_time = 0;
   struct chanset_t *chan = NULL;
 
   if (!use_exempts) {
@@ -179,7 +195,7 @@ static void cmd_pls_exempt(struct userrec *u, int idx, char *par)
     return;
   }
   if (!par[0]) {
-    dprintf(idx, "Usage: +exempt <hostmask> [channel] [%%<XdXhXm>] [reason]\n");
+    dprintf(idx, "Usage: +exempt <hostmask> [channel] [%%<XydXhXm>] [reason]\n");
   } else {
     who = newsplit(&par);
     if (par[0] && strchr(CHANMETA, par[0]))
@@ -208,31 +224,36 @@ static void cmd_pls_exempt(struct userrec *u, int idx, char *par)
       p_expire = p + 1;
       while (*(++p) != 0) {
         switch (tolower((unsigned) *p)) {
+        case 'y':
+          *p = 0;
+          expire_foo = strtol(p_expire, NULL, 10);
+          expire_time += 60 * 60 * 24 * 365 * expire_foo;
+          p_expire = p + 1;
+          break;
         case 'd':
           *p = 0;
           expire_foo = strtol(p_expire, NULL, 10);
-          if (expire_foo > 365)
-            expire_foo = 365;
-          expire_time += 86400 * expire_foo;
+          expire_time += 60 * 60 * 24 * expire_foo;
           p_expire = p + 1;
           break;
         case 'h':
           *p = 0;
           expire_foo = strtol(p_expire, NULL, 10);
-          if (expire_foo > 8760)
-            expire_foo = 8760;
-          expire_time += 3600 * expire_foo;
+          expire_time += 60 * 60 * expire_foo;
           p_expire = p + 1;
           break;
         case 'm':
           *p = 0;
           expire_foo = strtol(p_expire, NULL, 10);
-          if (expire_foo > 525600)
-            expire_foo = 525600;
           expire_time += 60 * expire_foo;
           p_expire = p + 1;
         }
       }
+      if (expire_time > (60 * 60 * 24 * 365 * 5)) {
+        dprintf(idx, "Exempt expiration time cannot exceed 5 years "
+            "(1825 days)\n");
+        return;
+      }
     }
     if (!par[0])
       par = "requested";
@@ -249,13 +270,9 @@ static void cmd_pls_exempt(struct userrec *u, int idx, char *par)
     } else if (!strchr(who, '@'))
       egg_snprintf(s, sizeof s, "%s@*", who);   /* brain-dead? */
     else
-      strncpyz(s, who, sizeof s);
+      strlcpy(s, who, sizeof s);
 
-    /* IRC can't understand exempts longer than 70 characters */
-    if (strlen(s) > 70) {
-      s[69] = '*';
-      s[70] = 0;
-    }
+    truncate_mask_hostname(s);
     if (chan) {
       u_addexempt(chan, s, dcc[idx].nick, par,
                   expire_time ? now + expire_time : 0, 0);
@@ -292,7 +309,8 @@ static void cmd_pls_exempt(struct userrec *u, int idx, char *par)
 static void cmd_pls_invite(struct userrec *u, int idx, char *par)
 {
   char *chname, *who, s[UHOSTLEN], *p, *p_expire;
-  unsigned long int expire_time = 0, expire_foo;
+  long expire_foo;
+  unsigned long expire_time = 0;
   struct chanset_t *chan = NULL;
 
   if (!use_invites) {
@@ -301,7 +319,7 @@ static void cmd_pls_invite(struct userrec *u, int idx, char *par)
   }
 
   if (!par[0]) {
-    dprintf(idx, "Usage: +invite <hostmask> [channel] [%%<XdXhXm>] [reason]\n");
+    dprintf(idx, "Usage: +invite <hostmask> [channel] [%%<XyXdXhXm>] [reason]\n");
   } else {
     who = newsplit(&par);
     if (par[0] && strchr(CHANMETA, par[0]))
@@ -330,31 +348,36 @@ static void cmd_pls_invite(struct userrec *u, int idx, char *par)
       p_expire = p + 1;
       while (*(++p) != 0) {
         switch (tolower((unsigned) *p)) {
+        case 'y':
+          *p = 0;
+          expire_foo = strtol(p_expire, NULL, 10);
+          expire_time += 60 * 60 * 24 * 365 * expire_foo;
+          p_expire = p + 1;
+          break;
         case 'd':
           *p = 0;
           expire_foo = strtol(p_expire, NULL, 10);
-          if (expire_foo > 365)
-            expire_foo = 365;
-          expire_time += 86400 * expire_foo;
+          expire_time += 60 * 60 * 24 * expire_foo;
           p_expire = p + 1;
           break;
         case 'h':
           *p = 0;
           expire_foo = strtol(p_expire, NULL, 10);
-          if (expire_foo > 8760)
-            expire_foo = 8760;
-          expire_time += 3600 * expire_foo;
+          expire_time += 60 * 60 * expire_foo;
           p_expire = p + 1;
           break;
         case 'm':
           *p = 0;
           expire_foo = strtol(p_expire, NULL, 10);
-          if (expire_foo > 525600)
-            expire_foo = 525600;
           expire_time += 60 * expire_foo;
           p_expire = p + 1;
         }
       }
+      if (expire_time > (60 * 60 * 24 * 365 * 5)) {
+        dprintf(idx, "Invite expiration time cannot exceed 5 years "
+            "(1825 days)\n");
+        return;
+      }
     }
     if (!par[0])
       par = "requested";
@@ -371,13 +394,9 @@ static void cmd_pls_invite(struct userrec *u, int idx, char *par)
     } else if (!strchr(who, '@'))
       egg_snprintf(s, sizeof s, "%s@*", who);   /* brain-dead? */
     else
-      strncpyz(s, who, sizeof s);
+      strlcpy(s, who, sizeof s);
 
-    /* IRC can't understand invites longer than 70 characters */
-    if (strlen(s) > 70) {
-      s[69] = '*';
-      s[70] = 0;
-    }
+    truncate_mask_hostname(s);
     if (chan) {
       u_addinvite(chan, s, dcc[idx].nick, par,
                   expire_time ? now + expire_time : 0, 0);
@@ -447,7 +466,7 @@ static void cmd_mns_ban(struct userrec *u, int idx, char *par)
       return;
     }
   }
-  strncpyz(s, ban, sizeof s);
+  strlcpy(s, ban, sizeof s);
   if (console) {
     i = u_delban(NULL, s, (u->flags & USER_OP));
     if (i > 0) {
@@ -471,7 +490,7 @@ static void cmd_mns_ban(struct userrec *u, int idx, char *par)
   }
   if (str_isdigit(ban)) {
     i = atoi(ban);
-    /* substract the numer of global bans to get the number of the channel ban */
+    /* subtract the number of global bans to get the number of the channel ban */
     egg_snprintf(s, sizeof s, "%d", i);
     j = u_delban(0, s, 0);
     if (j < 0) {
@@ -558,7 +577,7 @@ static void cmd_mns_exempt(struct userrec *u, int idx, char *par)
       return;
     }
   }
-  strncpyz(s, exempt, sizeof s);
+  strlcpy(s, exempt, sizeof s);
   if (console) {
     i = u_delexempt(NULL, s, (u->flags & USER_OP));
     if (i > 0) {
@@ -582,7 +601,7 @@ static void cmd_mns_exempt(struct userrec *u, int idx, char *par)
   }
   if (str_isdigit(exempt)) {
     i = atoi(exempt);
-    /* substract the numer of global exempts to get the number of the channel exempt */
+    /* subtract the number of global exempts to get the number of the channel exempt */
     egg_snprintf(s, sizeof s, "%d", i);
     j = u_delexempt(0, s, 0);
     if (j < 0) {
@@ -670,7 +689,7 @@ static void cmd_mns_invite(struct userrec *u, int idx, char *par)
       return;
     }
   }
-  strncpyz(s, invite, sizeof s);
+  strlcpy(s, invite, sizeof s);
   if (console) {
     i = u_delinvite(NULL, s, (u->flags & USER_OP));
     if (i > 0) {
@@ -694,7 +713,7 @@ static void cmd_mns_invite(struct userrec *u, int idx, char *par)
   }
   if (str_isdigit(invite)) {
     i = atoi(invite);
-    /* substract the numer of global invites to get the number of the channel invite */
+    /* subtract the number of global invites to get the number of the channel invite */
     egg_snprintf(s, sizeof s, "%d", i);
     j = u_delinvite(0, s, 0);
     if (j < 0) {
@@ -754,7 +773,7 @@ static void cmd_mns_invite(struct userrec *u, int idx, char *par)
 
 static void cmd_bans(struct userrec *u, int idx, char *par)
 {
-  if (!egg_strcasecmp(par, "all")) {
+  if (!strcasecmp(par, "all")) {
     putlog(LOG_CMDS, "*", "#%s# bans all", dcc[idx].nick);
     tell_bans(idx, 1, "");
   } else {
@@ -769,7 +788,7 @@ static void cmd_exempts(struct userrec *u, int idx, char *par)
     dprintf(idx, "This command can only be used with use-exempts enabled.\n");
     return;
   }
-  if (!egg_strcasecmp(par, "all")) {
+  if (!strcasecmp(par, "all")) {
     putlog(LOG_CMDS, "*", "#%s# exempts all", dcc[idx].nick);
     tell_exempts(idx, 1, "");
   } else {
@@ -784,7 +803,7 @@ static void cmd_invites(struct userrec *u, int idx, char *par)
     dprintf(idx, "This command can only be used with use-invites enabled.\n");
     return;
   }
-  if (!egg_strcasecmp(par, "all")) {
+  if (!strcasecmp(par, "all")) {
     putlog(LOG_CMDS, "*", "#%s# invites all", dcc[idx].nick);
     tell_invites(idx, 1, "");
   } else {
@@ -837,7 +856,7 @@ static void cmd_info(struct userrec *u, int idx, char *par)
     dprintf(idx, "Your info line is locked.  Sorry.\n");
     return;
   }
-  if (!egg_strcasecmp(par, "none")) {
+  if (!strcasecmp(par, "none")) {
     if (chname) {
       par[0] = 0;
       set_handle_chaninfo(userlist, dcc[idx].nick, chname, NULL);
@@ -909,7 +928,7 @@ static void cmd_chinfo(struct userrec *u, int idx, char *par)
   }
   putlog(LOG_CMDS, "*", "#%s# chinfo %s %s %s", dcc[idx].nick, handle,
          chname ? chname : par, chname ? par : "");
-  if (!egg_strcasecmp(par, "none"))
+  if (!strcasecmp(par, "none"))
     par[0] = 0;
   if (chname) {
     set_handle_chaninfo(userlist, handle, chname, par);
@@ -935,18 +954,18 @@ static void cmd_stick_yn(int idx, char *par, int yn)
 {
   int i = 0, j;
   struct chanset_t *chan, *achan;
-  char *stick_type, s[UHOSTLEN], chname[81];
+  char *stick_type, s[UHOSTLEN], chname[CHANNELLEN + 1];
   module_entry *me;
 
   stick_type = newsplit(&par);
-  strncpyz(s, newsplit(&par), sizeof s);
-  strncpyz(chname, newsplit(&par), sizeof chname);
+  strlcpy(s, newsplit(&par), sizeof s);
+  strlcpy(chname, newsplit(&par), sizeof chname);
 
-  if (egg_strcasecmp(stick_type, "exempt") &&
-      egg_strcasecmp(stick_type, "invite") &&
-      egg_strcasecmp(stick_type, "ban")) {
-    strncpyz(chname, s, sizeof chname);
-    strncpyz(s, stick_type, sizeof s);
+  if (strcasecmp(stick_type, "exempt") &&
+      strcasecmp(stick_type, "invite") &&
+      strcasecmp(stick_type, "ban")) {
+    strlcpy(chname, s, sizeof chname);
+    strlcpy(s, stick_type, sizeof s);
   }
   if (!s[0]) {
     dprintf(idx, "Usage: %sstick [ban/exempt/invite] <hostmask or number> "
@@ -954,7 +973,7 @@ static void cmd_stick_yn(int idx, char *par, int yn)
     return;
   }
   /* Now deal with exemptions */
-  if (!egg_strcasecmp(stick_type, "exempt")) {
+  if (!strcasecmp(stick_type, "exempt")) {
     if (!use_exempts) {
       dprintf(idx, "This command can only be used with use-exempts "
               "enabled.\n");
@@ -969,7 +988,7 @@ static void cmd_stick_yn(int idx, char *par, int yn)
         dprintf(idx, "%stuck exempt: %s\n", yn ? "S" : "Uns", s);
         return;
       }
-      strncpyz(chname, dcc[idx].u.chat->con_chan, sizeof chname);
+      strlcpy(chname, dcc[idx].u.chat->con_chan, sizeof chname);
     }
     /* Channel-specific exempt? */
     if (!(chan = findchan_by_dname(chname))) {
@@ -977,7 +996,7 @@ static void cmd_stick_yn(int idx, char *par, int yn)
       return;
     }
     if (str_isdigit(s)) {
-      /* substract the numer of global exempts to get the number of the channel exempt */
+      /* subtract the number of global exempts to get the number of the channel exempt */
       j = u_setsticky_exempt(NULL, s, -1);
       if (j < 0)
         egg_snprintf(s, sizeof s, "%d", -j);
@@ -993,7 +1012,7 @@ static void cmd_stick_yn(int idx, char *par, int yn)
     return;
   }
   /* Now the invites */
-  else if (!egg_strcasecmp(stick_type, "invite")) {
+  else if (!strcasecmp(stick_type, "invite")) {
     if (!use_invites) {
       dprintf(idx, "This command can only be used with use-invites enabled.\n");
       return;
@@ -1007,7 +1026,7 @@ static void cmd_stick_yn(int idx, char *par, int yn)
         dprintf(idx, "%stuck invite: %s\n", yn ? "S" : "Uns", s);
         return;
       }
-      strncpyz(chname, dcc[idx].u.chat->con_chan, sizeof chname);
+      strlcpy(chname, dcc[idx].u.chat->con_chan, sizeof chname);
     }
     /* Channel-specific invite? */
     if (!(chan = findchan_by_dname(chname))) {
@@ -1015,7 +1034,7 @@ static void cmd_stick_yn(int idx, char *par, int yn)
       return;
     }
     if (str_isdigit(s)) {
-      /* substract the numer of global invites to get the number of the channel invite */
+      /* subtract the number of global invites to get the number of the channel invite */
       j = u_setsticky_invite(NULL, s, -1);
       if (j < 0)
         egg_snprintf(s, sizeof s, "%d", -j);
@@ -1042,7 +1061,7 @@ static void cmd_stick_yn(int idx, char *par, int yn)
           (me->funcs[IRC_CHECK_THIS_BAN]) (achan, s, yn);
       return;
     }
-    strncpyz(chname, dcc[idx].u.chat->con_chan, sizeof chname);
+    strlcpy(chname, dcc[idx].u.chat->con_chan, sizeof chname);
   }
   /* Channel-specific ban? */
   if (!(chan = findchan_by_dname(chname))) {
@@ -1050,7 +1069,7 @@ static void cmd_stick_yn(int idx, char *par, int yn)
     return;
   }
   if (str_isdigit(s)) {
-    /* substract the numer of global bans to get the number of the channel ban */
+    /* subtract the number of global bans to get the number of the channel ban */
     j = u_setsticky_ban(NULL, s, -1);
     if (j < 0)
       egg_snprintf(s, sizeof s, "%d", -j);
@@ -1406,7 +1425,7 @@ static void cmd_chaninfo(struct userrec *u, int idx, char *par)
           dprintf(idx, "User defined channel settings:\n");
           tmp = 1;
         }
-        egg_snprintf(work + work_len, sizeof(work) - work_len, "%s: %d   ",
+        egg_snprintf(work + work_len, sizeof(work) - work_len, "%s: %" PRIdPTR "   ",
                      ul->name, getudef(ul->values, chan->dname));
         ii++;
         if (ii > 4) {
@@ -1490,8 +1509,7 @@ static void cmd_chanset(struct userrec *u, int idx, char *par)
         dprintf(idx, "Usage: chanset [%schannel] <settings>\n", CHANMETA);
         return;
       }
-      if (!chan &&
-          !(chan = findchan_by_dname(chname = dcc[idx].u.chat->con_chan))) {
+      if (!chan && !(chan = findchan_by_dname(dcc[idx].u.chat->con_chan))) {
         dprintf(idx, "Invalid console channel.\n");
         return;
       }
@@ -1544,9 +1562,8 @@ static void cmd_chanset(struct userrec *u, int idx, char *par)
           strcpy(parcpy, par);
           irp = Tcl_CreateInterp();
           if (tcl_channel_modify(irp, chan, 2, list) == TCL_OK) {
-            char tocat[sizeof answers];
-            egg_snprintf(tocat, sizeof tocat, "%s { %s }", list[0], parcpy);
-            strncat(answers, tocat, sizeof answers - strlen(answers) - 1);
+            int len = strlen(answers);
+            egg_snprintf(answers + len, (sizeof answers) - len, "%s { %s }", list[0], parcpy); /* Concatenation */
           } else if (!all || !chan->next)
             dprintf(idx, "Error trying to set %s for %s, %s\n",
                     list[0], all ? "all channels" : chname, Tcl_GetStringResult(irp));
