@@ -3,7 +3,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -203,11 +203,11 @@ static int tcl_stickexempt STDVAR
 static int tcl_isban STDVAR
 {
   struct chanset_t *chan;
-  int ok = 0;
+  int chanarg = 1, ok = 0;
 
-  BADARGS(2, 3, " ban ?channel?");
+  BADARGS(2, 4, " ban ?channel? ?-channel?");
 
-  if (argc == 3) {
+  if (argc >= 3) {
     chan = findchan_by_dname(argv[2]);
     if (!chan) {
       Tcl_AppendResult(irp, "invalid channel: ", argv[2], NULL);
@@ -216,7 +216,15 @@ static int tcl_isban STDVAR
     if (u_equals_mask(chan->bans, argv[1]))
       ok = 1;
   }
-  if (u_equals_mask(global_bans, argv[1]))
+  if (argc == 4) {
+    if (!strcasecmp(argv[3], "-channel")) {
+      chanarg = 0;
+    } else {
+      Tcl_AppendResult(irp, "invalid flag", NULL);
+      return TCL_ERROR;
+    }
+  }
+  if (u_equals_mask(global_bans, argv[1]) && chanarg)
     ok = 1;
   if (ok)
     Tcl_AppendResult(irp, "1", NULL);
@@ -228,11 +236,11 @@ static int tcl_isban STDVAR
 static int tcl_isexempt STDVAR
 {
   struct chanset_t *chan;
-  int ok = 0;
+  int chanarg = 1, ok = 0;
 
-  BADARGS(2, 3, " exempt ?channel?");
+  BADARGS(2, 4, " exempt ?channel? ?-channel?");
 
-  if (argc == 3) {
+  if (argc >= 3) {
     chan = findchan_by_dname(argv[2]);
     if (!chan) {
       Tcl_AppendResult(irp, "invalid channel: ", argv[2], NULL);
@@ -241,7 +249,15 @@ static int tcl_isexempt STDVAR
     if (u_equals_mask(chan->exempts, argv[1]))
       ok = 1;
   }
-  if (u_equals_mask(global_exempts, argv[1]))
+  if (argc == 4) {
+    if (!strcasecmp(argv[3], "-channel")) {
+      chanarg = 0;
+    } else {
+      Tcl_AppendResult(irp, "invalid flag", NULL);
+      return TCL_ERROR;
+    }
+  }
+  if (u_equals_mask(global_exempts, argv[1]) && chanarg)
     ok = 1;
   if (ok)
     Tcl_AppendResult(irp, "1", NULL);
@@ -253,11 +269,11 @@ static int tcl_isexempt STDVAR
 static int tcl_isinvite STDVAR
 {
   struct chanset_t *chan;
-  int ok = 0;
+  int chanarg = 1, ok = 0;
 
-  BADARGS(2, 3, " invite ?channel?");
+  BADARGS(2, 4, " invite ?channel? ?-channel?");
 
-  if (argc == 3) {
+  if (argc >= 3) {
     chan = findchan_by_dname(argv[2]);
     if (!chan) {
       Tcl_AppendResult(irp, "invalid channel: ", argv[2], NULL);
@@ -266,7 +282,15 @@ static int tcl_isinvite STDVAR
     if (u_equals_mask(chan->invites, argv[1]))
       ok = 1;
   }
-  if (u_equals_mask(global_invites, argv[1]))
+  if (argc == 4) {
+    if (!strcasecmp(argv[3], "-channel")) {
+      chanarg = 0;
+    } else {
+      Tcl_AppendResult(irp, "invalid flag", NULL);
+      return TCL_ERROR;
+    }
+  }
+  if (u_equals_mask(global_invites, argv[1]) && chanarg)
     ok = 1;
   if (ok)
     Tcl_AppendResult(irp, "1", NULL);
@@ -279,11 +303,11 @@ static int tcl_isinvite STDVAR
 static int tcl_isbansticky STDVAR
 {
   struct chanset_t *chan;
-  int ok = 0;
+  int chanarg = 1, ok = 0;
 
-  BADARGS(2, 3, " ban ?channel?");
+  BADARGS(2, 4, " ban ?channel? ?-channel?");
 
-  if (argc == 3) {
+  if (argc >= 3) {
     chan = findchan_by_dname(argv[2]);
     if (!chan) {
       Tcl_AppendResult(irp, "invalid channel: ", argv[2], NULL);
@@ -292,7 +316,15 @@ static int tcl_isbansticky STDVAR
     if (u_sticky_mask(chan->bans, argv[1]))
       ok = 1;
   }
-  if (u_sticky_mask(global_bans, argv[1]))
+  if (argc == 4) {
+    if (!strcasecmp(argv[3], "-channel")) {
+      chanarg = 0;
+    } else {
+      Tcl_AppendResult(irp, "invalid flag", NULL);
+      return TCL_ERROR;
+    }
+  }
+  if (u_sticky_mask(global_bans, argv[1]) && chanarg)
     ok = 1;
   if (ok)
     Tcl_AppendResult(irp, "1", NULL);
@@ -304,11 +336,11 @@ static int tcl_isbansticky STDVAR
 static int tcl_isexemptsticky STDVAR
 {
   struct chanset_t *chan;
-  int ok = 0;
+  int chanarg = 1, ok = 0;
 
-  BADARGS(2, 3, " exempt ?channel?");
+  BADARGS(2, 4, " exempt ?channel? ?-channel?");
 
-  if (argc == 3) {
+  if (argc >= 3) {
     chan = findchan_by_dname(argv[2]);
     if (!chan) {
       Tcl_AppendResult(irp, "invalid channel: ", argv[2], NULL);
@@ -317,7 +349,15 @@ static int tcl_isexemptsticky STDVAR
     if (u_sticky_mask(chan->exempts, argv[1]))
       ok = 1;
   }
-  if (u_sticky_mask(global_exempts, argv[1]))
+  if (argc == 4) {
+    if (!strcasecmp(argv[3], "-channel")) {
+      chanarg = 0;
+    } else {
+      Tcl_AppendResult(irp, "invalid flag", NULL);
+      return TCL_ERROR;
+    }
+  }
+  if (u_sticky_mask(global_exempts, argv[1]) && chanarg)
     ok = 1;
   if (ok)
     Tcl_AppendResult(irp, "1", NULL);
@@ -329,11 +369,11 @@ static int tcl_isexemptsticky STDVAR
 static int tcl_isinvitesticky STDVAR
 {
   struct chanset_t *chan;
-  int ok = 0;
+  int chanarg = 1, ok = 0;
 
-  BADARGS(2, 3, " invite ?channel?");
+  BADARGS(2, 4, " invite ?channel? ?-channel?");
 
-  if (argc == 3) {
+  if (argc >= 3) {
     chan = findchan_by_dname(argv[2]);
     if (!chan) {
       Tcl_AppendResult(irp, "invalid channel: ", argv[2], NULL);
@@ -342,7 +382,15 @@ static int tcl_isinvitesticky STDVAR
     if (u_sticky_mask(chan->invites, argv[1]))
       ok = 1;
   }
-  if (u_sticky_mask(global_invites, argv[1]))
+  if (argc == 4) {
+    if (!strcasecmp(argv[3], "-channel")) {
+      chanarg = 0;
+    } else {
+      Tcl_AppendResult(irp, "invalid flag", NULL);
+      return TCL_ERROR;
+    }
+  }
+  if (u_sticky_mask(global_invites, argv[1]) && chanarg)
     ok = 1;
   if (ok)
     Tcl_AppendResult(irp, "1", NULL);
@@ -354,11 +402,11 @@ static int tcl_isinvitesticky STDVAR
 static int tcl_ispermban STDVAR
 {
   struct chanset_t *chan;
-  int ok = 0;
+  int chanarg = 1, ok = 0;
 
-  BADARGS(2, 3, " ban ?channel?");
+  BADARGS(2, 4, " ban ?channel? ?-channel?");
 
-  if (argc == 3) {
+  if (argc >= 3) {
     chan = findchan_by_dname(argv[2]);
     if (chan == NULL) {
       Tcl_AppendResult(irp, "invalid channel: ", argv[2], NULL);
@@ -367,7 +415,15 @@ static int tcl_ispermban STDVAR
     if (u_equals_mask(chan->bans, argv[1]) == 2)
       ok = 1;
   }
-  if (u_equals_mask(global_bans, argv[1]) == 2)
+  if (argc == 4) {
+    if (!strcasecmp(argv[3], "-channel")) {
+      chanarg = 0;
+    } else {
+      Tcl_AppendResult(irp, "invalid flag", NULL);
+      return TCL_ERROR;
+    }
+  }
+  if ((u_equals_mask(global_bans, argv[1]) == 2) && chanarg)
     ok = 1;
   if (ok)
     Tcl_AppendResult(irp, "1", NULL);
@@ -379,11 +435,11 @@ static int tcl_ispermban STDVAR
 static int tcl_ispermexempt STDVAR
 {
   struct chanset_t *chan;
-  int ok = 0;
+  int chanarg = 1, ok = 0;
 
-  BADARGS(2, 3, " exempt ?channel?");
+  BADARGS(2, 4, " exempt ?channel? ?-channel?");
 
-  if (argc == 3) {
+  if (argc >= 3) {
     chan = findchan_by_dname(argv[2]);
     if (chan == NULL) {
       Tcl_AppendResult(irp, "invalid channel: ", argv[2], NULL);
@@ -392,7 +448,15 @@ static int tcl_ispermexempt STDVAR
     if (u_equals_mask(chan->exempts, argv[1]) == 2)
       ok = 1;
   }
-  if (u_equals_mask(global_exempts, argv[1]) == 2)
+  if (argc == 4) {
+    if (!strcasecmp(argv[3], "-channel")) {
+      chanarg = 0;
+    } else {
+      Tcl_AppendResult(irp, "invalid flag", NULL);
+      return TCL_ERROR;
+    }
+  }
+  if ((u_equals_mask(global_exempts, argv[1]) == 2) && chanarg)
     ok = 1;
   if (ok)
     Tcl_AppendResult(irp, "1", NULL);
@@ -404,11 +468,11 @@ static int tcl_ispermexempt STDVAR
 static int tcl_isperminvite STDVAR
 {
   struct chanset_t *chan;
-  int ok = 0;
+  int chanarg = 1, ok = 0;
 
-  BADARGS(2, 3, " invite ?channel?");
+  BADARGS(2, 4, " invite ?channel? ?-channel?");
 
-  if (argc == 3) {
+  if (argc >= 3) {
     chan = findchan_by_dname(argv[2]);
     if (chan == NULL) {
       Tcl_AppendResult(irp, "invalid channel: ", argv[2], NULL);
@@ -417,7 +481,15 @@ static int tcl_isperminvite STDVAR
     if (u_equals_mask(chan->invites, argv[1]) == 2)
       ok = 1;
   }
-  if (u_equals_mask(global_invites, argv[1]) == 2)
+  if (argc == 4) {
+    if (!strcasecmp(argv[3], "-channel")) {
+      chanarg = 0;
+    } else {
+      Tcl_AppendResult(irp, "invalid flag", NULL);
+      return TCL_ERROR;
+    }
+  }
+  if ((u_equals_mask(global_invites, argv[1]) == 2) && chanarg)
     ok = 1;
   if (ok)
     Tcl_AppendResult(irp, "1", NULL);
@@ -429,11 +501,11 @@ static int tcl_isperminvite STDVAR
 static int tcl_matchban STDVAR
 {
   struct chanset_t *chan;
-  int ok = 0;
+  int chanarg = 1, ok = 0;
 
-  BADARGS(2, 3, " user!nick@host ?channel?");
+  BADARGS(2, 4, " user!nick@host ?channel? ?-channel?");
 
-  if (argc == 3) {
+  if (argc >= 3) {
     chan = findchan_by_dname(argv[2]);
     if (chan == NULL) {
       Tcl_AppendResult(irp, "invalid channel: ", argv[2], NULL);
@@ -442,7 +514,15 @@ static int tcl_matchban STDVAR
     if (u_match_mask(chan->bans, argv[1]))
       ok = 1;
   }
-  if (u_match_mask(global_bans, argv[1]))
+  if (argc == 4) {
+    if (!strcasecmp(argv[3], "-channel")) {
+      chanarg = 0;
+    } else {
+      Tcl_AppendResult(irp, "invalid flag", NULL);
+      return TCL_ERROR;
+    }
+  }
+  if ((u_match_mask(global_bans, argv[1])) && chanarg)
     ok = 1;
   if (ok)
     Tcl_AppendResult(irp, "1", NULL);
@@ -454,11 +534,11 @@ static int tcl_matchban STDVAR
 static int tcl_matchexempt STDVAR
 {
   struct chanset_t *chan;
-  int ok = 0;
+  int chanarg = 1, ok = 0;
 
-  BADARGS(2, 3, " user!nick@host ?channel?");
+  BADARGS(2, 4, " user!nick@host ?channel? ?-channel?");
 
-  if (argc == 3) {
+  if (argc >= 3) {
     chan = findchan_by_dname(argv[2]);
     if (chan == NULL) {
       Tcl_AppendResult(irp, "invalid channel: ", argv[2], NULL);
@@ -467,7 +547,15 @@ static int tcl_matchexempt STDVAR
     if (u_match_mask(chan->exempts, argv[1]))
       ok = 1;
   }
-  if (u_match_mask(global_exempts, argv[1]))
+  if (argc == 4) {
+    if (!strcasecmp(argv[3], "-channel")) {
+      chanarg = 0;
+    } else {
+      Tcl_AppendResult(irp, "invalid flag", NULL);
+      return TCL_ERROR;
+    }
+  }
+  if ((u_match_mask(global_exempts, argv[1])) && chanarg)
     ok = 1;
   if (ok)
     Tcl_AppendResult(irp, "1", NULL);
@@ -479,11 +567,11 @@ static int tcl_matchexempt STDVAR
 static int tcl_matchinvite STDVAR
 {
   struct chanset_t *chan;
-  int ok = 0;
+  int chanarg = 1, ok = 0;
 
-  BADARGS(2, 3, " user!nick@host ?channel?");
+  BADARGS(2, 4, " user!nick@host ?channel? ?-channel?");
 
-  if (argc == 3) {
+  if (argc >= 3) {
     chan = findchan_by_dname(argv[2]);
     if (chan == NULL) {
       Tcl_AppendResult(irp, "invalid channel: ", argv[2], NULL);
@@ -492,7 +580,15 @@ static int tcl_matchinvite STDVAR
     if (u_match_mask(chan->invites, argv[1]))
       ok = 1;
   }
-  if (u_match_mask(global_invites, argv[1]))
+  if (argc == 4) {
+    if (!strcasecmp(argv[3], "-channel")) {
+      chanarg = 0;
+    } else {
+      Tcl_AppendResult(irp, "invalid flag", NULL);
+      return TCL_ERROR;
+    }
+  }
+  if ((u_match_mask(global_invites, argv[1])) && chanarg)
     ok = 1;
   if (ok)
     Tcl_AppendResult(irp, "1", NULL);
@@ -517,8 +613,8 @@ static int tcl_newchanban STDVAR
     return TCL_ERROR;
   }
   if (argc == 7) {
-    if (!egg_strcasecmp(argv[6], "none"));
-    else if (!egg_strcasecmp(argv[6], "sticky"))
+    if (!strcasecmp(argv[6], "none"));
+    else if (!strcasecmp(argv[6], "sticky"))
       sticky = 1;
     else {
       Tcl_AppendResult(irp, "invalid option ", argv[6], " (must be one of: ",
@@ -526,20 +622,16 @@ static int tcl_newchanban STDVAR
       return TCL_ERROR;
     }
   }
-  strncpyz(ban, argv[2], sizeof ban);
-  strncpyz(from, argv[3], sizeof from);
-  strncpyz(cmt, argv[4], sizeof cmt);
+  strlcpy(ban, argv[2], sizeof ban);
+  strlcpy(from, argv[3], sizeof from);
+  strlcpy(cmt, argv[4], sizeof cmt);
   if (argc == 5) {
     if (chan->ban_time == 0)
-      expire_time = 0L;
+      expire_time = 0;
     else
-      expire_time = now + (60 * chan->ban_time);
-  } else {
-    if (atoi(argv[5]) == 0)
-      expire_time = 0L;
-    else
-      expire_time = now + (atoi(argv[5]) * 60);
-  }
+      expire_time = now + 60 * chan->ban_time;
+  } else if ((expire_time = get_expire_time(irp, argv[5])) == -1)
+    return TCL_ERROR;
   if (u_addban(chan, ban, from, cmt, expire_time, sticky))
     if ((me = module_find("irc", 0, 0)))
       (me->funcs[IRC_CHECK_THIS_BAN]) (chan, ban, sticky);
@@ -557,8 +649,8 @@ static int tcl_newban STDVAR
   BADARGS(4, 6, " ban creator comment ?lifetime? ?options?");
 
   if (argc == 6) {
-    if (!egg_strcasecmp(argv[5], "none"));
-    else if (!egg_strcasecmp(argv[5], "sticky"))
+    if (!strcasecmp(argv[5], "none"));
+    else if (!strcasecmp(argv[5], "sticky"))
       sticky = 1;
     else {
       Tcl_AppendResult(irp, "invalid option ", argv[5], " (must be one of: ",
@@ -566,20 +658,16 @@ static int tcl_newban STDVAR
       return TCL_ERROR;
     }
   }
-  strncpyz(ban, argv[1], sizeof ban);
-  strncpyz(from, argv[2], sizeof from);
-  strncpyz(cmt, argv[3], sizeof cmt);
+  strlcpy(ban, argv[1], sizeof ban);
+  strlcpy(from, argv[2], sizeof from);
+  strlcpy(cmt, argv[3], sizeof cmt);
   if (argc == 4) {
     if (global_ban_time == 0)
-      expire_time = 0L;
+      expire_time = 0;
     else
-      expire_time = now + (60 * global_ban_time);
-  } else {
-    if (atoi(argv[4]) == 0)
-      expire_time = 0L;
-    else
-      expire_time = now + (atoi(argv[4]) * 60);
-  }
+      expire_time = now + 60 * global_ban_time;
+  } else if ((expire_time = get_expire_time(irp, argv[4])) == -1)
+    return TCL_ERROR;
   if (u_addban(NULL, ban, from, cmt, expire_time, sticky))
     if ((me = module_find("irc", 0, 0)))
       for (chan = chanset; chan != NULL; chan = chan->next)
@@ -602,8 +690,8 @@ static int tcl_newchanexempt STDVAR
     return TCL_ERROR;
   }
   if (argc == 7) {
-    if (!egg_strcasecmp(argv[6], "none"));
-    else if (!egg_strcasecmp(argv[6], "sticky"))
+    if (!strcasecmp(argv[6], "none"));
+    else if (!strcasecmp(argv[6], "sticky"))
       sticky = 1;
     else {
       Tcl_AppendResult(irp, "invalid option ", argv[6], " (must be one of: ",
@@ -611,20 +699,16 @@ static int tcl_newchanexempt STDVAR
       return TCL_ERROR;
     }
   }
-  strncpyz(exempt, argv[2], sizeof exempt);
-  strncpyz(from, argv[3], sizeof from);
-  strncpyz(cmt, argv[4], sizeof cmt);
+  strlcpy(exempt, argv[2], sizeof exempt);
+  strlcpy(from, argv[3], sizeof from);
+  strlcpy(cmt, argv[4], sizeof cmt);
   if (argc == 5) {
     if (chan->exempt_time == 0)
-      expire_time = 0L;
+      expire_time = 0;
     else
-      expire_time = now + (60 * chan->exempt_time);
-  } else {
-    if (atoi(argv[5]) == 0)
-      expire_time = 0L;
-    else
-      expire_time = now + (atoi(argv[5]) * 60);
-  }
+      expire_time = now + 60 * chan->exempt_time;
+  } else if ((expire_time = get_expire_time(irp, argv[5])) == -1)
+    return TCL_ERROR;
   if (u_addexempt(chan, exempt, from, cmt, expire_time, sticky))
     add_mode(chan, '+', 'e', exempt);
   return TCL_OK;
@@ -640,8 +724,8 @@ static int tcl_newexempt STDVAR
   BADARGS(4, 6, " exempt creator comment ?lifetime? ?options?");
 
   if (argc == 6) {
-    if (!egg_strcasecmp(argv[5], "none"));
-    else if (!egg_strcasecmp(argv[5], "sticky"))
+    if (!strcasecmp(argv[5], "none"));
+    else if (!strcasecmp(argv[5], "sticky"))
       sticky = 1;
     else {
       Tcl_AppendResult(irp, "invalid option ", argv[5], " (must be one of: ",
@@ -649,20 +733,16 @@ static int tcl_newexempt STDVAR
       return TCL_ERROR;
     }
   }
-  strncpyz(exempt, argv[1], sizeof exempt);
-  strncpyz(from, argv[2], sizeof from);
-  strncpyz(cmt, argv[3], sizeof cmt);
+  strlcpy(exempt, argv[1], sizeof exempt);
+  strlcpy(from, argv[2], sizeof from);
+  strlcpy(cmt, argv[3], sizeof cmt);
   if (argc == 4) {
     if (global_exempt_time == 0)
-      expire_time = 0L;
+      expire_time = 0;
     else
-      expire_time = now + (60 * global_exempt_time);
-  } else {
-    if (atoi(argv[4]) == 0)
-      expire_time = 0L;
-    else
-      expire_time = now + (atoi(argv[4]) * 60);
-  }
+      expire_time = now + 60 * global_exempt_time;
+  } else if ((expire_time = get_expire_time(irp, argv[4])) == -1)
+    return TCL_ERROR;
   u_addexempt(NULL, exempt, from, cmt, expire_time, sticky);
   for (chan = chanset; chan; chan = chan->next)
     add_mode(chan, '+', 'e', exempt);
@@ -684,8 +764,8 @@ static int tcl_newchaninvite STDVAR
     return TCL_ERROR;
   }
   if (argc == 7) {
-    if (!egg_strcasecmp(argv[6], "none"));
-    else if (!egg_strcasecmp(argv[6], "sticky"))
+    if (!strcasecmp(argv[6], "none"));
+    else if (!strcasecmp(argv[6], "sticky"))
       sticky = 1;
     else {
       Tcl_AppendResult(irp, "invalid option ", argv[6], " (must be one of: ",
@@ -693,20 +773,16 @@ static int tcl_newchaninvite STDVAR
       return TCL_ERROR;
     }
   }
-  strncpyz(invite, argv[2], sizeof invite);
-  strncpyz(from, argv[3], sizeof from);
-  strncpyz(cmt, argv[4], sizeof cmt);
+  strlcpy(invite, argv[2], sizeof invite);
+  strlcpy(from, argv[3], sizeof from);
+  strlcpy(cmt, argv[4], sizeof cmt);
   if (argc == 5) {
     if (chan->invite_time == 0)
-      expire_time = 0L;
+      expire_time = 0;
     else
-      expire_time = now + (60 * chan->invite_time);
-  } else {
-    if (atoi(argv[5]) == 0)
-      expire_time = 0L;
-    else
-      expire_time = now + (atoi(argv[5]) * 60);
-  }
+      expire_time = now + 60 * chan->invite_time;
+  } else if ((expire_time = get_expire_time(irp, argv[5])) == -1)
+    return TCL_ERROR;
   if (u_addinvite(chan, invite, from, cmt, expire_time, sticky))
     add_mode(chan, '+', 'I', invite);
   return TCL_OK;
@@ -722,8 +798,8 @@ static int tcl_newinvite STDVAR
   BADARGS(4, 6, " invite creator comment ?lifetime? ?options?");
 
   if (argc == 6) {
-    if (!egg_strcasecmp(argv[5], "none"));
-    else if (!egg_strcasecmp(argv[5], "sticky"))
+    if (!strcasecmp(argv[5], "none"));
+    else if (!strcasecmp(argv[5], "sticky"))
       sticky = 1;
     else {
       Tcl_AppendResult(irp, "invalid option ", argv[5], " (must be one of: ",
@@ -731,20 +807,16 @@ static int tcl_newinvite STDVAR
       return TCL_ERROR;
     }
   }
-  strncpyz(invite, argv[1], sizeof invite);
-  strncpyz(from, argv[2], sizeof from);
-  strncpyz(cmt, argv[3], sizeof cmt);
+  strlcpy(invite, argv[1], sizeof invite);
+  strlcpy(from, argv[2], sizeof from);
+  strlcpy(cmt, argv[3], sizeof cmt);
   if (argc == 4) {
     if (global_invite_time == 0)
-      expire_time = 0L;
+      expire_time = 0;
     else
-      expire_time = now + (60 * global_invite_time);
-  } else {
-    if (atoi(argv[4]) == 0)
-      expire_time = 0L;
-    else
-      expire_time = now + (atoi(argv[4]) * 60);
-  }
+      expire_time = now + 60 * global_invite_time;
+  } else if ((expire_time = get_expire_time(irp, argv[4])) == -1)
+    return TCL_ERROR;
   u_addinvite(NULL, invite, from, cmt, expire_time, sticky);
   for (chan = chanset; chan; chan = chan->next)
     add_mode(chan, '+', 'I', invite);
@@ -932,9 +1004,9 @@ static int tcl_channel_info(Tcl_Interp *irp, struct chanset_t *chan)
   return TCL_OK;
 }
 
-#define APPEND_KEYVAL(x, y) {	\
-  Tcl_AppendElement(irp, x);	\
-  Tcl_AppendElement(irp, y);	\
+#define APPEND_KEYVAL(x, y) { \
+  Tcl_AppendElement(irp, x);  \
+  Tcl_AppendElement(irp, y);  \
 }
 
 static int tcl_channel_getlist(Tcl_Interp *irp, struct chanset_t *chan)
@@ -1066,22 +1138,17 @@ static int tcl_channel_get(Tcl_Interp *irp, struct chanset_t *chan,
 
   if (!strcmp(setting, "chanmode"))
     get_mode_protect(chan, s);
-  else if (!strcmp(setting, "need-op")) {
-    strncpy(s, chan->need_op, 120);
-    s[120] = 0;
-  } else if (!strcmp(setting, "need-invite")) {
-    strncpy(s, chan->need_invite, 120);
-    s[120] = 0;
-  } else if (!strcmp(setting, "need-key")) {
-    strncpy(s, chan->need_key, 120);
-    s[120] = 0;
-  } else if (!strcmp(setting, "need-unban")) {
-    strncpy(s, chan->need_unban, 120);
-    s[120] = 0;
-  } else if (!strcmp(setting, "need-limit")) {
-    strncpy(s, chan->need_limit, 120);
-    s[120] = 0;
-  } else if (!strcmp(setting, "idle-kick"))
+  else if (!strcmp(setting, "need-op"))
+    strlcpy(s, chan->need_op, sizeof s);
+  else if (!strcmp(setting, "need-invite"))
+    strlcpy(s, chan->need_invite, sizeof s);
+  else if (!strcmp(setting, "need-key"))
+    strlcpy(s, chan->need_key, sizeof s);
+  else if (!strcmp(setting, "need-unban"))
+    strlcpy(s, chan->need_unban, sizeof s);
+  else if (!strcmp(setting, "need-limit"))
+    strlcpy(s, chan->need_limit, sizeof s);
+  else if (!strcmp(setting, "idle-kick"))
     simple_sprintf(s, "%d", chan->idle_kick);
   else if (!strcmp(setting, "stopnethack-mode") || !strcmp(setting, "stop-net-hack"))
     simple_sprintf(s, "%d", chan->stopnethack_mode);
@@ -1245,7 +1312,7 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
   int i, x = 0, found, old_status = chan->status,
       old_mode_mns_prot = chan->mode_mns_prot,
       old_mode_pls_prot = chan->mode_pls_prot;
-  struct udef_struct *ul = udef;
+  struct udef_struct *ul;
   char s[121];
   char *endptr;
   module_entry *me;
@@ -1258,8 +1325,7 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
           Tcl_AppendResult(irp, "channel need-op needs argument", NULL);
         return TCL_ERROR;
       }
-      strncpy(chan->need_op, item[i], 120);
-      chan->need_op[120] = 0;
+      strlcpy(chan->need_op, item[i], sizeof chan->need_op);
     } else if (!strcmp(item[i], "need-invite")) {
       i++;
       if (i >= items) {
@@ -1267,8 +1333,7 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
           Tcl_AppendResult(irp, "channel need-invite needs argument", NULL);
         return TCL_ERROR;
       }
-      strncpy(chan->need_invite, item[i], 120);
-      chan->need_invite[120] = 0;
+      strlcpy(chan->need_invite, item[i], sizeof chan->need_invite);
     } else if (!strcmp(item[i], "need-key")) {
       i++;
       if (i >= items) {
@@ -1276,8 +1341,7 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
           Tcl_AppendResult(irp, "channel need-key needs argument", NULL);
         return TCL_ERROR;
       }
-      strncpy(chan->need_key, item[i], 120);
-      chan->need_key[120] = 0;
+      strlcpy(chan->need_key, item[i], sizeof chan->need_key);
     } else if (!strcmp(item[i], "need-limit")) {
       i++;
       if (i >= items) {
@@ -1285,8 +1349,7 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
           Tcl_AppendResult(irp, "channel need-limit needs argument", NULL);
         return TCL_ERROR;
       }
-      strncpy(chan->need_limit, item[i], 120);
-      chan->need_limit[120] = 0;
+      strlcpy(chan->need_limit, item[i], sizeof chan->need_limit);
     } else if (!strcmp(item[i], "need-unban")) {
       i++;
       if (i >= items) {
@@ -1294,8 +1357,7 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
           Tcl_AppendResult(irp, "channel need-unban needs argument", NULL);
         return TCL_ERROR;
       }
-      strncpy(chan->need_unban, item[i], 120);
-      chan->need_unban[120] = 0;
+      strlcpy(chan->need_unban, item[i], sizeof chan->need_unban);
     } else if (!strcmp(item[i], "chanmode")) {
       i++;
       if (i >= items) {
@@ -1303,8 +1365,7 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
           Tcl_AppendResult(irp, "channel chanmode needs argument", NULL);
         return TCL_ERROR;
       }
-      strncpy(s, item[i], 120);
-      s[120] = 0;
+      strlcpy(s, item[i], sizeof s);
       set_mode_protect(chan, s);
     } else if (!strcmp(item[i], "idle-kick")) {
       i++;
@@ -1528,7 +1589,7 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
         }
       } else {
         if ((*item[i]) && !strtol(item[i], &endptr, 10) && !(*endptr)) {
-          *pthr = 0;  // Shortcut for .chanset #chan flood-x 0 to activate 0:0
+          *pthr = 0;  /* Shortcut for .chanset #chan flood-x 0 to activate 0:0 */
           *ptime = 0;
         } else {
           if (irp)
@@ -1541,8 +1602,8 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
 
       i++;
       if (i >= items) {
-		if (irp)
-		  Tcl_AppendResult(irp, item[i - 1], " needs argument", NULL);
+        if (irp)
+          Tcl_AppendResult(irp, item[i - 1], " needs argument", NULL);
         return TCL_ERROR;
       }
       p = strchr(item[i], ':');
@@ -1564,18 +1625,18 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
         initudef(UDEF_STR, item[i] + 9, 0);
       found = 0;
       for (ul = udef; ul; ul = ul->next) {
-        if (ul->type == UDEF_FLAG && (!egg_strcasecmp(item[i] + 1, ul->name) ||
+        if (ul->type == UDEF_FLAG && (!strcasecmp(item[i] + 1, ul->name) ||
             (!strncmp(item[i] + 1, "udef-flag-", 10) &&
-            !egg_strcasecmp(item[i] + 11, ul->name)))) {
+            !strcasecmp(item[i] + 11, ul->name)))) {
           if (item[i][0] == '+')
             setudef(ul, chan->dname, 1);
           else
             setudef(ul, chan->dname, 0);
           found = 1;
           break;
-        } else if (ul->type == UDEF_INT && (!egg_strcasecmp(item[i], ul->name) ||
+        } else if (ul->type == UDEF_INT && (!strcasecmp(item[i], ul->name) ||
                    (!strncmp(item[i], "udef-int-", 9) &&
-                   !egg_strcasecmp(item[i] + 9, ul->name)))) {
+                   !strcasecmp(item[i] + 9, ul->name)))) {
           i++;
           if (i >= items) {
             if (irp)
@@ -1586,9 +1647,9 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
           found = 1;
           break;
         } else if (ul->type == UDEF_STR &&
-                   (!egg_strcasecmp(item[i], ul->name) ||
+                   (!strcasecmp(item[i], ul->name) ||
                    (!strncmp(item[i], "udef-str-", 9) &&
-                   !egg_strcasecmp(item[i] + 9, ul->name)))) {
+                   !strcasecmp(item[i] + 9, ul->name)))) {
           char *val;
 
           i++;
@@ -1618,7 +1679,7 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
     }
   }
   /* If protect_readonly == 0 and chan_hack == 0 then
-   * bot is now processing the configfile, so dont do anything,
+   * bot is now processing the configfile, so don't do anything,
    * we've to wait the channelfile that maybe override these settings
    * (note: it may cause problems if there is no chanfile!)
    * <drummer/1999/10/21>
@@ -1830,7 +1891,7 @@ static int tcl_setchaninfo STDVAR
     Tcl_AppendResult(irp, "illegal channel: ", argv[2], NULL);
     return TCL_ERROR;
   }
-  if (!egg_strcasecmp(argv[3], "none")) {
+  if (!strcasecmp(argv[3], "none")) {
     set_handle_chaninfo(userlist, argv[1], argv[2], NULL);
     return TCL_OK;
   }
@@ -2100,8 +2161,7 @@ static int tcl_channel_add(Tcl_Interp *irp, char *newname, char *options)
      * any code later on. chan->name gets updated with the channel name as
      * the server knows it, when we join the channel. <cybah>
      */
-    strncpy(chan->dname, newname, 81);
-    chan->dname[80] = 0;
+    strlcpy(chan->dname, newname, sizeof chan->dname);
 
     /* Initialize chan->channel info */
     init_channel(chan, 0);
@@ -2113,7 +2173,7 @@ static int tcl_channel_add(Tcl_Interp *irp, char *newname, char *options)
 
   }
   /* If chan_hack is set, we're loading the userfile. Ignore errors while
-   * reading userfile and just return TCL_OK. This is for compatability
+   * reading userfile and just return TCL_OK. This is for compatibility
    * if a user goes back to an eggdrop that no-longer supports certain
    * (channel) options.
    */
@@ -2137,11 +2197,11 @@ static int tcl_setudef STDVAR
 
   BADARGS(3, 3, " type name");
 
-  if (!egg_strcasecmp(argv[1], "flag"))
+  if (!strcasecmp(argv[1], "flag"))
     type = UDEF_FLAG;
-  else if (!egg_strcasecmp(argv[1], "int"))
+  else if (!strcasecmp(argv[1], "int"))
     type = UDEF_INT;
-  else if (!egg_strcasecmp(argv[1], "str"))
+  else if (!strcasecmp(argv[1], "str"))
     type = UDEF_STR;
   else {
     Tcl_AppendResult(irp, "invalid type. Must be one of: flag, int, str",
@@ -2159,11 +2219,11 @@ static int tcl_renudef STDVAR
 
   BADARGS(4, 4, " type oldname newname");
 
-  if (!egg_strcasecmp(argv[1], "flag"))
+  if (!strcasecmp(argv[1], "flag"))
     type = UDEF_FLAG;
-  else if (!egg_strcasecmp(argv[1], "int"))
+  else if (!strcasecmp(argv[1], "int"))
     type = UDEF_INT;
-  else if (!egg_strcasecmp(argv[1], "str"))
+  else if (!strcasecmp(argv[1], "str"))
     type = UDEF_STR;
   else {
     Tcl_AppendResult(irp, "invalid type. Must be one of: flag, int, str",
@@ -2171,7 +2231,7 @@ static int tcl_renudef STDVAR
     return TCL_ERROR;
   }
   for (ul = udef; ul; ul = ul->next) {
-    if (ul->type == type && !egg_strcasecmp(ul->name, argv[2])) {
+    if (ul->type == type && !strcasecmp(ul->name, argv[2])) {
       nfree(ul->name);
       ul->name = nmalloc(strlen(argv[3]) + 1);
       strcpy(ul->name, argv[3]);
@@ -2192,11 +2252,11 @@ static int tcl_deludef STDVAR
 
   BADARGS(3, 3, " type name");
 
-  if (!egg_strcasecmp(argv[1], "flag"))
+  if (!strcasecmp(argv[1], "flag"))
     type = UDEF_FLAG;
-  else if (!egg_strcasecmp(argv[1], "int"))
+  else if (!strcasecmp(argv[1], "int"))
     type = UDEF_INT;
-  else if (!egg_strcasecmp(argv[1], "str"))
+  else if (!strcasecmp(argv[1], "str"))
     type = UDEF_STR;
   else {
     Tcl_AppendResult(irp, "invalid type. Must be one of: flag, int, str",
@@ -2207,7 +2267,7 @@ static int tcl_deludef STDVAR
     ull = ul->next;
     if (!ull)
       break;
-    if (ull->type == type && !egg_strcasecmp(ull->name, argv[2])) {
+    if (ull->type == type && !strcasecmp(ull->name, argv[2])) {
       ul->next = ull->next;
       nfree(ull->name);
       free_udef_chans(ull->values, ull->type);
@@ -2216,7 +2276,7 @@ static int tcl_deludef STDVAR
     }
   }
   if (udef) {
-    if (udef->type == type && !egg_strcasecmp(udef->name, argv[2])) {
+    if (udef->type == type && !strcasecmp(udef->name, argv[2])) {
       ul = udef->next;
       nfree(udef->name);
       free_udef_chans(udef->values, udef->type);
@@ -2240,11 +2300,11 @@ static int tcl_getudefs STDVAR
   BADARGS(1, 2, " ?type?");
 
   if (argc > 1) {
-    if (!egg_strcasecmp(argv[1], "flag"))
+    if (!strcasecmp(argv[1], "flag"))
       type = UDEF_FLAG;
-    else if (!egg_strcasecmp(argv[1], "int"))
+    else if (!strcasecmp(argv[1], "int"))
       type = UDEF_INT;
-    else if (!egg_strcasecmp(argv[1], "str"))
+    else if (!strcasecmp(argv[1], "str"))
       type = UDEF_STR;
     else {
       Tcl_AppendResult(irp, "invalid type. Valid types are: flag, int, str",
@@ -2282,7 +2342,6 @@ static int tcl_chansettype STDVAR
              !strcmp(argv[1], "flood-kick") ||
              !strcmp(argv[1], "flood-deop") ||
              !strcmp(argv[1], "flood-nick") ||
-             !strcmp(argv[1], "flood-nick") ||
              !strcmp(argv[1], "aop-delay")) {
     Tcl_AppendResult(irp, "pair", NULL);
   /* Integers now */
