@@ -5,7 +5,7 @@
  * Written by Fabian Knittel <fknittel@gmx.de>
  */
 /*
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,9 +22,11 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#include "src/mod/module.h"
+
+#ifndef EGG_TDNS
 #define MODULE_NAME "dns"
 
-#include "src/mod/module.h"
 #include "dns.h"
 
 static void dns_event_success(struct resolve *rp, int type);
@@ -37,7 +39,7 @@ static int dns_retrydelay = 3;
 static int dns_cache = 86400;
 static int dns_negcache = 600;
 
-static char dns_servers[121] = "";
+static char dns_servers[144] = "";
 
 #include "coredns.c"
 
@@ -131,7 +133,7 @@ static tcl_ints dnsints[] = {
 };
 
 static tcl_strings dnsstrings[] = {
-  {"dns-servers", dns_servers, 120,           0},
+  {"dns-servers", dns_servers, 143,           0},
   {NULL,          NULL,          0,           0}
 };
 
@@ -139,7 +141,7 @@ static char *dns_change(ClientData cdata, Tcl_Interp *irp,
                            EGG_CONST char *name1,
                            EGG_CONST char *name2, int flags)
 {
-  char buf[121], *p;
+  char buf[sizeof dns_servers], *p;
   unsigned short port;
   int i, lc, code;
   EGG_CONST char **list, *slist;
@@ -148,9 +150,9 @@ static char *dns_change(ClientData cdata, Tcl_Interp *irp,
     Tcl_DString ds;
 
     Tcl_DStringInit(&ds);
-    for (i = 0; i < _res.nscount; i++) {
+    for (i = 0; i < myres.nscount; i++) {
       egg_snprintf(buf, sizeof buf, "%s:%d", iptostr((struct sockaddr *)
-               &_res.nsaddr_list[i]), ntohs(_res.nsaddr_list[i].sin_port));
+               &myres.nsaddr_list[i]), ntohs(myres.nsaddr_list[i].sin_port));
       Tcl_DStringAppendElement(&ds, buf);
     }
     slist = Tcl_DStringValue(&ds);
@@ -162,8 +164,13 @@ static char *dns_change(ClientData cdata, Tcl_Interp *irp,
     if (code == TCL_ERROR)
       return "variable must be a list";
     /* reinitialize the list */
-    _res.nscount = 0;
+    myres.nscount = 0;
     for (i = 0; i < lc; i++) {
+      if (myres.nscount >= MAXNS) {
+        putlog(LOG_MISC, "*", "WARNING: %i dns-servers configured but kernel-defined "
+               "limit is %i, ignoring extra servers\n", lc, MAXNS);
+        break;
+      }
       if ((p = strchr(list[i], ':'))) {
         *p++ = 0;
         /* allow non-standard ports */
@@ -171,11 +178,13 @@ static char *dns_change(ClientData cdata, Tcl_Interp *irp,
       } else
         port = NAMESERVER_PORT; /* port 53 */
       /* Ignore invalid addresses */
-      if (egg_inet_aton(list[i], &_res.nsaddr_list[_res.nscount].sin_addr)) {
-        _res.nsaddr_list[_res.nscount].sin_port = htons(port);
-        _res.nsaddr_list[_res.nscount].sin_family = AF_INET;
-        _res.nscount++;
+      if (egg_inet_aton(list[i], &myres.nsaddr_list[myres.nscount].sin_addr)) {
+        myres.nsaddr_list[myres.nscount].sin_port = htons(port);
+        myres.nsaddr_list[myres.nscount].sin_family = AF_INET;
+        myres.nscount++;
       }
+      else
+        putlog(LOG_MISC, "*", "WARNING: Invalid dns-server %s", list[i]);
     }
     Tcl_Free((char *) list);
   }
@@ -225,10 +234,10 @@ static int dns_report(int idx, int details)
 
     dprintf(idx, "    Async DNS resolver is active.\n");
     dprintf(idx, "    DNS server list:");
-    for (i = 0; i < _res.nscount; i++)
-      dprintf(idx, " %s:%d", iptostr((struct sockaddr *) &_res.nsaddr_list[i]),
-              ntohs(_res.nsaddr_list[i].sin_port));
-    if (!_res.nscount)
+    for (i = 0; i < myres.nscount; i++)
+      dprintf(idx, " %s:%d", iptostr((struct sockaddr *) &myres.nsaddr_list[i]),
+              ntohs(myres.nsaddr_list[i].sin_port));
+    if (!myres.nscount)
       dprintf(idx, " NO DNS SERVERS FOUND!\n");
     dprintf(idx, "\n");
     dprintf(idx, "    Using %d byte%s of memory\n", size,
@@ -240,10 +249,10 @@ static int dns_report(int idx, int details)
 static int dns_check_servercount(void)
 {
   static int oldcount = -1;
-  if (oldcount != _res.nscount && !_res.nscount) {
+  if (oldcount != myres.nscount && !myres.nscount) {
     putlog(LOG_MISC, "*", "WARNING: No nameservers found. Please set the dns-servers config variable.");
   }
-  oldcount = _res.nscount;
+  oldcount = myres.nscount;
   return 0;
 }
 
@@ -284,14 +293,18 @@ static Function dns_table[] = {
   (Function) dns_report,
   /* 4 - 7 */
 };
+#endif /* EGG_TDNS */
 
 char *dns_start(Function *global_funcs)
 {
+#ifdef EGG_TDNS
+  return "Eggdrop was compiled with threaded DNS core; this module will not run with it. Not loading...";
+#else
   int idx;
 
   global = global_funcs;
 
-  module_register(MODULE_NAME, dns_table, 1, 1);
+  module_register(MODULE_NAME, dns_table, 1, 2);
   if (!module_depend(MODULE_NAME, "eggdrop", 108, 0)) {
     module_undepend(MODULE_NAME);
     return "This module requires Eggdrop 1.8.0 or later.";
@@ -307,9 +320,9 @@ char *dns_start(Function *global_funcs)
   dcc[idx].sock = resfd;
   dcc[idx].timeval = now;
   strcpy(dcc[idx].nick, "(dns)");
-  egg_memcpy(&dcc[idx].sockname.addr.sa, &_res.nsaddr_list[0],
-             sizeof(_res.nsaddr_list[0]));
-  dcc[idx].sockname.addrlen = sizeof(_res.nsaddr_list[0]);
+  memcpy(&dcc[idx].sockname.addr.sa, &myres.nsaddr_list[0],
+             sizeof(myres.nsaddr_list[0]));
+  dcc[idx].sockname.addrlen = sizeof(myres.nsaddr_list[0]);
 
   Tcl_TraceVar(interp, "dns-servers",
                TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
@@ -321,4 +334,5 @@ char *dns_start(Function *global_funcs)
   add_tcl_ints(dnsints);
   add_tcl_strings(dnsstrings);
   return NULL;
+#endif /* EGG_TDNS */
 }
