@@ -3,11 +3,9 @@
  *   header file for filelist.c
  *
  * Written by Fabian Knittel <fknittel@gmx.de>
- *
- * $Id: filelist.h,v 1.15 2011/02/13 14:19:33 simple Exp $
  */
 /*
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,8 +41,8 @@ typedef struct {
 static filelist_t *filelist_new(void);
 static void filelist_free(filelist_t *);
 static void filelist_add(filelist_t *, char *);
-static inline void filelist_addout(filelist_t *, char *);
-static inline void filelist_idxshow(filelist_t *, int);
+static void filelist_addout(filelist_t *, char *);
+static void filelist_idxshow(filelist_t *, int);
 static void filelist_sort(filelist_t *);
 
 #endif /* _EGG_MOD_FILESYS_FILELIST_H */
