@@ -4,7 +4,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -74,7 +74,7 @@ static char *getnick(char *handle, struct chanset_t *chan)
 
   for (m = chan->channel.member; m && m->nick[0]; m = m->next) {
     egg_snprintf(s, sizeof s, "%s!%s", m->nick, m->userhost);
-    if ((u = get_user_by_host(s)) && !egg_strcasecmp(u->handle, handle))
+    if ((u = get_user_by_host(s)) && !strcasecmp(u->handle, handle))
       return m->nick;
   }
   return NULL;
@@ -724,9 +724,21 @@ static void cmd_channel(struct userrec *u, int idx, char *par)
 {
   char handle[HANDLEN + 1], s[UHOSTLEN], s1[UHOSTLEN], atrflag, chanflag;
   struct chanset_t *chan;
+  struct capability *current;
   memberlist *m;
-  int maxnicklen, maxhandlen;
-  char format[81];
+  int maxnicklen, maxhandlen, extjoin = 0, acctnotify = 0;
+
+  /* Check if CAPs are enabled */
+  current = cap;
+  while (current != NULL) {
+    if (!strcasecmp("extended-join", current->name)) {
+      extjoin = current->enabled ? 1 : 0;
+    }
+    if (!strcasecmp("account-notify", current->name)) {
+      acctnotify = current->enabled ? 1 : 0;
+    }
+    current = current->next;
+  }
 
   chan = get_channel(idx, par);
   if (!chan || !has_oporhalfop(idx, chan))
@@ -758,15 +770,20 @@ static void cmd_channel(struct userrec *u, int idx, char *par)
       maxhandlen = 9;
 
     dprintf(idx, "(n = owner, m = master, o = op, d = deop, b = bot)\n");
-    egg_snprintf(format, sizeof format, " %%-%us %%-%us %%-6s %%-5s %%s\n",
-                 maxnicklen, maxhandlen);
-    dprintf(idx, format, "NICKNAME", "HANDLE", " JOIN", "IDLE", "USER@HOST");
+    if (use_354 && extjoin && acctnotify) {
+      dprintf(idx, " %-*s %-*s %-*s  %-6s %-5s %s\n", maxnicklen, "NICKNAME",
+                maxhandlen, "HANDLE", maxnicklen, "ACCOUNT", "JOIN", "IDLE",
+                "USER@HOST");
+    } else {
+      dprintf(idx, " %-*s %-*s  %-6s %-5s %s\n", maxnicklen, "NICKNAME",
+                maxhandlen, "HANDLE", "JOIN", "IDLE", "USER@HOST");
+    }
     for (m = chan->channel.member; m && m->nick[0]; m = m->next) {
       if (m->joined > 0) {
         if ((now - (m->joined)) > 86400)
-          egg_strftime(s, 6, "%d%b", localtime(&(m->joined)));
+          strftime(s, 6, "%d%b", localtime(&(m->joined)));
         else
-          egg_strftime(s, 6, "%H:%M", localtime(&(m->joined)));
+          strftime(s, 6, "%H:%M", localtime(&(m->joined)));
       } else
         strlcpy(s, " --- ", sizeof s);
       if (m->user == NULL) {
@@ -854,16 +871,24 @@ static void cmd_channel(struct userrec *u, int idx, char *par)
       else
         chanflag = ' ';
       if (chan_issplit(m)) {
-        egg_snprintf(format, sizeof format,
-                     "%%c%%-%us %%-%us %%s %%c     <- netsplit, %%lus\n",
-                     maxnicklen, maxhandlen);
-        dprintf(idx, format, chanflag, m->nick, handle, s, atrflag,
-                now - (m->split));
+        if (use_354 && extjoin && acctnotify) {
+          dprintf(idx, "%c%-*s %-*s %-*s %-6s %-5s <- netsplit, %lus\n",
+                chanflag, maxnicklen, m->nick, maxhandlen, handle, maxnicklen,
+                m->account, s, atrflag, now- (m->split));
+        } else {
+          dprintf(idx, "%c%-*s %-*s %-6s %-5s <- netsplit, %lus\n", chanflag,
+                maxnicklen, m->nick, maxhandlen, handle, s, atrflag,
+                now- (m->split));
+        }
       } else if (!rfc_casecmp(m->nick, botname)) {
-        egg_snprintf(format, sizeof format,
-                     "%%c%%-%us %%-%us %%s %%c     <- it's me!\n",
-                     maxnicklen, maxhandlen);
-        dprintf(idx, format, chanflag, m->nick, handle, s, atrflag);
+        if (use_354 && extjoin && acctnotify) {
+          dprintf(idx, "%c%-*s %-*s %-*s %-6s %c     <- it's me!\n", chanflag,
+                maxnicklen, m->nick, maxhandlen, handle, maxnicklen, m->account,
+                s, atrflag);
+        } else {
+          dprintf(idx, "%c%-*s %-*s %-6s %c     <- it's me!\n", chanflag,
+                maxnicklen, m->nick, maxhandlen, handle, s, atrflag);
+        }
       } else {
         /* Determine idle time */
         if (now - (m->last) > 86400)
@@ -874,11 +899,19 @@ static void cmd_channel(struct userrec *u, int idx, char *par)
           egg_snprintf(s1, sizeof s1, "%2lum", ((now - (m->last)) / 60));
         else
           strlcpy(s1, "   ", sizeof s1);
-        egg_snprintf(format, sizeof format,
-                     "%%c%%-%us %%-%us %%s %%c %%s  %%s\n", maxnicklen,
-                     maxhandlen);
-        dprintf(idx, format, chanflag, m->nick, handle, s, atrflag, s1,
-                m->userhost);
+        if (chan_ircaway(m)) {
+          egg_snprintf(s1+strlen(s1), ((sizeof s1)-strlen(s1)), " (away)");
+        } else {
+          egg_snprintf(s1+strlen(s1), ((sizeof s1)-strlen(s1)), "       ");
+        }
+        if (use_354 && extjoin && acctnotify) {
+          dprintf(idx, "%c%-*s %-*s %-*s %-6s %c %s  %s\n", chanflag, maxnicklen,
+                m->nick, maxhandlen, handle, maxnicklen, m->account, s, atrflag,
+                s1, m->userhost);
+        } else {
+          dprintf(idx, "%c%-*s %-*s %-6s %c %-5s  %s\n", chanflag, maxnicklen,
+                m->nick, maxhandlen, handle, s, atrflag, s1, m->userhost);
+        }
       }
       if (chan_fakeop(m))
         dprintf(idx, "    (%s)\n", IRC_FAKECHANOP);
@@ -1030,7 +1063,7 @@ static void cmd_adduser(struct userrec *u, int idx, char *par)
   }
   u = get_user_by_handle(userlist, hand);
   if (u && (u->flags & (USER_OWNER | USER_MASTER)) &&
-      !(atr & USER_OWNER) && egg_strcasecmp(dcc[idx].nick, hand)) {
+      !(atr & USER_OWNER) && strcasecmp(dcc[idx].nick, hand)) {
     dprintf(idx, "You can't add hostmasks to the bot owner/master.\n");
     return;
   }
@@ -1040,12 +1073,7 @@ static void cmd_adduser(struct userrec *u, int idx, char *par)
     strlcpy(s1, s, sizeof s1);
     p1 = strchr(s1, '!');
     if (strchr("~^+=-", p1[1])) {
-      if (strict_host)
-        p1[1] = '?';
-      else {
-        p1[1] = '!';
-        p1++;
-      }
+      p1[1] = '?';
     }
     p1--;
     p1[0] = '*';
@@ -1100,7 +1128,7 @@ static void cmd_deluser(struct userrec *u, int idx, char *par)
    * so deluser on a channel they're not on should work
    */
   /* Shouldn't allow people to remove permanent owners (guppy 9Jan1999) */
-  if ((glob_owner(victim) && egg_strcasecmp(dcc[idx].nick, nick)) ||
+  if ((glob_owner(victim) && strcasecmp(dcc[idx].nick, nick)) ||
       isowner(u->handle)) {
     dprintf(idx, "You can't remove a bot owner!\n");
   } else if (glob_botmast(victim) && !glob_owner(user)) {
@@ -1141,7 +1169,7 @@ static void cmd_reset(struct userrec *u, int idx, char *par)
       else {
         putlog(LOG_CMDS, "*", "#%s# reset %s", dcc[idx].nick, par);
         dprintf(idx, "Resetting channel info for %s...\n", chan->dname);
-        reset_chan_info(chan, CHAN_RESETALL);
+        reset_chan_info(chan, CHAN_RESETALL, 1);
       }
     }
   } else if (!(u->flags & USER_MASTER))
@@ -1151,7 +1179,7 @@ static void cmd_reset(struct userrec *u, int idx, char *par)
     dprintf(idx, "Resetting channel info for all channels...\n");
     for (chan = chanset; chan; chan = chan->next) {
       if (channel_active(chan))
-        reset_chan_info(chan, CHAN_RESETALL);
+        reset_chan_info(chan, CHAN_RESETALL, 1);
     }
   }
 }
