@@ -3,7 +3,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -69,6 +69,81 @@ static int tcl_chanlist STDVAR
   return TCL_OK;
 }
 
+static int tcl_monitor STDVAR
+{
+  Tcl_Obj *monitorlist;
+  int ret;
+  BADARGS(2, 3, " command ?nick?");
+
+  monitorlist = Tcl_NewListObj(0, NULL);
+  if (!strcmp(argv[1], "add")) {
+    if (argc == 3) {
+      ret = monitor_add(argv[2], 1);
+      if (!ret) {
+        Tcl_AppendResult(irp, "1", NULL);
+        return TCL_OK;
+      } else if (ret == 1) {
+        Tcl_AppendResult(irp, "nickname already present in monitor list", NULL);
+        return TCL_ERROR;
+        /* ret = 2 */
+      } else {
+        Tcl_AppendResult(irp,
+                "maximum number of nicknames allowed by server reached", NULL);
+        return TCL_ERROR;
+      }
+    } else {
+      Tcl_AppendResult(irp, "nickname required", NULL);
+      return TCL_ERROR;
+    }
+  } else if (!strcmp(argv[1], "delete")) {
+    if (argc == 3) {
+      ret = monitor_del(argv[2]);
+      if (ret) {
+        Tcl_AppendResult(irp, "nickname not found", NULL);
+        return TCL_ERROR;
+      } else {
+        Tcl_AppendResult(irp, "1", NULL);
+        return TCL_OK;
+      }
+    } else {
+      Tcl_AppendResult(irp, "nickname required", NULL);
+      return TCL_ERROR;
+    }
+  } else if (!strcmp(argv[1], "list")) {
+    monitor_show(monitorlist, 0, NULL);
+    Tcl_AppendResult(irp, Tcl_GetString(monitorlist), NULL);
+    return TCL_OK;
+  } else if (!strcmp(argv[1], "online")) {
+    monitor_show(monitorlist, 1, NULL);
+    Tcl_AppendResult(irp, Tcl_GetString(monitorlist), NULL);
+    return TCL_OK;
+  } else if (!strcmp(argv[1], "offline")) {
+    monitor_show(monitorlist, 2, NULL);
+    Tcl_AppendResult(irp, Tcl_GetString(monitorlist), NULL);
+    return TCL_OK;
+  } else if (!strcmp(argv[1], "status")) {
+    if (argc < 3) {
+      Tcl_AppendResult(irp, "nickname required", NULL);
+      return TCL_OK;
+    }
+    ret = monitor_show(monitorlist, 3, argv[2]);
+    if (!ret) {
+      Tcl_AppendResult(irp, Tcl_GetString(monitorlist), NULL);
+      return TCL_OK;
+    } else {
+      Tcl_AppendResult(irp, "nickname not found", NULL);
+      return TCL_ERROR;
+    }
+  } else if (!strcasecmp(argv[1], "clear")) {
+    monitor_clear();
+    Tcl_AppendResult(irp, "MONITOR list cleared.", NULL);
+    return TCL_OK;
+  } else {
+    Tcl_AppendResult(irp, "command must be add, delete, list, clear, online, offline, status", NULL);
+    return TCL_ERROR;
+  }
+}
+
 static int tcl_botisop STDVAR
 {
   struct chanset_t *chan, *thechan = NULL;
@@ -434,6 +509,70 @@ static int tcl_ischaninvite STDVAR
   return TCL_OK;
 }
 
+/* Checks internal tracking of IRC server away status, as updated by IRC
+ * server 352 and AWAY messages. Meant mostly for use with the IRCv3 away-notify
+ * capability, it may not be accurate using only 352s.
+ */
+static int tcl_isaway STDVAR
+{
+  struct chanset_t *chan, *thechan = NULL;
+  memberlist *m;
+
+  BADARGS(2, 3, " nick ?channel?");
+
+  if (argc > 2) { /* If channel specified, does it exist? */
+    chan = findchan_by_dname(argv[2]);
+    thechan = chan;
+    if (!thechan) {
+      Tcl_AppendResult(irp, "illegal channel: ", argv[2], NULL);
+      return TCL_ERROR;
+    }
+  } else {
+    chan = chanset;
+  }
+  while (chan && (thechan == NULL || thechan == chan)) {
+    if ((m = ismember(chan, argv[1])) && chan_ircaway(m)) {
+      Tcl_AppendResult(irp, "1", NULL);
+      return TCL_OK;
+    }
+    chan = chan->next;
+  }
+  Tcl_AppendResult(irp, "0", NULL);
+  return TCL_OK;
+}
+
+/* Checks if user is registered with the server as a bot. This requires the
+ * server to have the 005 BOT feature advertised, and the user to have set
+ * the corresponding usermode.
+ */
+static int tcl_isircbot STDVAR
+{
+  struct chanset_t *chan, *thechan = NULL;
+  memberlist *m;
+
+  BADARGS(2, 3, " nick ?channel?");
+
+  if (argc > 2) { /* If channel specified, does it exist? */
+    chan = findchan_by_dname(argv[2]);
+    thechan = chan;
+    if (!thechan) {
+      Tcl_AppendResult(irp, "illegal channel: ", argv[2], NULL);
+      return TCL_ERROR;
+    }
+  } else {
+    chan = chanset;
+  }
+  while (chan && (thechan == NULL || thechan == chan)) {
+    if ((m = ismember(chan, argv[1])) && chan_ircbot(m)) {
+      Tcl_AppendResult(irp, "1", NULL);
+      return TCL_OK;
+    }
+    chan = chan->next;
+  }
+  Tcl_AppendResult(irp, "0", NULL);
+  return TCL_OK;
+}
+
 static int tcl_getchanhost STDVAR
 {
   struct chanset_t *chan, *thechan = NULL;
@@ -532,7 +671,7 @@ static int tcl_getchanidle STDVAR
   return TCL_OK;
 }
 
-static inline int tcl_chanmasks(masklist *m, Tcl_Interp *irp)
+static int tcl_chanmasks(masklist *m, Tcl_Interp *irp)
 {
   char work[20], *p;
   EGG_CONST char *list[3];
@@ -802,9 +941,38 @@ static int tcl_resetchanjoin STDVAR
   return TCL_OK;
 }
 
-static int tcl_resetchan STDVAR
-{
+static int setflags(int *flags, char *argflags) {
   char *c;
+
+  for (c = argflags; *c; c++) {
+    switch(*c) {
+    case 'w':
+      *flags |= CHAN_RESETWHO;
+      break;
+    case 'm':
+      *flags |= CHAN_RESETMODES;
+      break;
+    case 'b':
+      *flags |= CHAN_RESETBANS;
+      break;
+    case 'e':
+      *flags |= CHAN_RESETEXEMPTS;
+      break;
+    case 'I':
+      *flags |= CHAN_RESETINVITED;
+      break;
+    case 't':
+      *flags |= CHAN_RESETTOPIC;
+      break;
+    default:
+      return 1; /* Found a flag we don't support, return an error */
+    }
+  }
+  return 0;
+}
+
+static int tcl_refreshchan STDVAR
+{
   int flags = 0;
   struct chanset_t *chan;
 
@@ -817,36 +985,41 @@ static int tcl_resetchan STDVAR
   }
 
   if (argc == 2) {
-    reset_chan_info(chan, CHAN_RESETALL);
+    reset_chan_info(chan, CHAN_RESETALL, 0);
     return TCL_OK;
   }
-  for (c = argv[2]; *c; c++) {
-    switch(*c) {
-    case 'w':
-      flags |= CHAN_RESETWHO;
-      break;
-    case 'm':
-      flags |= CHAN_RESETMODES;
-      break;
-    case 'b':
-      flags |= CHAN_RESETBANS;
-      break;
-    case 'e':
-      flags |= CHAN_RESETEXEMPTS;
-      break;
-    case 'I':
-      flags |= CHAN_RESETINVITED;
-      break;
-    case 't':
-      flags |= CHAN_RESETTOPIC;
-      break;
-    default:
+  if (setflags(&flags, argv[2])) {       /* Set flags to refresh */
+      Tcl_AppendResult(irp, "invalid refresh flags: ", argv[2], NULL);
+      return TCL_ERROR;
+  } else {
+    reset_chan_info(chan, flags, 0);
+  }
+  return TCL_OK;
+}
+
+static int tcl_resetchan STDVAR
+{
+  int flags = 0;
+  struct chanset_t *chan;
+
+  BADARGS(2, 3, " channel ?flags?");
+
+  chan = findchan_by_dname(argv[1]);
+  if (chan == NULL) {
+    Tcl_AppendResult(irp, "invalid channel ", argv[1], NULL);
+    return TCL_ERROR;
+  }
+
+  if (argc == 2) {
+    reset_chan_info(chan, CHAN_RESETALL, 1);
+    return TCL_OK;
+  }
+  if (setflags(&flags, argv[2])) {       /* Set flags to refresh */
       Tcl_AppendResult(irp, "invalid reset flags: ", argv[2], NULL);
       return TCL_ERROR;
-    }
+  } else {
+    reset_chan_info(chan, flags, 1);
   }
-  if (flags)
-    reset_chan_info(chan, flags);
   return TCL_OK;
 }
 
@@ -865,6 +1038,102 @@ static int tcl_topic STDVAR
   return TCL_OK;
 }
 
+static int tcl_account2nicks STDVAR
+{
+  memberlist *m;
+  struct chanset_t *chan, *thechan = NULL;
+  Tcl_Obj *nicks;
+  Tcl_Obj **nicksv = NULL;
+  int nicksc = 0, i, found;
+
+  BADARGS(2, 3, " account ?channel?");
+
+  if (argc > 2) {
+    chan = findchan_by_dname(argv[2]);
+    thechan = chan;
+    if (chan == NULL) {
+      Tcl_AppendResult(irp, "invalid channel: ", argv[2], NULL);
+      return TCL_ERROR;
+    }
+  } else
+    chan = chanset;
+
+  nicks = Tcl_NewListObj(0, NULL);
+  while (chan && (thechan == NULL || thechan == chan)) {
+    for (m = chan->channel.member; m && m->nick[0]; m = m->next) {
+      found = 0;
+      /* Does this user have the account we're looking for? */
+      if (!rfc_casecmp(m->account, argv[1])) {
+        /* Is the nick of the user already in the list? */
+        Tcl_ListObjGetElements(irp, nicks, &nicksc, &nicksv);
+        for (i = 0; i < nicksc; i++) {
+          if (!rfc_casecmp(m->nick, Tcl_GetString(nicksv[i]))) {
+            found = 1;
+            break;
+          }
+        }
+        if (!found) {
+          Tcl_ListObjAppendElement(irp, nicks, Tcl_NewStringObj(m->nick, -1));
+        }
+      }
+    }
+    chan = chan->next;
+  }
+  Tcl_SetObjResult(irp, nicks);
+  return TCL_OK;
+}
+
+static int tcl_hand2nicks STDVAR
+{
+  char nuh[1024];
+  memberlist *m;
+  struct chanset_t *chan, *thechan = NULL;
+  Tcl_Obj *nicks;
+  Tcl_Obj **nicksv = NULL;
+  int nicksc = 0, i, found;
+
+  BADARGS(2, 3, " handle ?channel?");
+
+  if (argc > 2) {
+    chan = findchan_by_dname(argv[2]);
+    thechan = chan;
+    if (chan == NULL) {
+      Tcl_AppendResult(irp, "invalid channel: ", argv[2], NULL);
+      return TCL_ERROR;
+    }
+  } else
+    chan = chanset;
+
+  nicks = Tcl_NewListObj(0, NULL);
+  while (chan && (thechan == NULL || thechan == chan)) {
+    for (m = chan->channel.member; m && m->nick[0]; m = m->next) {
+      found = 0;
+      /* Does this user have the account we're looking for? */
+      if (!m->user && !m->tried_getuser) {
+        egg_snprintf(nuh, sizeof nuh, "%s!%s", m->nick, m->userhost);
+        m->tried_getuser = 1;
+        m->user = get_user_by_host(nuh);
+      }
+      if (m->user && !rfc_casecmp(m->user->handle, argv[1])) {
+        /* Is the nick of the user already in the list? */
+        Tcl_ListObjGetElements(irp, nicks, &nicksc, &nicksv);
+        for (i = 0; i < nicksc; i++) {
+          if (!rfc_casecmp(m->nick, Tcl_GetString(nicksv[i]))) {
+            found = 1;
+            break;
+          }
+        }
+        if (!found) {
+          Tcl_ListObjAppendElement(irp, nicks, Tcl_NewStringObj(m->nick, -1));
+        }
+      }
+    }
+    chan = chan->next;
+  }
+  Tcl_SetObjResult(irp, nicks);
+  return TCL_OK;
+}
+
 static int tcl_hand2nick STDVAR
 {
   char nuh[1024];
@@ -973,7 +1242,7 @@ static int tcl_putkick STDVAR
       return TCL_ERROR;
     }
     if (!m)
-      continue;                 /* Skip non-existant nicks */
+      continue;                 /* Skip non-existent nicks */
     m->flags |= SENTKICK;       /* Mark as pending kick */
     if (kicknick[0])
       strncat(kicknick, ",", sizeof kicknick - strlen(kicknick) - 1);
@@ -1014,9 +1283,13 @@ static tcl_cmds tclchan_cmds[] = {
   {"onchansplit",    tcl_onchansplit},
   {"maskhost",       tcl_maskhost},
   {"getchanidle",    tcl_getchanidle},
+  {"isaway",         tcl_isaway},
+  {"isircbot",       tcl_isircbot},
   {"chanbans",       tcl_chanbans},
   {"chanexempts",    tcl_chanexempts},
   {"chaninvites",    tcl_chaninvites},
+  {"account2nicks",  tcl_account2nicks},
+  {"hand2nicks",     tcl_hand2nicks},
   {"hand2nick",      tcl_hand2nick},
   {"nick2hand",      tcl_nick2hand},
   {"getchanmode",    tcl_getchanmode},
@@ -1029,10 +1302,12 @@ static tcl_cmds tclchan_cmds[] = {
   {"resetchanidle",  tcl_resetchanidle},
   {"resetchanjoin",  tcl_resetchanjoin},
   {"resetchan",      tcl_resetchan},
+  {"refreshchan",    tcl_refreshchan},
   {"topic",          tcl_topic},
   {"botonchan",      tcl_botonchan},
   {"putkick",        tcl_putkick},
   {"channame2dname", tcl_channame2dname},
   {"chandname2name", tcl_chandname2name},
+  {"monitor",        tcl_monitor},
   {NULL,             NULL}
 };
