@@ -1,11 +1,9 @@
 /*
  * modvals.h
- *
- * $Id: modvals.h,v 1.33 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -53,6 +51,8 @@
 #define HOOK_DNS_IPBYHOST       113
 #define HOOK_ENCRYPT_STRING     114
 #define HOOK_DECRYPT_STRING     115
+#define HOOK_ENCRYPT_PASS2      116
+#define HOOK_VERIFY_PASS2       117
 
 /* These are FIXED once they are in a release they STAY */
 #define MODCALL_START             0
@@ -79,6 +79,7 @@
 #define IRC_DO_CHANNEL_PART       19
 #define IRC_CHECK_THIS_BAN        20
 #define IRC_CHECK_THIS_USER       21
+#define IRC_RESET_CHAN_INFO       25
 /* Notes */
 #define NOTES_CMD_NOTE            4
 /* Console */
