@@ -7,7 +7,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -94,7 +94,7 @@ static int num_notes(char *user)
     if ((s[0]) && (s[0] != '#') && (s[0] != ';')) {   /* Not comment */
       s1 = s;
       to = newsplit(&s1);
-      if (!egg_strcasecmp(to, user))
+      if (!strcasecmp(to, user))
         tot++;
     }
   }
@@ -114,7 +114,7 @@ static void notes_change(char *oldnick, char *newnick)
   char s[513], *to, *s1;
   int tot = 0;
 
-  if (!egg_strcasecmp(oldnick, newnick))
+  if (!strcasecmp(oldnick, newnick))
     return;
   if (!notefile[0])
     return;
@@ -136,7 +136,7 @@ static void notes_change(char *oldnick, char *newnick)
     if ((s[0]) && (s[0] != '#') && (s[0] != ';')) {   /* Not comment */
       s1 = s;
       to = newsplit(&s1);
-      if (!egg_strcasecmp(to, oldnick)) {
+      if (!strcasecmp(to, oldnick)) {
         tot++;
         fprintf(g, "%s %s\n", newnick, s1);
       } else
@@ -229,9 +229,9 @@ static int tcl_storenote STDVAR
     /* User is valid & has a valid forwarding address */
      strlcpy(fwd, f1, sizeof fwd); /* Only 40 bytes are stored in the userfile */
      p = strchr(fwd, '@');
-    if (p && !egg_strcasecmp(p + 1, botnetnick)) {
+    if (p && !strcasecmp(p + 1, botnetnick)) {
       *p = 0;
-      if (!egg_strcasecmp(fwd, argv[2]))
+      if (!strcasecmp(fwd, argv[2]))
         /* They're forwarding to themselves on the same bot, llama's */
         ok = 0;
       strcpy(fwd2, fwd);
@@ -243,7 +243,7 @@ static int tcl_storenote STDVAR
       if ((f2 = get_user(&USERENTRY_FWD, ur2))) {
         strcpy(fwd2, f2);
         splitc(fwd2, fwd2, '@');
-        if (!egg_strcasecmp(fwd2, argv[2]))
+        if (!strcasecmp(fwd2, argv[2]))
           /* They're forwarding to someone who forwards back to them! */
           ok = 0;
       }
@@ -260,7 +260,7 @@ static int tcl_storenote STDVAR
         q++;
         if ((r = strchr(q, ' '))) {
           *r = 0;
-          if (!egg_strcasecmp(fwd, q))
+          if (!strcasecmp(fwd, q))
             ok = 0;
           *r = ' ';
         }
@@ -412,11 +412,11 @@ static int tcl_erasenotes STDVAR
   while (!feof(f) && fgets(s, sizeof s, f) != NULL) {
     if (s[strlen(s) - 1] == '\n')
       s[strlen(s) - 1] = 0;
-      rmspace(s);
+    rmspace(s);
     if ((s[0]) && (s[0] != '#') && (s[0] != ';')) {   /* Not comment */
       s1 = s;
       to = newsplit(&s1);
-      if (!egg_strcasecmp(to, argv[1])) {
+      if (!strcasecmp(to, argv[1])) {
         read++;
         if (!notes_in(nl, read))
           fprintf(g, "%s %s\n", to, s1);
@@ -502,13 +502,13 @@ static void notes_read(char *hand, char *nick, char *srd, int idx)
     if ((s[0]) && (s[0] != '#') && (s[0] != ';')) {   /* Not comment */
       s1 = s;
       to = newsplit(&s1);
-      if (!egg_strcasecmp(to, hand)) {
+      if (!strcasecmp(to, hand)) {
         int lapse;
 
         from = newsplit(&s1);
         dt = newsplit(&s1);
         tt = atoi(dt);
-        egg_strftime(wt, 14, "%b %d %H:%M", localtime(&tt));
+        strftime(wt, 14, "%b %d %H:%M", localtime(&tt));
         dt = wt;
         lapse = (int) ((now - tt) / 86400);
         if (lapse > note_life - 7) {
@@ -615,7 +615,7 @@ static void notes_del(char *hand, char *nick, char *sdl, int idx)
     if ((s[0]) && (s[0] != '#') && (s[0] != ';')) {   /* Not comment */
       s1 = s;
       to = newsplit(&s1);
-      if (!egg_strcasecmp(to, hand)) {
+      if (!strcasecmp(to, hand)) {
         if (!notes_in(dl, in))
           fprintf(g, "%s %s\n", to, s1);
         else
@@ -699,7 +699,7 @@ static int tcl_notes STDVAR
     if ((s[0]) && (s[0] != '#') && (s[0] != ';')) {   /* Not comment */
       s1 = s;
       to = newsplit(&s1);
-      if (!egg_strcasecmp(to, argv[1])) {
+      if (!strcasecmp(to, argv[1])) {
         read++;
         if (notes_in(nl, read)) {
           count++;
@@ -751,19 +751,19 @@ static int msg_notes(char *nick, char *host, struct userrec *u, char *par)
   }
 
   fcn = newsplit(&par);
-  if (!egg_strcasecmp(fcn, "INDEX"))
+  if (!strcasecmp(fcn, "INDEX"))
     notes_read(u->handle, nick, "+", -1);
-  else if (!egg_strcasecmp(fcn, "READ")) {
-    if (!egg_strcasecmp(par, "ALL"))
+  else if (!strcasecmp(fcn, "READ")) {
+    if (!strcasecmp(par, "ALL"))
       notes_read(u->handle, nick, "-", -1);
     else
       notes_read(u->handle, nick, par, -1);
-  } else if (!egg_strcasecmp(fcn, "ERASE")) {
-    if (!egg_strcasecmp(par, "ALL"))
+  } else if (!strcasecmp(fcn, "ERASE")) {
+    if (!strcasecmp(par, "ALL"))
       notes_del(u->handle, nick, "-", -1);
     else
       notes_del(u->handle, nick, par, -1);
-  } else if (!egg_strcasecmp(fcn, "TO")) {
+  } else if (!strcasecmp(fcn, "TO")) {
     char *to;
     int i;
     FILE *f;
@@ -784,7 +784,7 @@ static int msg_notes(char *nick, char *host, struct userrec *u, char *par)
       return 1;
     }
     for (i = 0; i < dcc_total; i++) {
-      if ((!egg_strcasecmp(dcc[i].nick, to)) &&
+      if ((!strcasecmp(dcc[i].nick, to)) &&
           (dcc[i].type->flags & DCT_GETNOTES)) {
         int aok = 1;
 
@@ -845,7 +845,7 @@ static void notes_hourly()
           k = num_notes(u->handle);
           for (l = 0; l < dcc_total; l++)
             if ((dcc[l].type->flags & DCT_CHAT) &&
-                !egg_strcasecmp(dcc[l].nick, u->handle)) {
+                !strcasecmp(dcc[l].nick, u->handle)) {
               k = 0;            /* They already know they have notes */
               break;
             }
@@ -872,7 +872,7 @@ static void away_notes(char *bot, int sock, char *msg)
 {
   int idx = findanyidx(sock);
 
-  if (egg_strcasecmp(bot, botnetnick))
+  if (strcasecmp(bot, botnetnick))
     return;
   if (msg && msg[0])
     dprintf(idx, "%s\n", NOTES_STORED);
@@ -895,7 +895,7 @@ static void join_notes(char *nick, char *uhost, char *handle, char *par)
   if (notify_onjoin) {          /* drummer */
     for (j = 0; j < dcc_total; j++)
       if ((dcc[j].type->flags & DCT_CHAT) &&
-          (!egg_strcasecmp(dcc[j].nick, handle)))
+          (!strcasecmp(dcc[j].nick, handle)))
         return;                 /* They already know they have notes */
 
     while (chan) {
@@ -925,7 +925,7 @@ static struct xtra_key *getnotesentry(struct userrec *u)
     return NULL;
   /* Search for the notes ignore list entry */
   for (xk = ue->u.extra; xk; xk = xk->next)
-    if (xk->key && !egg_strcasecmp(xk->key, NOTES_IGNKEY)) {
+    if (xk->key && !strcasecmp(xk->key, NOTES_IGNKEY)) {
       nxk = xk;
       break;
     }
