@@ -3,7 +3,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,6 +23,13 @@
 #ifndef _EGG_MOD_SERVER_SERVER_H
 #define _EGG_MOD_SERVER_SERVER_H
 
+#define CAPMAX       499    /*  (512 - "CAP REQ :XXX\r\n")     */
+#define CLITAGMAX    4096   /* Max size for IRCv3 message-tags sent by client*/
+#define TOTALTAGMAX  8191   /* @ + Server tag len + ; + Client tag len + ' ' */
+#define MSGMAX       511    /* Max size of IRC message line    */
+#define SENDLINEMAX  CLITAGMAX + MSGMAX
+#define RECVLINEMAX  TOTALTAGMAX + MSGMAX
+
 #define check_tcl_ctcp(a,b,c,d,e,f) check_tcl_ctcpr(a,b,c,d,e,f,H_ctcp)
 #define check_tcl_ctcr(a,b,c,d,e,f) check_tcl_ctcpr(a,b,c,d,e,f,H_ctcr)
 
@@ -44,7 +51,7 @@
 /* 12 - 15 */
 #define match_my_nick ((int(*)(char *))server_funcs[12])
 #define check_tcl_flud ((int (*)(char *,char *,struct userrec *,char *,char *))server_funcs[13])
-/* Was fixfrom (moved to core) */
+/* Empty, formally msgtag */
 #define answer_ctcp (*(int *)(server_funcs[15]))
 /* 16 - 19 */
 #define trigger_on_ignore (*(int *)(server_funcs[16]))
@@ -59,7 +66,7 @@
 /* 24 - 27 */
 #define default_port (*(int *)(server_funcs[24]))
 #define server_online (*(int *)(server_funcs[25]))
-/* Was min_servs */
+#define H_rawt (*(p_tcl_bind_list *)(server_funcs[26]))
 #define H_raw (*(p_tcl_bind_list *)(server_funcs[27]))
 /* 28 - 31 */
 #define H_wall (*(p_tcl_bind_list *)(server_funcs[28]))
@@ -78,19 +85,16 @@
 #define exclusive_binds (*(int *)(server_funcs[39]))
 /* 40 - 43 */
 #define H_out (*(p_tcl_bind_list *)(server_funcs[40]))
-#else /* MAKING_SERVER */
-
-/* Macros for commonly used commands. */
-#define free_null(ptr)  do {                            \
-        nfree(ptr);                                     \
-        ptr = NULL;                                     \
-} while (0)
-
-#define write_to_server(x,y) do {                       \
-        tputs(serv, (x), (y));                          \
-        tputs(serv, "\r\n", 2);                         \
-} while (0)
-
+#define net_type_int (*(int *)(server_funcs[41]))
+#define H_account (*(p_tcl_bind_list *)(server_funcs[42]))
+#define cap (*(capability_t **)(server_funcs[43]))
+/* 44 - 47 */
+#define extended_join (*(int *)(server_funcs[44]))
+#define account_notify (*(int *)(server_funcs[45]))
+#define H_isupport (*(p_tcl_bind_list *)(server_funcs[46]))
+#define isupport_get ((struct isupport *(*)(const char *, size_t))(server_funcs[47]))
+/* 48 - 52 */
+#define isupport_parseint ((int (*)(const char *, const char *, int, int, int, int, int *))(server_funcs[48]))
 #endif /* MAKING_SERVER */
 
 struct server_list {
@@ -105,13 +109,46 @@ struct server_list {
   char *realname;
 };
 
-/* Available net types.  */
+/* struct to store values associated with a capability, such as "PLAIN" and
+ * "EXTERNAL" for SASL
+ */
+typedef struct cap_values {
+  struct cap_values *next;
+  char name[CAPMAX];
+} cap_values_t;
+
+typedef struct capability {
+  struct capability *next;
+  char name[CAPMAX+1];  /* Name of capability, +1 bc CAPMAX is for REQ not LS */
+  struct cap_values *value; /* List of values associated with the capability  */
+  int enabled;      /* Is the capability currently negotiated with the server */
+  int requested;    /* Does Eggdrop  want this capability, if available?      */
+} capability_t;
+
+/* Available net types. */
 enum {
-  NETT_EFNET        = 0, /* EFnet                    */
-  NETT_IRCNET       = 1, /* IRCnet                   */
-  NETT_UNDERNET     = 2, /* UnderNet                 */
-  NETT_DALNET       = 3, /* DALnet                   */
-  NETT_HYBRID_EFNET = 4  /* +e/+I/max-bans 20 Hybrid */
-} nett_t;
+  NETT_DALNET,       /* DALnet                            */
+  NETT_EFNET,        /* EFnet                             */
+  NETT_FREENODE,     /* freenode                          */
+  NETT_HYBRID_EFNET, /* Hybrid-6+ EFnet +e/+I/max-bans 20 */
+  NETT_IRCNET,       /* IRCnet                            */
+  NETT_LIBERA,       /* Libera Chat                       */
+  NETT_QUAKENET,     /* QuakeNet                          */
+  NETT_RIZON,        /* Rizon                             */
+  NETT_UNDERNET,     /* UnderNet                          */
+  NETT_TWITCH,       /* Twitch! *shudder*                 */
+  NETT_OTHER         /* Others                            */
+};
+
+/* Available sasl mechanisms. */
+enum {
+  SASL_MECHANISM_PLAIN,
+  SASL_MECHANISM_ECDSA_NIST256P_CHALLENGE,
+  SASL_MECHANISM_EXTERNAL,
+  SASL_MECHANISM_NUM
+};
+
+/* Must be extern to avoid odr-violation and allow make static */
+extern char const *SASL_MECHANISMS[];
 
 #endif /* _EGG_MOD_SERVER_SERVER_H */
