@@ -4,7 +4,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -112,7 +112,7 @@ static void add_delay(struct chanset_t *chan, int plsmns, int mode, char *mask)
   d->seconds = now + randint(30);
 
   d->mask = nmalloc(strlen(mask) + 1);
-  strncpyz(d->mask, mask, strlen(mask) + 1);
+  strcpy(d->mask, mask);
 
   if (!delay_head)
     delay_head = d;
@@ -586,7 +586,7 @@ static void share_pls_host(int idx, char *par)
 
 static void share_pls_bothost(int idx, char *par)
 {
-  char *hand, p[32];
+  char *hand, pass[PASSWORDLEN];
   struct userrec *u;
 
   if ((dcc[idx].status & STAT_SHARE) && !private_user) {
@@ -601,8 +601,9 @@ static void share_pls_bothost(int idx, char *par)
           return;               /* ignore */
         set_user(&USERENTRY_HOSTS, u, par);
       } else {
-        makepass(p);
-        userlist = adduser(userlist, hand, par, p, USER_BOT);
+        makepass(pass);
+        userlist = adduser(userlist, hand, par, pass, USER_BOT);
+        explicit_bzero(pass, sizeof pass);
       }
       if (!(dcc[idx].status & STAT_GETTING))
         putlog(LOG_CMDS, "*", "%s: +host %s %s", dcc[idx].nick, hand, par);
@@ -630,7 +631,7 @@ static void share_mns_host(int idx, char *par)
 
 static void share_change(int idx, char *par)
 {
-  char *key, *hand;
+  char *key, *hand, pass[PASSWORDLEN];
   struct userrec *u;
   struct user_entry_type *uet;
   struct user_entry *e;
@@ -648,10 +649,9 @@ static void share_change(int idx, char *par)
           shareout_but(NULL, idx, "c %s %s %s\n", key, hand, par);
         noshare = 1;
         if (!u && (uet == &USERENTRY_BOTADDR)) {
-          char pass[30];
-
           makepass(pass);
           userlist = adduser(userlist, hand, "none", pass, USER_BOT);
+          explicit_bzero(pass, sizeof pass);
           u = get_user_by_handle(userlist, hand);
         } else if (!u) {
           noshare = 0;
@@ -1130,7 +1130,7 @@ static void share_userfileq(int idx, char *par)
   int ok = 1, i, bfl = bot_flags(dcc[idx].user);
 
   flush_tbuf(dcc[idx].nick);
-  if (bfl & BOT_AGGRESSIVE)
+  if (bfl & (BOT_AGGRESSIVE|BOT_SHPERMS))
     dprintf(idx, "s un I have you marked for Aggressive sharing.\n");
   else if (!(bfl & BOT_PASSIVE))
     dprintf(idx, "s un You are not marked for sharing with me.\n");
@@ -1191,7 +1191,7 @@ static void share_ufsend(int idx, char *par)
     newsplit(&par);
     port = newsplit(&par);
     i = new_dcc(&DCC_FORK_SEND, sizeof(struct xfer_info));
-    /* Use same addr we succesfully linked to and change port */
+    /* Use same addr we successfully linked to and change port */
     memcpy(&dcc[i].sockname, &dcc[idx].sockname, sizeof dcc[i].sockname);
     dcc[i].port = atoi(port);
     setsnport(dcc[i].sockname, dcc[i].port);
@@ -1274,7 +1274,7 @@ static void share_version(int idx, char *par)
                        STAT_OFFERED | STAT_AGGRESSIVE);
   dcc[idx].u.bot->uff_flags = 0;
   if ((dcc[idx].u.bot->numver >= min_share) &&
-      (bot_flags(dcc[idx].user) & BOT_AGGRESSIVE)) {
+      (bot_flags(dcc[idx].user) & (BOT_AGGRESSIVE|BOT_SHPERMS))) {
     if (can_resync(dcc[idx].nick))
       dprintf(idx, "s r?\n");
     else
@@ -1322,49 +1322,52 @@ static void share_feats(int idx, char *par)
 }
 
 
-/* Note: these MUST be sorted. */
-static botcmd_t C_share[] = {
-  {"!",        (IntFunc) share_endstartup},
-  {"+b",       (IntFunc) share_pls_ban},
-  {"+bc",      (IntFunc) share_pls_banchan},
-  {"+bh",      (IntFunc) share_pls_bothost},
-  {"+cr",      (IntFunc) share_pls_chrec},
-  {"+e",       (IntFunc) share_pls_exempt},
-  {"+ec",      (IntFunc) share_pls_exemptchan},
-  {"+h",       (IntFunc) share_pls_host},
-  {"+i",       (IntFunc) share_pls_ignore},
-  {"+inv",     (IntFunc) share_pls_invite},
-  {"+invc",    (IntFunc) share_pls_invitechan},
-  {"-b",       (IntFunc) share_mns_ban},
-  {"-bc",      (IntFunc) share_mns_banchan},
-  {"-cr",      (IntFunc) share_mns_chrec},
-  {"-e",       (IntFunc) share_mns_exempt},
-  {"-ec",      (IntFunc) share_mns_exemptchan},
-  {"-h",       (IntFunc) share_mns_host},
-  {"-i",       (IntFunc) share_mns_ignore},
-  {"-inv",     (IntFunc) share_mns_invite},
-  {"-invc",    (IntFunc) share_mns_invitechan},
-  {"a",        (IntFunc) share_chattr},
-  {"c",        (IntFunc) share_change},
-  {"chchinfo", (IntFunc) share_chchinfo},
-  {"e",        (IntFunc) share_end},
-  {"feats",    (IntFunc) share_feats},
-  {"h",        (IntFunc) share_chhand},
-  {"k",        (IntFunc) share_killuser},
-  {"n",        (IntFunc) share_newuser},
-  {"nc",       (IntFunc) share_newchan},
-  {"r!",       (IntFunc) share_resync},
-  {"r?",       (IntFunc) share_resyncq},
-  {"rn",       (IntFunc) share_resync_no},
-  {"s",        (IntFunc) share_stick_ban},
-  {"se",       (IntFunc) share_stick_exempt},
-  {"sInv",     (IntFunc) share_stick_invite},
-  {"u?",       (IntFunc) share_userfileq},
-  {"un",       (IntFunc) share_ufno},
-  {"us",       (IntFunc) share_ufsend},
-  {"uy",       (IntFunc) share_ufyes},
-  {"v",        (IntFunc) share_version},
-  {NULL,       NULL}
+/* Note: these MUST be sorted.
+ * Flags (second arg) are compared with OR,
+ * here in particular this means +p can execute all
+ */
+static botscmd_t C_share[] = {
+  {"!",        "",  (IntFunc) share_endstartup},
+  {"+b",       "psb", (IntFunc) share_pls_ban},
+  {"+bc",      "psb", (IntFunc) share_pls_banchan},
+  {"+bh",      "psu", (IntFunc) share_pls_bothost},
+  {"+cr",      "psc", (IntFunc) share_pls_chrec},
+  {"+e",       "pse", (IntFunc) share_pls_exempt},
+  {"+ec",      "pse", (IntFunc) share_pls_exemptchan},
+  {"+h",       "psu", (IntFunc) share_pls_host},
+  {"+i",       "psn", (IntFunc) share_pls_ignore},
+  {"+inv",     "psj", (IntFunc) share_pls_invite},
+  {"+invc",    "psj", (IntFunc) share_pls_invitechan},
+  {"-b",       "psb", (IntFunc) share_mns_ban},
+  {"-bc",      "psb", (IntFunc) share_mns_banchan},
+  {"-cr",      "psc", (IntFunc) share_mns_chrec},
+  {"-e",       "pse", (IntFunc) share_mns_exempt},
+  {"-ec",      "pse", (IntFunc) share_mns_exemptchan},
+  {"-h",       "psu", (IntFunc) share_mns_host},
+  {"-i",       "psn", (IntFunc) share_mns_ignore},
+  {"-inv",     "psj", (IntFunc) share_mns_invite},
+  {"-invc",    "psj", (IntFunc) share_mns_invitechan},
+  {"a",        "psu", (IntFunc) share_chattr},
+  {"c",        "psu", (IntFunc) share_change},
+  {"chchinfo", "psu", (IntFunc) share_chchinfo},
+  {"e",        "",  (IntFunc) share_end},
+  {"feats",    "",  (IntFunc) share_feats},
+  {"h",        "psu", (IntFunc) share_chhand},
+  {"k",        "psu", (IntFunc) share_killuser},
+  {"n",        "psu", (IntFunc) share_newuser},
+  {"nc",       "psc", (IntFunc) share_newchan},
+  {"r!",       "",  (IntFunc) share_resync},
+  {"r?",       "",  (IntFunc) share_resyncq},
+  {"rn",       "",  (IntFunc) share_resync_no},
+  {"s",        "psb", (IntFunc) share_stick_ban},
+  {"se",       "pse", (IntFunc) share_stick_exempt},
+  {"sInv",     "psj", (IntFunc) share_stick_invite},
+  {"u?",       "",  (IntFunc) share_userfileq},
+  {"un",       "",  (IntFunc) share_ufno},
+  {"us",       "",  (IntFunc) share_ufsend},
+  {"uy",       "",  (IntFunc) share_ufyes},
+  {"v",        "",  (IntFunc) share_version},
+  {NULL,       NULL, NULL}
 };
 
 
@@ -1375,11 +1378,21 @@ static void sharein_mod(int idx, char *msg)
 
   code = newsplit(&msg);
   for (f = 0, i = 0; C_share[i].name && !f; i++) {
-    int y = egg_strcasecmp(code, C_share[i].name);
+    int y = strcasecmp(code, C_share[i].name);
 
-    if (!y)
+    if (!y) {
       /* Found a match */
-      (C_share[i].func) (idx, msg);
+      struct flag_record fr = { FR_BOT , 0, 0, 0, 0, 0 };
+      struct flag_record req = { FR_BOT | FR_OR, 0, 0, 0, 0, 0 };
+
+      break_down_flags(C_share[i].flags, &req, NULL);
+      get_user_flagrec(dcc[idx].user, &fr, NULL);
+      if (flagrec_eq(&req, &fr)) {
+        (C_share[i].func) (idx, msg);
+      } else {
+        putlog(LOG_DEBUG, "*", "Userfile modification from %s rejected: incorrect bot flag permissions for \"%s %s\"", dcc[idx].nick, code, msg);
+      }
+    }
     if (y <= 0)
       f = 1;
   }
@@ -1409,8 +1422,10 @@ static void shareout_mod EGG_VARARGS_DEF(struct chanset_t *, arg1)
           fr.match = (FR_CHAN | FR_BOT);
           get_user_flagrec(dcc[i].user, &fr, chan->dname);
         }
-        if (!chan || bot_chan(fr) || bot_global(fr))
+        if (!chan || bot_chan(fr) || bot_global(fr)) {
+          putlog(LOG_BOTSHROUT, "*", "{b->%s} %s", dcc[i].nick, s + 2);
           tputs(dcc[i].sock, s, l + 2);
+        }
       }
     q_resync(s, chan);
   }
@@ -1441,8 +1456,10 @@ static void shareout_but EGG_VARARGS_DEF(struct chanset_t *, arg1)
         fr.match = (FR_CHAN | FR_BOT);
         get_user_flagrec(dcc[i].user, &fr, chan->dname);
       }
-      if (!chan || bot_chan(fr) || bot_global(fr))
+      if (!chan || bot_chan(fr) || bot_global(fr)) {
+        putlog(LOG_BOTSHROUT, "*", "{b->%s} %s", dcc[i].nick, s + 2);
         tputs(dcc[i].sock, s, l + 2);
+      }
     }
   q_resync(s, chan);
   va_end(va);
@@ -1457,14 +1474,14 @@ static void shareout_but EGG_VARARGS_DEF(struct chanset_t *, arg1)
  */
 static void new_tbuf(char *bot)
 {
-  tandbuf **old = &tbuf, *new;
+  tandbuf *new;
 
   new = nmalloc(sizeof(tandbuf));
-  strncpyz(new->bot, bot, sizeof new->bot);
+  strlcpy(new->bot, bot, sizeof new->bot);
   new->q = NULL;
   new->timer = now;
-  new->next = *old;
-  *old = new;
+  new->next = tbuf;
+  tbuf = new;
   putlog(LOG_BOTS, "*", "Creating resync buffer for %s", bot);
 }
 
@@ -1498,7 +1515,7 @@ static int flush_tbuf(char *bot)
 
   for (t = tbuf; t; t = tnext) {
     tnext = t->next;
-    if (!egg_strcasecmp(t->bot, bot)) {
+    if (!strcasecmp(t->bot, bot)) {
       del_tbuf(t);
       return 1;
     }
@@ -1525,13 +1542,13 @@ static void check_expired_tbufs()
     if (dcc[i].type->flags & DCT_BOT) {
       if (dcc[i].status & STAT_OFFERED) {
         if ((now - dcc[i].timeval > 120) && (dcc[i].user &&
-            (bot_flags(dcc[i].user) & BOT_AGGRESSIVE)))
+            (bot_flags(dcc[i].user) & (BOT_AGGRESSIVE|BOT_SHPERMS))))
           dprintf(i, "s u?\n");
           /* ^ send it again in case they missed it */
         /* If it's a share bot that hasnt been sharing, ask again */
       } else if (!(dcc[i].status & STAT_SHARE)) {
         /* Patched from original source by giusc@gbss.it <20040207> */
-        if (dcc[i].user && (bot_flags(dcc[i].user) & BOT_AGGRESSIVE))  {
+        if (dcc[i].user && (bot_flags(dcc[i].user) & (BOT_AGGRESSIVE|BOT_SHPERMS)))  {
           dprintf(i, "s u?\n");
           dcc[i].status |= STAT_OFFERED;
         }
@@ -1577,7 +1594,7 @@ static void q_tbuf(char *bot, char *s, struct chanset_t *chan)
   tandbuf *t;
 
   for (t = tbuf; t && t->bot[0]; t = t->next)
-    if (!egg_strcasecmp(t->bot, bot)) {
+    if (!strcasecmp(t->bot, bot)) {
       if (chan) {
         fr.match = (FR_CHAN | FR_BOT);
         get_user_flagrec(get_user_by_handle(userlist, bot), &fr, chan->dname);
@@ -1614,7 +1631,7 @@ static int can_resync(char *bot)
   tandbuf *t;
 
   for (t = tbuf; t && t->bot[0]; t = t->next)
-    if (!egg_strcasecmp(bot, t->bot))
+    if (!strcasecmp(bot, t->bot))
       return 1;
   return 0;
 }
@@ -1627,7 +1644,7 @@ static void dump_resync(int idx)
   tandbuf *t;
 
   for (t = tbuf; t && t->bot[0]; t = t->next)
-    if (!egg_strcasecmp(dcc[idx].nick, t->bot)) {
+    if (!strcasecmp(dcc[idx].nick, t->bot)) {
       for (q = t->q; q && q->msg[0]; q = q->next) {
         dprintf(idx, "%s", q->msg);
       }
@@ -1727,6 +1744,9 @@ static struct userrec *dup_userlist(int t)
         ((t == 1) && (u->flags & (USER_BOT | USER_UNSHARED))) || (t == 2)) {
       p = get_user(&USERENTRY_PASS, u);
       u1 = adduser(NULL, u->handle, 0, p, u->flags);
+      p = get_user(&USERENTRY_PASS2, u);
+      if (p)
+        set_user(&USERENTRY_PASS2, u1, p);
       u1->flags_udef = u->flags_udef;
       if (!nu)
         nu = retu = u1;
@@ -1783,7 +1803,7 @@ static void finish_share(int idx)
   int i, j = -1;
 
   for (i = 0; i < dcc_total; i++)
-    if (!egg_strcasecmp(dcc[i].nick, dcc[idx].host) &&
+    if (!strcasecmp(dcc[i].nick, dcc[idx].host) &&
         (dcc[i].type->flags & DCT_BOT))
       j = i;
   if (j == -1)
@@ -1834,7 +1854,7 @@ static void finish_share(int idx)
   userlist = (void *) -1;       /* Do this to prevent .user messups     */
 
   /* Bot user pointers are updated to point to the new list, all others
-   * are set to NULL. If our userfile will be overriden, just set _all_
+   * are set to NULL. If our userfile will be overridden, just set _all_
    * to NULL directly.
    */
   if (u == NULL)
@@ -2045,14 +2065,14 @@ static void start_sending_users(int idx)
           egg_snprintf(s2, sizeof s2, "s a %s %s\n", u->handle, s1);
           q_tbuf(dcc[idx].nick, s2, NULL);
           for (ch = u->chanrec; ch; ch = ch->next) {
-            if ((ch->flags & ~BOT_SHARE) &&
+            if ((ch->flags & ~BOT_AGGRESSIVE) &&
                 ((cst = findchan_by_dname(ch->channel)) &&
                  channel_shared(cst))) {
               fr.match = (FR_CHAN | FR_BOT);
               get_user_flagrec(dcc[idx].user, &fr, ch->channel);
               if (bot_chan(fr) || bot_global(fr)) {
                 fr.match = FR_CHAN;
-                fr.chan = ch->flags & ~BOT_SHARE;
+                fr.chan = ch->flags & ~BOT_AGGRESSIVE;
                 fr.udef_chan = ch->flags_udef;
                 build_flags(s1, &fr, NULL);
                 egg_snprintf(s2, sizeof s2, "s a %s %s %s\n", u->handle, s1,
@@ -2088,7 +2108,7 @@ static void cancel_user_xfer(int idx, void *x)
     if (dcc[idx].status & STAT_GETTING) {
       j = 0;
       for (i = 0; i < dcc_total; i++)
-        if (!egg_strcasecmp(dcc[i].host, dcc[idx].nick) &&
+        if (!strcasecmp(dcc[i].host, dcc[idx].nick) &&
             ((dcc[i].type->flags & (DCT_FILETRAN | DCT_FILESEND)) ==
              (DCT_FILETRAN | DCT_FILESEND)))
           j = i;
@@ -2102,7 +2122,7 @@ static void cancel_user_xfer(int idx, void *x)
     if (dcc[idx].status & STAT_SENDING) {
       j = 0;
       for (i = 0; i < dcc_total; i++)
-        if ((!egg_strcasecmp(dcc[i].host, dcc[idx].nick)) &&
+        if ((!strcasecmp(dcc[i].host, dcc[idx].nick)) &&
             ((dcc[i].type->flags & (DCT_FILETRAN | DCT_FILESEND)) ==
             DCT_FILETRAN))
           j = i;
@@ -2222,7 +2242,7 @@ static void share_report(int idx, int details)
           for (j = 0; j < dcc_total; j++)
             if (((dcc[j].type->flags & (DCT_FILETRAN | DCT_FILESEND)) ==
                 (DCT_FILETRAN | DCT_FILESEND)) &&
-                !egg_strcasecmp(dcc[j].host, dcc[i].nick)) {
+                !strcasecmp(dcc[j].host, dcc[i].nick)) {
               dprintf(idx, "    Downloading userlist from %s (%d%% done)\n",
                       dcc[i].nick, (int) (100.0 * ((float) dcc[j].status) /
                       ((float) dcc[j].u.xfer->length)));
@@ -2235,7 +2255,7 @@ static void share_report(int idx, int details)
         } else if (dcc[i].status & STAT_SENDING) {
           for (j = 0; j < dcc_total; j++) {
             if (((dcc[j].type->flags & (DCT_FILETRAN | DCT_FILESEND)) ==
-                DCT_FILETRAN) && !egg_strcasecmp(dcc[j].host, dcc[i].nick)) {
+                DCT_FILETRAN) && !strcasecmp(dcc[j].host, dcc[i].nick)) {
               if (dcc[j].type == &DCC_GET)
                 dprintf(idx, "    Sending userlist to %s (%d%% done)\n",
                         dcc[i].nick, (int) (100.0 * ((float) dcc[j].status) /
@@ -2307,7 +2327,7 @@ char *share_start(Function *global_funcs)
   return NULL;
 }
 
-int private_globals_bitmask()
+static int private_globals_bitmask()
 {
   struct flag_record fr = { FR_GLOBAL, 0, 0, 0, 0, 0 };
 
