@@ -1,7 +1,7 @@
 /*
  * transferqueue.c -- part of transfer.mod
  *
- * Copyright (C) 2003 - 2018 Eggheads Development Team
+ * Copyright (C) 2003 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,12 +38,12 @@ static void queue_file(char *dir, char *file, char *from, char *to)
   fileq->next = q;
   l = strlen(dir) + 1;
   fileq->dir = nmalloc(l);
-  strncpyz(fileq->dir, dir, l);
+  strlcpy(fileq->dir, dir, l);
   l = strlen(file) + 1;
   fileq->file = nmalloc(l);
-  strncpyz(fileq->file, file, l);
-  strncpyz(fileq->nick, from, sizeof fileq->nick);
-  strncpyz(fileq->to, to, sizeof fileq->to);
+  strlcpy(fileq->file, file, l);
+  strlcpy(fileq->nick, from, sizeof fileq->nick);
+  strlcpy(fileq->to, to, sizeof fileq->to);
 }
 
 static void deq_this(fileq_t *this)
@@ -71,14 +71,14 @@ static void deq_this(fileq_t *this)
  */
 static void flush_fileq(char *to)
 {
-  fileq_t *q = fileq;
+  fileq_t *q;
   int fnd = 1;
 
   while (fnd) {
     q = fileq;
     fnd = 0;
     while (q != NULL) {
-      if (!egg_strcasecmp(q->to, to)) {
+      if (!strcasecmp(q->to, to)) {
         deq_this(q);
         q = NULL;
         fnd = 1;
@@ -96,7 +96,7 @@ static void send_next_file(char *to)
   int x;
 
   for (q = fileq; q; q = q->next)
-    if (!egg_strcasecmp(q->to, to))
+    if (!strcasecmp(q->to, to))
       this = q;
 
   if (this == NULL)
@@ -127,7 +127,7 @@ static void send_next_file(char *to)
   }
   x = raw_dcc_send(s, this->to, this->nick);
   if (x == DCCSEND_OK) {
-    if (egg_strcasecmp(this->to, this->nick))
+    if (strcasecmp(this->to, this->nick))
       dprintf(DP_HELP, TRANSFER_FILE_ARRIVE, this->to, this->nick);
     deq_this(this);
   } else if (x == DCCSEND_FULL) {
@@ -164,7 +164,7 @@ static void show_queued_files(int idx)
   fileq_t *q;
 
   for (q = fileq; q; q = q->next) {
-    if (!egg_strcasecmp(q->nick, dcc[idx].nick)) {
+    if (!strcasecmp(q->nick, dcc[idx].nick)) {
       if (!cnt) {
         spaces[HANDLEN - 9] = 0;
         dprintf(idx, TRANSFER_SEND_TO, spaces);
@@ -183,8 +183,8 @@ static void show_queued_files(int idx)
   }
   for (i = 0; i < dcc_total; i++) {
     if ((dcc[i].type == &DCC_GET_PENDING || dcc[i].type == &DCC_GET) &&
-        (!egg_strcasecmp(dcc[i].nick, dcc[idx].nick) ||
-         !egg_strcasecmp(dcc[i].u.xfer->from, dcc[idx].nick))) {
+        (!strcasecmp(dcc[i].nick, dcc[idx].nick) ||
+         !strcasecmp(dcc[i].u.xfer->from, dcc[idx].nick))) {
       char *nfn;
 
       if (!cnt) {
@@ -224,7 +224,7 @@ static void fileq_cancel(int idx, char *par)
     q = fileq;
     fnd = 0;
     while (q != NULL) {
-      if (!egg_strcasecmp(dcc[idx].nick, q->nick)) {
+      if (!strcasecmp(dcc[idx].nick, q->nick)) {
         s = nrealloc(s, strlen(q->dir) + strlen(q->file) + 3);
         if (q->dir[0] == '*')
           sprintf(s, "%s/%s", &q->dir[1], q->file);
@@ -255,8 +255,8 @@ static void fileq_cancel(int idx, char *par)
 
   for (i = 0; i < dcc_total; i++) {
     if ((dcc[i].type == &DCC_GET_PENDING || dcc[i].type == &DCC_GET) &&
-        (!egg_strcasecmp(dcc[i].nick, dcc[idx].nick) ||
-         !egg_strcasecmp(dcc[i].u.xfer->from, dcc[idx].nick))) {
+        (!strcasecmp(dcc[i].nick, dcc[idx].nick) ||
+         !strcasecmp(dcc[i].u.xfer->from, dcc[idx].nick))) {
       char *nfn = strrchr(dcc[i].u.xfer->origname, '/');
 
       if (nfn == NULL)
@@ -265,7 +265,7 @@ static void fileq_cancel(int idx, char *par)
         nfn++;
       if (wild_match_file(par, nfn)) {
         dprintf(idx, TRANSFER_ABORT_DCCSEND, nfn);
-        if (egg_strcasecmp(dcc[i].nick, dcc[idx].nick))
+        if (strcasecmp(dcc[i].nick, dcc[idx].nick))
           dprintf(DP_HELP, TRANSFER_NOTICE_ABORT, dcc[i].nick, nfn,
                   dcc[idx].nick);
         if (dcc[i].type == &DCC_GET)
