@@ -6,7 +6,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,7 +23,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include <ctype.h>
 #include <signal.h>
 #include "main.h"
 #include "modules.h"
@@ -53,27 +52,19 @@
 #  endif
 
 #  ifdef MOD_USE_DL
-#    ifdef DLOPEN_1
-char *dlerror();
-void *dlopen(const char *, int);
-int dlclose(void *);
-void *dlsym(void *, char *);
-#      define DLFLAGS 1
-#    else /* DLOPEN_1 */
-#      include <dlfcn.h>
+#    include <dlfcn.h>
 
-#      ifndef RTLD_GLOBAL
-#        define RTLD_GLOBAL 0
-#      endif
-#      ifndef RTLD_NOW
-#        define RTLD_NOW 1
-#      endif
-#      ifdef RTLD_LAZY
-#        define DLFLAGS RTLD_LAZY|RTLD_GLOBAL
-#      else
-#        define DLFLAGS RTLD_NOW|RTLD_GLOBAL
-#      endif
-#    endif /* DLOPEN_1 */
+#    ifndef RTLD_GLOBAL
+#      define RTLD_GLOBAL 0
+#    endif
+#    ifndef RTLD_NOW
+#      define RTLD_NOW 1
+#    endif
+#    ifdef RTLD_LAZY
+#      define DLFLAGS RTLD_LAZY|RTLD_GLOBAL
+#    else
+#      define DLFLAGS RTLD_NOW|RTLD_GLOBAL
+#    endif
 #  endif /* MOD_USE_DL */
 #endif /* !STATIC */
 
@@ -84,7 +75,8 @@ extern struct userrec *userlist, *lastuser;
 extern struct chanset_t *chanset;
 
 extern char botnetnick[], botname[], origbotname[], botuser[], ver[], log_ts[],
-            admin[], userfile[], notify_new[], helpdir[], version[], quit_msg[];
+            admin[], userfile[], notify_new[], helpdir[], version[], quit_msg[],
+            pid_file[];
 
 extern int parties, noshare, dcc_total, egg_numver, userfile_perm, ignore_time,
            must_be_owner, raw_log, max_dcc, make_userfile, default_flags,
@@ -110,6 +102,7 @@ extern sock_list *socklist;
 int cmd_die();
 int xtra_kill();
 int xtra_unpack();
+char *check_validpass();
 static int module_rename(char *name, char *newname);
 
 #ifndef STATIC
@@ -165,6 +158,8 @@ static void null_share(int idx, char *x)
 }
 
 void (*encrypt_pass) (char *, char *) = 0;
+char *(*encrypt_pass2) (char *) = 0;
+char *(*verify_pass2) (char *, char *) = 0;
 char *(*encrypt_string) (char *, char *) = 0;
 char *(*decrypt_string) (char *, char *) = 0;
 void (*shareout) () = null_func;
@@ -177,8 +172,8 @@ int (*rfc_casecmp) (const char *, const char *) = _rfc_casecmp;
 int (*rfc_ncasecmp) (const char *, const char *, int) = _rfc_ncasecmp;
 int (*rfc_toupper) (int) = _rfc_toupper;
 int (*rfc_tolower) (int) = _rfc_tolower;
-void (*dns_hostbyip) (sockname_t *) = block_dns_hostbyip;
-void (*dns_ipbyhost) (char *) = block_dns_ipbyhost;
+void (*dns_hostbyip) (sockname_t *) = core_dns_hostbyip;
+void (*dns_ipbyhost) (char *) = core_dns_ipbyhost;
 
 module_entry *module_list;
 dependancy *dependancy_list = NULL;
@@ -350,7 +345,7 @@ Function global_table[] = {
   (Function) & tls_vfyclients,    /* int                                 */
   (Function) & tls_vfydcc,        /* int                                 */
 #else
-  (Function) 0,                   /* was natip -- use getmyip() instead  */
+  (Function) 0,                   /* was natip                           */
   (Function) 0,                   /* was myip -- use getvhost() instead  */
 #endif
   (Function) origbotname,         /* char *                              */
@@ -536,9 +531,9 @@ Function global_table[] = {
   (Function) egg_snprintf,
   (Function) egg_vsnprintf,
   (Function) 0,                   /* was egg_memset -- use memset() or egg_bzero() instead */
-  (Function) egg_strcasecmp,
+  (Function) 0,                   /* was egg_strcasecmp -- use strcasecmp() instead */
   /* 256 - 259 */
-  (Function) egg_strncasecmp,
+  (Function) 0,                   /* was egg_strncasecmp -- use strncasecmp() instead */
   (Function) is_file,
   (Function) & must_be_owner,     /* int                                 */
   (Function) & tandbot,           /* tand_t *                            */
@@ -551,7 +546,7 @@ Function global_table[] = {
   (Function) str_unescape,
   (Function) egg_strcatn,
   (Function) clear_chanlist_member,
-  (Function) fixfrom,
+  (Function) 0,                   /* was fixfrom */
   /* 268 - 271 */
   (Function) & socklist,          /* sock_list *                         */
   (Function) sockoptions,
@@ -606,7 +601,29 @@ Function global_table[] = {
   (Function) 0,
 #endif
   /* 304 - 307 */
-  (Function) strncpyz
+  (Function) strncpyz,
+#ifndef HAVE_BASE64
+  (Function) b64_ntop,
+  (Function) b64_pton,
+#else
+  (Function) 0,
+  (Function) 0,
+#endif
+  (Function) check_validpass,
+  /* 308 - 311 */
+  (Function) make_rand_str_from_chars,
+  (Function) add_tcl_objcommands,
+  (Function) pid_file,            /* char                                */
+#ifndef HAVE_EXPLICIT_BZERO
+  (Function) explicit_bzero,
+#else
+  (Function) 0,
+#endif
+/* 312 - 315 */    
+  (Function) & USERENTRY_PASS2,   /* struct user_entry_type *            */
+  (Function) crypto_verify,
+  (Function) egg_uname,
+  (Function) get_expire_time
 };
 
 void init_modules(void)
@@ -663,7 +680,7 @@ int module_register(char *name, Function *funcs, int major, int minor)
   module_entry *p;
 
   for (p = module_list; p && p->name; p = p->next) {
-    if (!egg_strcasecmp(name, p->name)) {
+    if (!strcasecmp(name, p->name)) {
       p->major = major;
       p->minor = minor;
       p->funcs = funcs;
@@ -798,7 +815,7 @@ const char *module_load(char *name)
 #endif /* !STATIC */
 
 #ifdef STATIC
-  for (sl = static_modules; sl && egg_strcasecmp(sl->name, name); sl = sl->next);
+  for (sl = static_modules; sl && strcasecmp(sl->name, name); sl = sl->next);
   if (!sl)
     return "Unknown module.";
   f = (Function) sl->func;
@@ -892,11 +909,12 @@ module_entry *module_find(char *name, int major, int minor)
   module_entry *p;
 
   for (p = module_list; p && p->name; p = p->next) {
-    if ((major == p->major || !major) && minor <= p->minor &&
-        !egg_strcasecmp(name, p->name))
+    if ( (((major < p->major) || !major) || ((major == p->major) &&
+        (minor <= p->minor))) && !strcasecmp(name, p->name) )
       return p;
   }
   return NULL;
+
 }
 
 static int module_rename(char *name, char *newname)
@@ -904,11 +922,11 @@ static int module_rename(char *name, char *newname)
   module_entry *p;
 
   for (p = module_list; p; p = p->next)
-    if (!egg_strcasecmp(newname, p->name))
+    if (!strcasecmp(newname, p->name))
       return 0;
 
   for (p = module_list; p && p->name; p = p->next) {
-    if (!egg_strcasecmp(name, p->name)) {
+    if (!strcasecmp(name, p->name)) {
       nfree(p->name);
       p->name = nmalloc(strlen(newname) + 1);
       strcpy(p->name, newname);
@@ -1030,6 +1048,12 @@ void add_hook(int hook_num, Function func)
     case HOOK_ENCRYPT_PASS:
       encrypt_pass = (void (*)(char *, char *)) func;
       break;
+    case HOOK_ENCRYPT_PASS2:
+      encrypt_pass2 = (char *(*)(char *)) func;
+      break;
+    case HOOK_VERIFY_PASS2:
+      verify_pass2 = (char *(*)(char *, char*)) func;
+      break;
     case HOOK_ENCRYPT_STRING:
       encrypt_string = (char *(*)(char *, char *)) func;
       break;
@@ -1070,11 +1094,11 @@ void add_hook(int hook_num, Function func)
         match_noterej = (int (*)(struct userrec *, char *)) func;
       break;
     case HOOK_DNS_HOSTBYIP:
-      if (dns_hostbyip == block_dns_hostbyip)
+      if (dns_hostbyip == core_dns_hostbyip)
         dns_hostbyip = (void (*)(sockname_t *)) func;
       break;
     case HOOK_DNS_IPBYHOST:
-      if (dns_ipbyhost == block_dns_ipbyhost)
+      if (dns_ipbyhost == core_dns_ipbyhost)
         dns_ipbyhost = (void (*)(char *)) func;
       break;
     }
@@ -1103,6 +1127,14 @@ void del_hook(int hook_num, Function func)
       if (encrypt_pass == (void (*)(char *, char *)) func)
         encrypt_pass = (void (*)(char *, char *)) null_func;
       break;
+    case HOOK_ENCRYPT_PASS2:
+      if (encrypt_pass2 == (char *(*)(char *)) func)
+        encrypt_pass2 = (char *(*)(char *)) null_func;
+      break;
+    case HOOK_VERIFY_PASS2:
+      if (verify_pass2 == (char *(*)(char *, char *)) func)
+        verify_pass2 = (char *(*)(char *, char *)) null_func;
+      break;
     case HOOK_ENCRYPT_STRING:
       if (encrypt_string == (char *(*)(char *, char *)) func)
         encrypt_string = (char *(*)(char *, char *)) null_func;
@@ -1133,11 +1165,11 @@ void del_hook(int hook_num, Function func)
       break;
     case HOOK_DNS_HOSTBYIP:
       if (dns_hostbyip == (void (*)(sockname_t *)) func)
-        dns_hostbyip = block_dns_hostbyip;
+        dns_hostbyip = core_dns_hostbyip;
       break;
     case HOOK_DNS_IPBYHOST:
       if (dns_ipbyhost == (void (*)(char *)) func)
-        dns_ipbyhost = block_dns_ipbyhost;
+        dns_ipbyhost = core_dns_ipbyhost;
       break;
     }
 }
@@ -1149,7 +1181,7 @@ void do_module_report(int idx, int details, char *which)
   if (p && !which)
     dprintf(idx, "Loaded module information:\n");
   for (; p; p = p->next) {
-    if (!which || !egg_strcasecmp(which, p->name)) {
+    if (!which || !strcasecmp(which, p->name)) {
       dependancy *d;
 
       if (details)
