@@ -9,7 +9,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -44,6 +44,8 @@ struct tand_t_struct;
 
 #ifndef MAKING_MODS
 extern void (*encrypt_pass) (char *, char *);
+extern char *(*encrypt_pass2) (char *);
+extern char *(*verify_pass2) (char *, char *);
 extern char *(*encrypt_string) (char *, char *);
 extern char *(*decrypt_string) (char *, char *);
 extern int (*rfc_casecmp) (const char *, const char *);
@@ -68,7 +70,7 @@ void tell_bottree(int, int);
 int botlink(char *, int, char *);
 int botunlink(int, char *, char *, char *);
 void dump_links(int);
-void addbot(char *, char *, char *, char, int);
+void addbot(char *, char *, char *, char, int, int);
 void updatebot(int, char *, char, int);
 void rembot(char *);
 struct tand_t_struct *findbot(char *);
@@ -122,6 +124,7 @@ int check_dcc_chanattrs(struct userrec *, char *, int, int);
 int check_int_range(char *value, int min, int max);
 int stripmodes(char *);
 char *stripmasktype(int);
+char *check_validpass(struct userrec *, char *);
 
 /* dcc.c */
 void failed_link(int);
@@ -165,12 +168,12 @@ void changeover_dcc(int, struct dcc_table *, int);
 
 /* dns.c */
 extern void (*dns_hostbyip) (sockname_t *);
-void block_dns_hostbyip(sockname_t *);
+void core_dns_hostbyip(sockname_t *);
 void call_hostbyip(sockname_t *, char *, int);
 void call_ipbyhost(char *, sockname_t *, int);
 void dcc_dnshostbyip(sockname_t *);
 extern void (*dns_ipbyhost) (char *);
-void block_dns_ipbyhost(char *);
+void core_dns_ipbyhost(char *);
 void dcc_dnsipbyhost(char *);
 
 /* language.c */
@@ -237,6 +240,7 @@ void dumplots(int, const char *, const char *);
 void daysago(time_t, time_t, char *);
 void days(time_t, time_t, char *);
 void daysdur(time_t, time_t, char *);
+char *egg_uname();
 void help_subst(char *, char *, struct flag_record *, int, char *);
 void sub_lang(int, char *);
 void show_motd(int);
@@ -250,6 +254,7 @@ void debug_help(int);
 void reload_help_data(void);
 char *extracthostname(char *);
 void show_banner(int i);
+void make_rand_str_from_chars(char *, int, char *);
 void make_rand_str(char *, int);
 int oatoi(const char *);
 int is_file(const char *);
@@ -259,14 +264,13 @@ char *strchr_unescape(char *, const char, const char);
 void str_unescape(char *, const char);
 int str_isdigit(const char *);
 void kill_bot(char *, char *);
-
 void maskaddr(const char *, char *, int);
 #define maskhost(a,b) maskaddr((a),(b),3)
 #define maskban(a,b)  maskaddr((a),(b),3)
+int crypto_verify(const char *, const char *);
 
 /* net.c */
 IP my_atoul(char *);
-unsigned long iptolong(IP);
 void setsock(int, int);
 int allocsock(int, int);
 int alloctclsock(int, int, Tcl_FileProc *, ClientData);
@@ -336,7 +340,6 @@ void write_userfile(int);
 struct userrec *check_dcclist_hand(char *);
 void touch_laston(struct userrec *, char *, time_t);
 void user_del_chan(char *);
-char *fixfrom(char *);
 int check_conflags(struct flag_record *fr, int md);
 
 /* users.c */
