@@ -3,7 +3,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,12 +28,10 @@
 
 typedef struct tcl_cmd_b {
   struct tcl_cmd_b *next;
-
   struct flag_record flags;
   char *func_name;              /* Proc name. */
-  /* FIXME: 'hits' could overflow if a bind is triggered enough. */
-  int hits;                     /* Number of times this proc was triggered. */
-  u_8bit_t attributes;          /* Flags for this entry. TC_* */
+  uint8_t attributes;           /* Flags for this entry. TC_* */
+  unsigned int hits;            /* Number of times this proc was triggered. */
 } tcl_cmd_t;
 
 struct threaddata {
@@ -49,10 +47,9 @@ struct threaddata {
 
 typedef struct tcl_bind_mask_b {
   struct tcl_bind_mask_b *next;
-
-  tcl_cmd_t *first;             /* List of commands registered for this bind. */
   char *mask;
-  u_8bit_t flags;               /* Flags for this entry. TBM_* */
+  uint8_t flags;                /* Flags for this entry. TBM_* */
+  tcl_cmd_t *first;             /* List of commands registered for this bind. */
 } tcl_bind_mask_t;
 
 
@@ -62,12 +59,11 @@ typedef struct tcl_bind_mask_b {
 
 typedef struct tcl_bind_list_b {
   struct tcl_bind_list_b *next;
-
-  tcl_bind_mask_t *first;       /* Pointer to registered binds for this list. */
-  char name[5];                 /* Name of the bind. */
-  u_8bit_t flags;               /* Flags for this element. HT_* */
+  char name[16];                /* Name of the bind. */
+  uint8_t flags;                /* Flags for this element. HT_* */
   IntFunc func;                 /* Function used as the Tcl calling interface
                                  * for procs actually representing C functions. */
+  tcl_bind_mask_t *first;       /* Pointer to registered binds for this list. */
 } tcl_bind_list_t, *p_tcl_bind_list;
 
 
@@ -95,8 +91,7 @@ void check_tcl_disc(const char *);
 const char *check_tcl_filt(int, const char *);
 int check_tcl_note(const char *, const char *, const char *);
 void check_tcl_listen(const char *, int);
-void check_tcl_time(struct tm *);
-void check_tcl_cron(struct tm *);
+void check_tcl_time_and_cron(struct tm *);
 void tell_binds(int, char *);
 void check_tcl_nkch(const char *, const char *);
 void check_tcl_away(const char *, int, const char *);
