@@ -4,7 +4,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,10 +38,7 @@
 #endif
 
 #include <sys/stat.h>
-
-#ifdef HAVE_UNAME
-#  include <sys/utsname.h>
-#endif
+#include <sys/utsname.h>
 
 extern p_tcl_bind_list bind_table_list;
 extern tcl_timer_t *timer, *utimer;
@@ -252,7 +249,7 @@ static int tcl_binds STDVAR
 static int tcl_timer STDVAR
 {
   unsigned long x;
-  char s[16];
+  char s[26];
 
   BADARGS(3, 4, " minutes command ?count?");
 
@@ -267,7 +264,7 @@ static int tcl_timer STDVAR
   if (argv[2][0] != '#') {
     x = add_timer(&timer, atoi(argv[1]), (argc == 4 ? atoi(argv[3]) : 1),
                   argv[2], 0L);
-    egg_snprintf(s, sizeof s, "timer%lu", x);
+    snprintf(s, sizeof s, "timer%lu", x);
     Tcl_AppendResult(irp, s, NULL);
   }
   return TCL_OK;
@@ -276,7 +273,7 @@ static int tcl_timer STDVAR
 static int tcl_utimer STDVAR
 {
   unsigned long x;
-  char s[16];
+  char s[26];
 
   BADARGS(3, 4, " seconds command ?count?");
 
@@ -291,7 +288,7 @@ static int tcl_utimer STDVAR
   if (argv[2][0] != '#') {
     x = add_timer(&utimer, atoi(argv[1]), (argc == 4 ? atoi(argv[3]) : 1),
                   argv[2], 0L);
-    egg_snprintf(s, sizeof s, "timer%lu", x);
+    snprintf(s, sizeof s, "timer%lu", x);
     Tcl_AppendResult(irp, s, NULL);
   }
   return TCL_OK;
@@ -428,7 +425,7 @@ static int tcl_strftime STDVAR
   else
     t = now;
   tm1 = localtime(&t);
-  if (egg_strftime(buf, sizeof(buf) - 1, argv[1], tm1)) {
+  if (strftime(buf, sizeof(buf) - 1, argv[1], tm1)) {
     Tcl_AppendResult(irp, buf, NULL);
     return TCL_OK;
   }
@@ -573,21 +570,7 @@ static int tcl_unloadmodule STDVAR
 
 static int tcl_unames STDVAR
 {
-  char *unix_n, *vers_n;
-#ifdef HAVE_UNAME
-  struct utsname un;
-
-  if (uname(&un) < 0) {
-#endif
-    unix_n = "*unknown*";
-    vers_n = "";
-#ifdef HAVE_UNAME
-  } else {
-    unix_n = un.sysname;
-    vers_n = un.release;
-  }
-#endif
-  Tcl_AppendResult(irp, unix_n, " ", vers_n, NULL);
+  Tcl_AppendResult(irp, egg_uname(), NULL);
   return TCL_OK;
 }
 
