@@ -4,7 +4,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,7 +32,8 @@ extern Tcl_Interp *interp;
 extern char whois_fields[];
 
 
-int share_greet = 0;            /* Share greeting info                  */
+int share_greet = 0; /* Share greeting info                      */
+int remove_pass = 0; /* create and keep encryption mod passwords */
 struct user_entry_type *entry_type_list;
 
 
@@ -45,6 +46,7 @@ void init_userent()
   add_entry_type(&USERENTRY_LASTON);
   add_entry_type(&USERENTRY_BOTADDR);
   add_entry_type(&USERENTRY_PASS);
+  add_entry_type(&USERENTRY_PASS2);
   add_entry_type(&USERENTRY_HOSTS);
   add_entry_type(&USERENTRY_BOTFL);
 #ifdef TLS
@@ -130,7 +132,7 @@ int def_set(struct userrec *u, struct user_entry *e, void *buf)
 
     e->u.string = user_realloc(e->u.string, l + 1);
 
-    strncpyz(e->u.string, string, l + 1);
+    strlcpy(e->u.string, string, l + 1);
 
     for (i = e->u.string; *i; i++)
       /* Allow bold, inverse, underline, color text here...
@@ -237,37 +239,140 @@ struct user_entry_type USERENTRY_INFO = {
   def_tcl_append
 };
 
+int pass2_set(struct userrec *u, struct user_entry *e, void *new)
+{
+  if (e->u.extra) {
+    explicit_bzero(e->u.extra, strlen(e->u.extra));
+    nfree(e->u.extra);
+  }
+  if (new) { /* set PASS2 */
+    e->u.extra = user_malloc(strlen(new) + 1);
+    strcpy(e->u.extra, new);
+  }
+  else /* remove PASS2 */
+    e->u.extra = NULL;
+  return 0;
+}
+
+static int def_tcl_null(Tcl_Interp * irp, struct userrec *u,
+                        struct user_entry *e, int argc, char **argv)
+{
+  Tcl_AppendResult(irp, "Please use PASS instead.", NULL);
+  return TCL_ERROR;
+}
+
+struct user_entry_type USERENTRY_PASS2 = {
+  0,
+  0,
+  0,
+  def_unpack,
+  def_pack,
+  def_write_userfile,
+  def_kill,
+  def_get,
+  pass2_set,
+  def_tcl_null,
+  def_tcl_null,
+  def_expmem,
+  0,
+  "PASS2",
+  def_tcl_append
+};
+
 int pass_set(struct userrec *u, struct user_entry *e, void *buf)
 {
-  char new[32];
-  register char *pass = buf;
+  char *pass = buf;
+  unsigned char *p;
+  char new[PASSWORDLEN];
+  char *new2 = 0;
 
-  if (e->u.extra)
+  /* encrypt_pass means encryption module is loaded
+   * encrypt_pass2 means encryption2 module is loaded
+   */
+  if (encrypt_pass && e->u.extra) {
+    explicit_bzero(e->u.extra, strlen(e->u.extra));
     nfree(e->u.extra);
-  if (!pass || !pass[0] || (pass[0] == '-'))
-    e->u.extra = NULL;
+  }
+  if (!pass || !pass[0] || (pass[0] == '-')) {
+    /* empty string or '-' means remove passwords */
+    if (encrypt_pass) /* remove PASS */
+      e->u.extra = NULL;
+    if (encrypt_pass2) /* remove PASS2 */
+      set_user(&USERENTRY_PASS2, u, NULL);
+  }
   else {
-    unsigned char *p = (unsigned char *) pass;
-
-    if (strlen(pass) > 30)
-      pass[30] = 0;
+    /* sanitize password */
+    if (strlen(pass) > PASSWORDMAX)
+      pass[PASSWORDMAX] = 0;
+    p = (unsigned char *) pass;
     while (*p) {
       if ((*p <= 32) || (*p == 127))
         *p = '?';
       p++;
     }
-    if ((u->flags & USER_BOT) || (pass[0] == '+'))
-      strcpy(new, pass);
-    else
-      encrypt_pass(pass, new);
-    e->u.extra = user_malloc(strlen(new) + 1);
-    strcpy(e->u.extra, new);
+    /* load new with new PASS
+     * load new2 with new PASS2
+     */
+    if (u->flags & USER_BOT) {
+      /* set PASS and PASS2 cleartext password */
+      strlcpy(new, pass, sizeof new);
+      if (encrypt_pass2)
+        new2 = new;
+    }
+    else if (pass[0] == '+') {
+      /* '+' means pass is already encrypted, set PASS = pass */
+      strlcpy(new, pass, sizeof new);
+      /* due to module api encrypted pass2 cannot be available here
+       * caller must do set_user(&USERENTRY_PASS2, u, password);
+       * probably only share.c:dup_userlist()
+       */
+    }
+    else {
+      /* encrypt password into new and/or new2 depending on the encryption
+       * modules loaded and the value of remove-pass
+       */
+      if (encrypt_pass && (!encrypt_pass2 || !remove_pass))
+        encrypt_pass(pass, new);
+      if (encrypt_pass2)
+        new2 = encrypt_pass2(pass);
+    }
+    /* set PASS to new and PASS2 to new2 depending on the encryption modules
+     * loaded and the value of remove-pass
+     */
+    if (encrypt_pass && (!encrypt_pass2 || !remove_pass)) {
+      /* set PASS */
+      e->u.extra = user_malloc(strlen(new) + 1);
+      strcpy(e->u.extra, new);
+    }
+    if (new2) { /* implicit encrypt_pass2 && */
+      /* set PASS2 */
+      set_user(&USERENTRY_PASS2, u, new2);
+      if (encrypt_pass && remove_pass && e->u.extra)
+        e->u.extra = NULL; /* remove PASS, e->u.extra already freed */
+    }
+    explicit_bzero(new, sizeof new);
+    if (new2 && new2 != new)
+      explicit_bzero(new2, strlen(new2));
   }
   if (!noshare && !(u->flags & (USER_BOT | USER_UNSHARED)))
     shareout(NULL, "c PASS %s %s\n", u->handle, pass ? pass : "");
   return 1;
 }
 
+static int pass_tcl_get(Tcl_Interp * interp, struct userrec *u,
+                        struct user_entry *e, int argc, char **argv)
+{
+  char *pass = 0;
+
+  if (encrypt_pass2)
+    pass = get_user(&USERENTRY_PASS2, u);
+  if (!pass)
+    pass = e->u.string;
+  Tcl_AppendResult(interp, pass, NULL);
+
+  return TCL_OK;
+}
+
 static int pass_tcl_set(Tcl_Interp * irp, struct userrec *u,
                         struct user_entry *e, int argc, char **argv)
 {
@@ -287,7 +392,7 @@ struct user_entry_type USERENTRY_PASS = {
   def_kill,
   def_get,
   pass_set,
-  def_tcl_get,
+  pass_tcl_get,
   pass_tcl_set,
   def_expmem,
   0,
@@ -364,7 +469,7 @@ static int laston_set(struct userrec *u, struct user_entry *e, void *buf)
       nfree(li);
     }
 
-    li = e->u.extra = buf;
+    e->u.extra = buf;
   }
   /* donut share laston info */
   return 1;
@@ -374,8 +479,7 @@ static int laston_tcl_get(Tcl_Interp * irp, struct userrec *u,
                           struct user_entry *e, int argc, char **argv)
 {
   struct laston_info *li = (struct laston_info *) e->u.extra;
-  char number[20];
-  long tv;
+  char number[22];
   struct chanuserrec *cr;
 
   BADARGS(3, 4, " handle LASTON ?channel?");
@@ -390,8 +494,7 @@ static int laston_tcl_get(Tcl_Interp * irp, struct userrec *u,
     if (!cr)
       Tcl_AppendResult(irp, "0", NULL);
   } else {
-    tv = li->laston;
-    sprintf(number, "%lu ", tv);
+    snprintf(number, sizeof number, "%" PRId64 " ", (int64_t) li->laston);
     Tcl_AppendResult(irp, number, li->lastonplace, NULL);
   }
   return TCL_OK;
@@ -570,7 +673,7 @@ static int botaddr_write_userfile(FILE *f, struct userrec *u,
 {
   int ret = 1;
   char *p, *q, *addr;
-  register struct bot_addr *bi = (struct bot_addr *) e->u.extra;
+  struct bot_addr *bi = (struct bot_addr *) e->u.extra;
 
   p = bi->address;
   addr = user_malloc(strlen(bi->address) + 1);
@@ -595,7 +698,7 @@ static int botaddr_write_userfile(FILE *f, struct userrec *u,
 
 static int botaddr_set(struct userrec *u, struct user_entry *e, void *buf)
 {
-  register struct bot_addr *bi = (struct bot_addr *) e->u.extra;
+  struct bot_addr *bi = (struct bot_addr *) e->u.extra;
 
   if (!bi && !buf)
     return 1;
@@ -667,7 +770,7 @@ static int botaddr_tcl_append(Tcl_Interp * interp, struct userrec *u,
 static int botaddr_tcl_set(Tcl_Interp * irp, struct userrec *u,
                            struct user_entry *e, int argc, char **argv)
 {
-  register struct bot_addr *bi = (struct bot_addr *) e->u.extra;
+  struct bot_addr *bi = (struct bot_addr *) e->u.extra;
 
   BADARGS(4, 6, " handle type address ?telnetport ?relayport??");
 
@@ -721,14 +824,14 @@ static int botaddr_tcl_set(Tcl_Interp * irp, struct userrec *u,
 
 static int botaddr_expmem(struct user_entry *e)
 {
-  register struct bot_addr *bi = (struct bot_addr *) e->u.extra;
+  struct bot_addr *bi = (struct bot_addr *) e->u.extra;
 
   return strlen(bi->address) + 1 + sizeof(struct bot_addr);
 }
 
 static void botaddr_display(int idx, struct user_entry *e)
 {
-  register struct bot_addr *bi = (struct bot_addr *) e->u.extra;
+  struct bot_addr *bi = (struct bot_addr *) e->u.extra;
 
   dprintf(idx, "  ADDRESS: %.70s\n", bi->address);
 #ifdef TLS
@@ -812,13 +915,13 @@ int xtra_set(struct userrec *u, struct user_entry *e, void *buf)
   struct xtra_key *curr, *old = NULL, *new = buf;
 
   for (curr = e->u.extra; curr; curr = curr->next) {
-    if (curr->key && !egg_strcasecmp(curr->key, new->key)) {
+    if (curr->key && !strcasecmp(curr->key, new->key)) {
       old = curr;
       break;
     }
   }
   if (!old && (!new->data || !new->data[0])) {
-    /* Delete non-existant entry -- doh ++rtc */
+    /* Delete non-existent entry -- doh ++rtc */
     nfree(new->key);
     if (new->data)
       nfree(new->data);
@@ -869,7 +972,7 @@ static int xtra_tcl_set(Tcl_Interp * irp, struct userrec *u,
   if (l > 500)
     l = 500;
   xk->key = user_malloc(l + 1);
-  strncpyz(xk->key, argv[3], l + 1);
+  strlcpy(xk->key, argv[3], l + 1);
 
   if (argc == 5) {
     int k = strlen(argv[4]);
@@ -877,7 +980,7 @@ static int xtra_tcl_set(Tcl_Interp * irp, struct userrec *u,
     if (k > 500 - l)
       k = 500 - l;
     xk->data = user_malloc(k + 1);
-    strncpyz(xk->data, argv[4], k + 1);
+    strlcpy(xk->data, argv[4], k + 1);
   }
   xtra_set(u, e, xk);
   return TCL_OK;
@@ -943,7 +1046,7 @@ static void xtra_display(int idx, struct user_entry *e)
   for (xk = e->u.extra; xk; xk = xk->next) {
     /* Ok, it's a valid xtra field entry */
     for (j = 0; j < lc; j++) {
-      if (!egg_strcasecmp(list[j], xk->key))
+      if (!strcasecmp(list[j], xk->key))
         dprintf(idx, "  %s: %s\n", xk->key, xk->data);
     }
   }
@@ -967,7 +1070,7 @@ static int xtra_gotshare(struct userrec *u, struct user_entry *e,
   if (l > 500)
     l = 500;
   xk->key = user_malloc(l + 1);
-  strncpyz(xk->key, arg, l + 1);
+  strlcpy(xk->key, arg, l + 1);
 
   if (buf[0]) {
     int k = strlen(buf);
@@ -975,7 +1078,7 @@ static int xtra_gotshare(struct userrec *u, struct user_entry *e,
     if (k > 500 - l)
       k = 500 - l;
     xk->data = user_malloc(k + 1);
-    strncpyz(xk->data, buf, k + 1);
+    strlcpy(xk->data, buf, k + 1);
   }
   xtra_set(u, e, xk);
   return 1;
@@ -1054,7 +1157,7 @@ static int xtra_tcl_get(Tcl_Interp * irp, struct userrec *u,
     return TCL_OK;
   }
   for (x = e->u.extra; x; x = x->next)
-    if (!egg_strcasecmp(argv[3], x->key)) {
+    if (!strcasecmp(argv[3], x->key)) {
       Tcl_AppendResult(irp, x->data, NULL);
       return TCL_OK;
     }
@@ -1170,7 +1273,7 @@ static void hosts_display(int idx, struct user_entry *e)
 
 static int hosts_set(struct userrec *u, struct user_entry *e, void *buf)
 {
-  if (!buf || !egg_strcasecmp(buf, "none")) {
+  if (!buf || !strcasecmp(buf, "none")) {
     /* When the bot crashes, it's in this part, not in the 'else' part */
     list_type_kill(e->u.list);
     e->u.list = NULL;
@@ -1317,7 +1420,7 @@ static int fprint_tcl_set(Tcl_Interp * irp, struct userrec *u,
 struct user_entry_type USERENTRY_FPRINT = {
   0,
   def_gotshare,
-  0,
+  def_dupuser,
   fprint_unpack,
   def_pack,
   def_write_userfile,
@@ -1401,7 +1504,7 @@ struct user_entry_type *find_entry_type(char *name)
   struct user_entry_type *p;
 
   for (p = entry_type_list; p; p = p->next) {
-    if (!egg_strcasecmp(name, p->name))
+    if (!strcasecmp(name, p->name))
       return p;
   }
   return NULL;
@@ -1414,7 +1517,7 @@ struct user_entry *find_user_entry(struct user_entry_type *et,
 
   for (e = &(u->entries); *e; e = &((*e)->next)) {
     if (((*e)->type == et) ||
-        ((*e)->name && !egg_strcasecmp((*e)->name, et->name))) {
+        ((*e)->name && !strcasecmp((*e)->name, et->name))) {
       t = *e;
       *e = t->next;
       t->next = u->entries;
