@@ -6,7 +6,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,19 +32,20 @@
 
 extern struct dcc_t *dcc;
 extern struct chanset_t *chanset;
-extern int default_flags, default_uflags, quiet_save, dcc_total, share_greet;
-extern char userfile[], ver[], botnetnick[];
+extern int default_flags, default_uflags, quiet_save, dcc_total, share_greet,
+           remove_pass;
+extern char ver[], botnetnick[];
 extern time_t now;
 
-int noshare = 1;                   /* don't send out to sharebots   */
-struct userrec *userlist = NULL;   /* user records are stored here  */
-struct userrec *lastuser = NULL;   /* last accessed user record     */
+int noshare = 1;                   /* don't send out to sharebots       */
+struct userrec *userlist = NULL;   /* user records are stored here      */
+struct userrec *lastuser = NULL;   /* last accessed user record         */
 maskrec *global_bans = NULL, *global_exempts = NULL, *global_invites = NULL;
 struct igrec *global_ign = NULL;
-int cache_hit = 0, cache_miss = 0; /* temporary cache accounting    */
-int strict_host = 1;
-int userfile_perm = 0600;         /* Userfile permissions
-                                   * (default rw-------) */
+int cache_hit = 0, cache_miss = 0; /* temporary cache accounting        */
+int userfile_perm = 0600;          /* Userfile permissions
+                                    * (default rw-------)               */
+char userfile[121];                /* where the user records are stored */
 
 void *_user_malloc(int size, const char *file, int line)
 {
@@ -160,36 +161,12 @@ int count_users(struct userrec *bu)
   return tot;
 }
 
-/* Removes a username prefix (~+-^=) from a userhost.
- * e.g, "nick!~user@host" -> "nick!user@host"
- */
-char *fixfrom(char *s)
-{
-  static char uhost[UHOSTLEN];
-  char *p = uhost;
-
-  if (!s || !*s || strict_host)
-    return s;
-
-  while (*s) {
-    *p++ = *s;
-    if (*s == '!' && strchr("~+-^=", s[1]) && s[2] != '@') {
-      strcpy(p, s + 2);
-      return uhost;
-    }
-    s++;
-  }
-
-  *p = 0;
-  return uhost;
-}
-
 struct userrec *check_dcclist_hand(char *handle)
 {
   int i;
 
   for (i = 0; i < dcc_total; i++)
-    if (!egg_strcasecmp(dcc[i].nick, handle))
+    if (!strcasecmp(dcc[i].nick, handle))
       return dcc[i].user;
   return NULL;
 }
@@ -205,7 +182,7 @@ struct userrec *get_user_by_handle(struct userrec *bu, char *handle)
   if (!handle[0] || (handle[0] == '*'))
     return NULL;
   if (bu == userlist) {
-    if (lastuser && !egg_strcasecmp(lastuser->handle, handle)) {
+    if (lastuser && !strcasecmp(lastuser->handle, handle)) {
       cache_hit++;
       return lastuser;
     }
@@ -222,7 +199,7 @@ struct userrec *get_user_by_handle(struct userrec *bu, char *handle)
     cache_miss++;
   }
   for (u = bu; u; u = u->next)
-    if (!egg_strcasecmp(u->handle, handle)) {
+    if (!strcasecmp(u->handle, handle)) {
       if (bu == userlist)
         lastuser = u;
       return u;
@@ -322,7 +299,6 @@ struct userrec *get_user_by_host(char *host)
   }
   cache_miss++;
   strlcpy(host2, host, sizeof host2);
-  host = fixfrom(host);
   for (u = userlist; u; u = u->next) {
     q = get_user(&USERENTRY_HOSTS, u);
     for (; q; q = q->next) {
@@ -343,35 +319,71 @@ struct userrec *get_user_by_host(char *host)
 /* Description: checks the password given against the user's password.
  * Check against the password "-" to find out if a user has no password set.
  *
+ * If encryption2 module is loaded and PASS2 is set PASS2 is compared; else
+ * PASS.
+ *
  * Returns: 1 if the password matches for that user; 0 otherwise. Or if we are
  * checking against the password "-": 1 if the user has no password set; 0
  * otherwise.
  */
 int u_pass_match(struct userrec *u, char *pass)
 {
-  char *cmp, new[32];
+  char *cmp = 0, *new, new2[32];
+  int pass2 = 1;
+  struct user_entry *e;
 
   if (!u || !pass)
     return 0;
-  cmp = get_user(&USERENTRY_PASS, u);
+  if (encrypt_pass2)
+    cmp = get_user(&USERENTRY_PASS2, u);
+  if (!cmp) { /* implicit && encrypt_pass, due to eggdrop has at least one
+                 encryption module loaded */
+    cmp = get_user(&USERENTRY_PASS, u);
+    pass2 = 0;
+  }
   if (pass[0] == '-') {
     if (!cmp)
       return 1;
-    else
-      return 0;
+    return 0;
   }
   /* If password is not set in userrecord, or password is not sent */
   if (!cmp || !pass[0])
     return 0;
   if (u->flags & USER_BOT) {
-    if (!strcmp(cmp, pass))
+    if (!crypto_verify(cmp, pass)) /* verify successful */
       return 1;
-  } else {
-    if (strlen(pass) > 30)
-      pass[30] = 0;
-    encrypt_pass(pass, new);
-    if (!strcmp(cmp, new))
+    return 0;
+  }
+  if (strlen(pass) > PASSWORDMAX)
+    pass[PASSWORDMAX] = 0;
+  if (pass2) {
+    new = verify_pass2(pass, cmp);
+    if (new) { /* verify successful */
+      if (new != cmp) /* reenrypted with new parameters,
+                         no need to strcmp() */
+        set_user(&USERENTRY_PASS2, u, new);
       return 1;
+    }
+  }
+  else if (encrypt_pass) {
+    encrypt_pass(pass, new2);
+    if (!crypto_verify(cmp, new2)) { /* verify successful */
+      if (encrypt_pass2) {
+        new = encrypt_pass2(pass);
+        if (new) {
+          set_user(&USERENTRY_PASS2, u, new);
+          if (remove_pass) { /* implicit e->u.extra != NULL */
+            e = find_user_entry(&USERENTRY_PASS, u);
+            explicit_bzero(e->u.extra, strlen(e->u.extra));
+            nfree(e->u.extra);
+            e->u.extra = NULL;
+            egg_list_delete((struct list_type **) &(u->entries), (struct list_type *) e);
+            nfree(e);
+          }
+        }
+      }
+      return 1;
+    }
   }
   return 0;
 }
@@ -398,8 +410,8 @@ int write_user(struct userrec *u, FILE *f, int idx)
         fr.match = (FR_CHAN | FR_BOT);
         get_user_flagrec(dcc[idx].user, &fr, ch->channel);
       } else
-        fr.chan = BOT_SHARE;
-      if ((fr.chan & BOT_SHARE) || (fr.bot & BOT_GLOBAL)) {
+        fr.chan = BOT_AGGRESSIVE;
+      if ((fr.chan & BOT_AGGRESSIVE) || (fr.bot & BOT_GLOBAL)) {
         fr.match = FR_CHAN;
         fr.chan = ch->flags;
         fr.udef_chan = ch->flags_udef;
@@ -451,7 +463,7 @@ int write_ignores(FILE *f, int idx)
   return 1;
 }
 
-int sort_compare(struct userrec *a, struct userrec *b)
+static int sort_compare(struct userrec *a, struct userrec *b)
 {
   /* Order by flags, then alphabetically
    * first bots: +h / +a / +l / other bots
@@ -493,10 +505,10 @@ int sort_compare(struct userrec *a, struct userrec *b)
     if (a->flags & ~b->flags & USER_HALFOP)
       return 0;
   }
-  return (egg_strcasecmp(a->handle, b->handle) > 0);
+  return (strcasecmp(a->handle, b->handle) > 0);
 }
 
-void sort_userlist()
+static void sort_userlist()
 {
   int again;
   struct userrec *last, *p, *c, *n;
@@ -532,7 +544,7 @@ void sort_userlist()
 void write_userfile(int idx)
 {
   FILE *f;
-  char *new_userfile;
+  char new_userfile[(sizeof userfile) + 4]; /* 4 = strlen("~new") */
   char s1[81];
   time_t tt;
   struct userrec *u;
@@ -541,14 +553,12 @@ void write_userfile(int idx)
   if (userlist == NULL)
     return;                     /* No point in saving userfile */
 
-  new_userfile = nmalloc(strlen(userfile) + 5);
-  sprintf(new_userfile, "%s~new", userfile);
+  egg_snprintf(new_userfile, sizeof new_userfile, "%s~new", userfile);
 
   f = fopen(new_userfile, "w");
   chmod(new_userfile, userfile_perm);
   if (f == NULL) {
     putlog(LOG_MISC, "*", USERF_ERRWRITE);
-    nfree(new_userfile);
     return;
   }
   if (!quiet_save)
@@ -561,18 +571,26 @@ void write_userfile(int idx)
   ok = 1;
   /* Add all users except the -tn user */
   for (u = userlist; u && ok; u = u->next)
-    if (egg_strcasecmp(u->handle, EGG_BG_HANDLE) && !write_user(u, f, idx))
+    if (strcasecmp(u->handle, EGG_BG_HANDLE) && !write_user(u, f, idx))
       ok = 0;
   if (!ok || !write_ignores(f, -1) || fflush(f)) {
     putlog(LOG_MISC, "*", "%s (%s)", USERF_ERRWRITE, strerror(ferror(f)));
     fclose(f);
-    nfree(new_userfile);
     return;
   }
   fclose(f);
   call_hook(HOOK_USERFILE);
   movefile(new_userfile, userfile);
-  nfree(new_userfile);
+}
+
+void backup_userfile(void)
+{
+  char s[(sizeof userfile) + 4]; /* 4 = strlen("~bak") */
+
+  if (quiet_save < 2)
+    putlog(LOG_MISC, "*", USERF_BACKUP);
+  egg_snprintf(s, sizeof s, "%s~bak", userfile);
+  copyfile(userfile, s);
 }
 
 int change_handle(struct userrec *u, char *newh)
@@ -583,7 +601,7 @@ int change_handle(struct userrec *u, char *newh)
   if (!u)
     return 0;
   /* Don't allow the -tn handle to be changed */
-  if (!egg_strcasecmp(u->handle, EGG_BG_HANDLE))
+  if (!strcasecmp(u->handle, EGG_BG_HANDLE))
     return 0;
   /* Nothing that will confuse the userfile */
   if (!newh[1] && strchr(BADHANDCHARS, newh[0]))
@@ -596,7 +614,7 @@ int change_handle(struct userrec *u, char *newh)
   strlcpy(u->handle, newh, sizeof u->handle);
   for (i = 0; i < dcc_total; i++)
     if ((dcc[i].type == &DCC_CHAT || dcc[i].type == &DCC_CHAT_PASS) &&
-        !egg_strcasecmp(dcc[i].nick, s)) {
+        !strcasecmp(dcc[i].nick, s)) {
       strlcpy(dcc[i].nick, newh, sizeof dcc[i].nick);
       if (dcc[i].type == &DCC_CHAT && dcc[i].u.chat->channel >= 0) {
         chanout_but(-1, dcc[i].u.chat->channel,
@@ -649,13 +667,6 @@ struct userrec *adduser(struct userrec *bu, char *handle, char *host,
   if (host && host[0]) {
     char *p;
 
-    /* About this fixfrom():
-     *   We should use this fixfrom before every call of adduser()
-     *   but its much easier to use here...  (drummer)
-     *   Only use it if we have a host :) (dw)
-     */
-    host = fixfrom(host);
-
     p = strchr(host, ',');
     while (p != NULL) {
       *p = '?';
@@ -736,7 +747,7 @@ int deluser(char *handle)
   int fnd = 0;
 
   while ((u != NULL) && (!fnd)) {
-    if (!egg_strcasecmp(u->handle, handle))
+    if (!strcasecmp(u->handle, handle))
       fnd = 1;
     else {
       prev = u;
@@ -866,7 +877,7 @@ struct userrec *get_user_by_nick(char *nick)
 
         egg_snprintf(word, sizeof word, "%s!%s", m->nick, m->userhost);
         /* No need to check the return value ourself */
-        return get_user_by_host(word);;
+        return get_user_by_host(word);
       }
     }
   }
